// Cthulhu.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "Cthulhu.h"
#include "ChildFrm.h"
#include "MainFrm.h"
#include "CthulhuDoc.h"
#include "CthulhuView.h"
#include "TelegramDoc.h"
#include "TelegramView.h"
#include "NewDocument.h"
#include "Options.h"
#include "Skill.h"
#include "Splash.h"
#include "CopyrightDlg.h"
#include "ByakheeProjectDlg.h"
#include "Hyperlink.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Global Functions
CString GetDefaultDirectory()
{
    //get application's name
    char szBuffer[MAX_PATH+1];
    GetModuleFileName( AfxGetInstanceHandle(), szBuffer, MAX_PATH );

    //find the last path separator
    char* pszEndOfPath = strrchr( szBuffer,'\\' );
    if( pszEndOfPath )
    {
        //terminate the string just after it
        pszEndOfPath++;
        *pszEndOfPath = '\0';
        return CString( szBuffer );
    }
    else
        //there was no path!
        return "";
}


int GetValueInRange( int nMin, int nMax, int nMultiplier )
{
    int nTotal = 0;
    for( int i = 0; i < nMultiplier; i++ )
        nTotal += (rand() % ((nMax+1)-nMin)) + nMin;

    return nTotal;
}

/////////////////////////////////////////////////////////////////////////////
// CCthulhuApp

BEGIN_MESSAGE_MAP(CCthulhuApp, CWinApp)
	ON_COMMAND(CG_IDS_TIPOFTHEDAY, ShowTipOfTheDay)
	//{{AFX_MSG_MAP(CCthulhuApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(ID_TOOLS_OPTIONS, OnToolsOptions)
	ON_COMMAND(ID_HELP_LEGALTRADEMARKS, OnHelpLegalTrademarks)
	ON_COMMAND(ID_HELP_THEBYAKHEEPROJECT, OnHelpTheByakheeProject)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCthulhuApp construction

CCthulhuApp::CCthulhuApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CCthulhuApp object

CCthulhuApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CCthulhuApp initialization

BOOL CCthulhuApp::InitInstance()
{
	CSplashWnd::EnableSplashScreen(TRUE);

	m_nCmdShow = SW_SHOWMAXIMIZED;

	SetRegistryKey(_T("DHSoft"));

	// CG: The following line was added by the Splash Screen component.
	CSplashWnd::ShowSplashScreen();

    //load the data tables (after setting the registry key this time... doh)
	if( !LoadDataTables() ) 
    {
        PostQuitMessage(0);
        return FALSE;
    }


    CSplashWnd::DisplayStatus( "Initialising..." );
	srand( GetTickCount() );

	LoadStdProfileSettings(8);  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_SHEETTYPE,
		RUNTIME_CLASS(CCthulhuDoc),
		RUNTIME_CLASS(CChildFrame),
		RUNTIME_CLASS(CCthulhuView));
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_HANDOUTTYPE,
		RUNTIME_CLASS(CTelegramDoc),
		RUNTIME_CLASS(CChildFrame),
		RUNTIME_CLASS(CTelegramView));
	AddDocTemplate(pDocTemplate);

	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

    // Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
    BOOL fCreateNew = FALSE;
    if( cmdInfo.m_nShellCommand == CCommandLineInfo::FileNew ) 
    {
        cmdInfo.m_nShellCommand = CCommandLineInfo::FileNothing;
        fCreateNew = TRUE;
    }
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// The one and only window has been initialized, so show and update it.
    CSplashWnd::DisplayStatus( "Almost Done..." );
	m_pMainWnd->ShowWindow(SW_SHOWMAXIMIZED);
	m_pMainWnd->UpdateWindow();

    //initialise hyperlinks
    CHyperLink::SetLinkCursor( AfxGetApp()->LoadCursor( IDC_HAND ) );


	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// CG: This line inserted by 'Tip of the Day' component.
	ShowTipAtStartup();

    //create a new document now
    if( fCreateNew ) OnFileNew();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();
    ~CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CHyperLink	m_wndMailTips;
	CHyperLink	m_wndWebSplash;
	CHyperLink	m_wndWebAuthor;
	CHyperLink	m_wndMailSplash;
	CHyperLink	m_wndMailAuthor;
	CString	    m_strRuleFileCredits;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_strRuleFileCredits = _T("");
	//}}AFX_DATA_INIT

}

CAboutDlg::~CAboutDlg()
{
}


BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    //load data file string
    CString strDatafile = AfxGetApp()->GetProfileString( "Data", "Rules", "coc5.rul" );
    if( strDatafile.GetLength() == 0 ) strDatafile = "coc5.rul";

    //display the rule file credit, if any
    CString* pstrCredit = GetOptions( "CREDIT", strDatafile );
    if( pstrCredit )
    {
        int i = 0;
        while( pstrCredit[i].GetLength() > 0 )
        {
            m_strRuleFileCredits += pstrCredit[i];
            m_strRuleFileCredits += "\r\n";
            i++;
        }

        delete[] pstrCredit;
    }
    else
        m_strRuleFileCredits = "No credits for the current rule file";

    //set the URLs for each hyperlink
	m_wndWebAuthor.SetURL( "http://www.geocities.com/SiliconValley/Haven/4173/byakhee.html" );
	m_wndMailAuthor.SetURL( "mailto:dharvey@btinternet.com" );

	m_wndWebSplash.SetURL( "http://www.atacomputer.com.tr/oir/gallery" );
	m_wndMailSplash.SetURL( "mailto:yanasikt@superonline.com" );

    m_wndMailTips.SetURL( "mailto:christe@sti.com.br" );

    UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAboutDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

    //draw Byakhee text
    HBITMAP hBm = (HBITMAP)::LoadImage( AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ABOUTTEXT), IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS );
    BITMAP bm;
    ::GetObject( hBm, sizeof(BITMAP), &bm );
    CDC dcOff;
    dcOff.CreateCompatibleDC( &dcOff );
    CBitmap* pOld = dcOff.SelectObject( CBitmap::FromHandle(hBm) );
    dc.BitBlt( 60, 20, bm.bmWidth, bm.bmHeight, &dcOff, 0, 0, SRCCOPY );
    dcOff.SelectObject( pOld );
    ::DeleteObject( hBm );

	// Do not call CDialog::OnPaint() for painting messages
}


void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_TIPSMAIL, m_wndMailTips);
	DDX_Control(pDX, IDC_WEBSPLASH, m_wndWebSplash);
	DDX_Control(pDX, IDC_WEBPAGEAUTHOR, m_wndWebAuthor);
	DDX_Control(pDX, IDC_MAILSPLASH, m_wndMailSplash);
	DDX_Control(pDX, IDC_MAILAUTHOR, m_wndMailAuthor);
	DDX_Text(pDX, IDC_RULEFILE, m_strRuleFileCredits);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CCthulhuApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

BOOL CCthulhuApp::PreTranslateMessage(MSG* pMsg)
{
	// CG: The following lines were added by the Splash Screen component.
	if (CSplashWnd::PreTranslateAppMessage(pMsg))
		return TRUE;

	return CWinApp::PreTranslateMessage(pMsg);
}

void CCthulhuApp::ShowTipAtStartup(void)
{
	// CG: This function added by 'Tip of the Day' component.

	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
	if (cmdInfo.m_bShowSplash)
	{
		CTipDlg dlg;
		if (dlg.m_bStartup)
			dlg.DoModal();
	}

}

void CCthulhuApp::ShowTipOfTheDay(void)
{
	// CG: This function added by 'Tip of the Day' component.

	CTipDlg dlg;
	dlg.DoModal();

}


int CCthulhuApp::ExitInstance() 
{
	return CWinApp::ExitInstance();
}

void CCthulhuApp::OnFileNew() 
{
    CNewDocument NewDocument;
    if( NewDocument.DoModal() == IDOK )
    {
        CDocument* pDoc = NULL;
        switch( NewDocument.m_NewDocument )
        {
            case CNewDocument::NOTHING:
                TRACE0( "NOTHING Type from New Document Dialog!\n" );
                break;

            case CNewDocument::BLANK_1890:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1890;
                pDoc->OnNewDocument();
                ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::BLANK_1920:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1920;
                pDoc->OnNewDocument();
                ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::BLANK_1990:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1990;
                pDoc->OnNewDocument();
                ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::SMALLWIZ_1890:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1890;
                pDoc->OnNewDocument();
                if( !((CCthulhuDoc*)pDoc)->ShowMiniWizard() )
                    delete pDoc;
                else 
                    ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::SMALLWIZ_1920:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1920;
                pDoc->OnNewDocument();
                if( !((CCthulhuDoc*)pDoc)->ShowMiniWizard() )
                    delete pDoc;
                else 
                    ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::SMALLWIZ_1990:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1990;
                pDoc->OnNewDocument();
                if( !((CCthulhuDoc*)pDoc)->ShowMiniWizard() )
                    delete pDoc;
                else 
                    ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::WIZARD_1890:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1890;
                pDoc->OnNewDocument();
                if( !((CCthulhuDoc*)pDoc)->ShowFullWizard() )
                    delete pDoc;
                else 
                    ShowDocument( pDoc, "Investigator" );
               break;

            case CNewDocument::WIZARD_1920:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1920;
                pDoc->OnNewDocument();
                if( !((CCthulhuDoc*)pDoc)->ShowFullWizard() )
                    delete pDoc;
                else 
                    ShowDocument( pDoc, "Investigator" );
                break;

            case CNewDocument::WIZARD_1990:
                if( !(pDoc = CreateDocument( "investigator" )) ) return;
                ((CCthulhuDoc*)pDoc)->m_Investigator.m_era = ERA_1990;
                pDoc->OnNewDocument();
                if( !((CCthulhuDoc*)pDoc)->ShowFullWizard() )
                    delete pDoc;
                else 
                    ShowDocument( pDoc, "Investigator" );
                break;

            //the handout type currenly only supports telegrams.
            //eventually it will have it's own scriping language
            //for creating handouts, and m_strHandoutName will
            //be the name of the script to display it.
            case CNewDocument::HANDOUT:
                if( !(pDoc = CreateDocument( "handout" )) ) return;
                ((CTelegramDoc*)pDoc)->m_strHandoutName = NewDocument.m_strHandoutName;
                pDoc->OnNewDocument();
                ShowDocument( pDoc, "Handout", NewDocument.m_strHandoutName );
                break;

            /*the future:
            case CNewDocument::MONSTER:
            case CNewDocument::NEWSPAPERCUTTING:
            case CNewDocument::CULTISTWIZARD: //?
            case your_idea_here: //e-mail: dharvey@btinternet.com with details!
            */

            default:
                TRACE0( "Unexpected type from New Document Dialog!\n" );
                break;
        }
    }
}

CDocument* CCthulhuApp::CreateDocument( CString strName )
{
	POSITION p;
	p = GetFirstDocTemplatePosition();
	CDocTemplate* pTemplate;
    while( p )
    {
		pTemplate = (CDocTemplate*)GetNextDocTemplate( p );

        if( pTemplate )
        {
		    CString str;
		    if( pTemplate->GetDocString( str, CDocTemplate::docName ) )
			    if( str.CompareNoCase( strName ) == 0 )
				    return pTemplate->CreateNewDocument();
        }
    }

	return NULL;
}

void CCthulhuApp::ShowDocument( CDocument* pDoc, CString strName, CString strFrameName /*""*/ )
{
	POSITION p;
	p = GetFirstDocTemplatePosition();
	CDocTemplate* pTemplate;
	while( p )
	{
		pTemplate = (CDocTemplate*)GetNextDocTemplate( p );

        if( pTemplate )
        {
		    CString str;
		    if( pTemplate->GetDocString( str, CDocTemplate::docName ) )
			    if( str.CompareNoCase( strName ) == 0 )
                {
	                CFrameWnd* pWnd = pTemplate->CreateNewFrame( pDoc, NULL );
	                if( pWnd )
	                {
		                pTemplate->InitialUpdateFrame( pWnd, pDoc, TRUE );
	                    pDoc->SetModifiedFlag();

                        //can't use SetDefaultTitle because we've taken over
                        //document creation and the indexing isn't updated,
                        //so all documents are #1 (we could keep our own count,
                        //but this is easier...)
                        CString strTitle = "New ";
                        strTitle += strFrameName.GetLength() > 0 ? strFrameName : strName;
                        pDoc->SetTitle( strTitle );
                        return;
	                }
                }
        }
	}


    AfxMessageBox( "Cannot create new document!", MB_ICONERROR );
    delete pDoc;
}


void CCthulhuApp::OnToolsOptions() 
{
    COptions Options;
    if( Options.DoModal() == IDOK )
    {
        //toggle the flatness on the toolbar
        if( AfxGetApp()->GetProfileInt( "Display", "FlatToolbar", TRUE ) == TRUE )
            ((CMainFrame*)AfxGetMainWnd())->GetToolBar().GetToolBarCtrl().ModifyStyle( 0, TBSTYLE_FLAT ); //make it flat
        else
            ((CMainFrame*)AfxGetMainWnd())->GetToolBar().GetToolBarCtrl().ModifyStyle( TBSTYLE_FLAT, 0 ); //make it non-flat

        //redraw the toolbar
        ((CMainFrame*)AfxGetMainWnd())->GetToolBar().RedrawWindow();

        //update all documents and their views
		POSITION p;
		p = GetFirstDocTemplatePosition();
		CMultiDocTemplate* pTemplate = NULL;
		while( p )
        {
			pTemplate = (CMultiDocTemplate*)GetNextDocTemplate( p );

			if( pTemplate  )
            {
			    CString str;
				if( pTemplate->GetDocString( str, CDocTemplate::docName ) && str.CompareNoCase( "investigator" ) == 0 )
				{
					CCthulhuDoc* pDoc;
					POSITION pp;
					pp = pTemplate->GetFirstDocPosition();
					while(pp)
                    {
						pDoc = (CCthulhuDoc*)pTemplate->GetNextDoc( pp );

                        if( pDoc )
                        {
                            pDoc->m_fNeedRedrawRecalculation = TRUE;

						    CView* pView;
						    POSITION ppp;
						    ppp = pDoc->GetFirstViewPosition();
						    while(ppp)
                            {
							    pView = (CCthulhuView*)pDoc->GetNextView(ppp);
							    if( pView && pView->IsKindOf( RUNTIME_CLASS(CCthulhuView) ) )
                                {
                                    CCthulhuView* pCthulhuView = (CCthulhuView*)pView;
                                    pCthulhuView->UpdateSettings();
                                }
						    }

                            pDoc->UpdateAllViews( NULL );
                        }
					}
				}
                else
                {
                    //update all views, without anything fancy
					CDocument* pDoc;
					POSITION pp;
					pp = pTemplate->GetFirstDocPosition();
					while(pp)
                    {
						pDoc = pTemplate->GetNextDoc( pp );
                        if( pDoc ) pDoc->UpdateAllViews( NULL );
					}
                }
            }
		}
    }
}

void CCthulhuApp::OnHelpLegalTrademarks() 
{
    CCopyrightDlg Copyright;
    Copyright.DoModal();
}

void CCthulhuApp::OnHelpTheByakheeProject() 
{
    CByakheeProjectDlg ProjectInfo;
    ProjectInfo.DoModal();
}
