// CthulhuView.cpp : implementation of the CCthulhuView class
//

#include "stdafx.h"
#include "Cthulhu.h"
#include "Skill.h"
#include "CthulhuDoc.h"
#include "CthulhuView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//global coordinate control constants
#define MARGIN_X                    80      //left & right hand margin size
#define BOTTOM_MARGIN               20      //bottom margin
#define USERTEXT_GAP                15      //how far after text to draw user text
#define TABLECIRCLE_SIZE            20      //size of circle to put around current value
#define TABLECIRCLE_PENSIZE         2       //size of pen to draw circle with
#define EDIT_FIELD_HEIGHT           34      //height of edit field rect returned by RectFromField

/////////////////////////////////////////////////////////////////////////////
// CCthulhuView

IMPLEMENT_DYNCREATE(CCthulhuView, CScrollView)

BEGIN_MESSAGE_MAP(CCthulhuView, CScrollView)
	//{{AFX_MSG_MAP(CCthulhuView)
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_EDIT_SHOWTEXT, OnEditShowText)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SHOWTEXT, OnUpdateEditShowText)
	ON_WM_MOUSEMOVE()
	ON_WM_SETCURSOR()
	ON_WM_CREATE()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	ON_COMMAND(ID_EDIT_NEXTFIELD, OnEditNextField)
	ON_COMMAND(ID_EDIT_PREVFIELD, OnEditPrevField)
	ON_COMMAND(ID_VIEW_WHATSHEETTYPE, OnViewWhatSheetType)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)

    ON_COMMAND_RANGE(ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50, ID_VIEW_CHARACTERSHEETSTYLES_DELTAGREEN, OnSheetStyleChange)
    ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50, ID_VIEW_CHARACTERSHEETSTYLES_DELTAGREEN, OnUpdateSheetStyle)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCthulhuView construction/destruction

CCthulhuView::CCthulhuView()
{
    m_DisplayType = DT_COC50;

    //load current view settings (text face names, colours etc)
    LoadSettings();

    m_pFieldList = NULL;
    m_nFieldCount = 0;
    m_iCurrent = -1;

    m_hEmfSide = NULL;
    m_hEmfDeltaGreen = NULL;
    LoadMetafiles();

    m_fShowText = TRUE;

}

CCthulhuView::~CCthulhuView()
{
    //delete current
    if( m_hEmfSide ) DeleteEnhMetaFile(m_hEmfSide); 
    if( m_hEmfDeltaGreen ) DeleteEnhMetaFile(m_hEmfDeltaGreen);

    DeleteFieldList();
}

BOOL CCthulhuView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CCthulhuView drawing

void CCthulhuView::OnDraw(CDC* pDC)
{
	CCthulhuDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//draw the character sheet
    CDC* pDrawDC;

    //create an off-screen buffer
    if( pDC->IsPrinting() == FALSE )
    {
        //see if the data needs a recalc
        if( pDoc->m_fNeedRedrawRecalculation == TRUE )
        {
            //yup, turn off the need recalc flag
            pDoc->m_fNeedRedrawRecalculation = FALSE;

            //update calculated bases
            pDoc->UpdateCalculatedBases();

            //release current ones
            m_DCOffscreen.DeleteDC();
            m_BmOffscreen.DeleteObject();

            //create an off-screen DC and a bitmap the same size as the sheet
            m_DCOffscreen.CreateCompatibleDC( pDC );
            CPoint pt; GetSheetSize( pt );
            RECT rc = { 0, 0, pt.x, -pt.y };
            m_DCOffscreen.SetMapMode(m_nMapMode);
            m_DCOffscreen.LPtoDP( &pt );
            m_BmOffscreen.CreateCompatibleBitmap( pDC, pt.x, -pt.y );
            m_DCOffscreen.SelectObject( &m_BmOffscreen );

            //clear the background
            m_DCOffscreen.FillRect( &rc, CBrush::FromHandle( (HBRUSH)GetStockObject(WHITE_BRUSH) ));

            pDrawDC = &m_DCOffscreen;

            DeleteFieldList();
        }
        else
        {
            CPoint pt;
            GetSheetSize( pt );
            //just copy the off-screen buffer over
            if( pDC->BitBlt( 0, 0, pt.x, -pt.y, &m_DCOffscreen, 0, 0, SRCCOPY ) == FALSE )
                //something wrong... redraw from scratch
                pDrawDC = pDC;
            else return;
        }
    }
    else pDrawDC = pDC;

    //drawing the sheet is a pretty lengthy operation!
    AfxGetApp()->BeginWaitCursor();

    //draw a version 5 character sheet
    switch( m_DisplayType )
    {
        default:
            AfxMessageBox( "Unknown or unexpected sheet display type!" );
            m_DisplayType = DT_COC50;
            //drop though...
        case DT_COC50:      DrawV5CharSheet( pDrawDC ); break;
        case DT_COC50SMALL: DrawV5CharSheetSmall( pDrawDC ); break;
        case DT_DELTAGREEN: DrawDeltaGreenCharSheet( pDrawDC ); break;
    }

    //copy buffer over
    if( pDC->IsPrinting() == FALSE )
    {
        CPoint pt;
        GetSheetSize( pt );
       //copy the off-screen buffer over
        pDC->BitBlt( 0, 0, pt.x, -pt.y, &m_DCOffscreen, 0, 0, SRCCOPY );
    }

    //OK, the wait is over...
    AfxGetApp()->EndWaitCursor();
}

/////////////////////////////////////////////////////////////////////////////
// CCthulhuView printing

BOOL CCthulhuView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CCthulhuView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CCthulhuView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CCthulhuView diagnostics

#ifdef _DEBUG
void CCthulhuView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CCthulhuView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CCthulhuDoc* CCthulhuView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCthulhuDoc)));
	return (CCthulhuDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCthulhuView message handlers



/*********************************************************************************************************
 ***  Character Sheet OnDraw() Helper Functions  *********************************************************
 *********************************************************************************************************/

void CCthulhuView::DrawCaptionBox( CDC* pDC, CString strCaption, CRect rcCaption )
{
    CFont TitleFont;
    TitleFont.CreateFont( -37, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, m_strTitleFace );
    
    //draw border
    pDC->Rectangle( &rcCaption );

    //draw title bar
    CRect rcTitle = rcCaption;
    rcTitle.bottom = rcTitle.top - 70;
    pDC->FillRect( rcTitle, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );

    //caption text
    CFont* pOldFont = pDC->SelectObject( &TitleFont );
    pDC->SetTextColor( 0xFFFFFF /*white*/ );
    pDC->DrawText( strCaption, &rcTitle, DT_CENTER|DT_SINGLELINE|DT_VCENTER|DT_NOCLIP|DT_NOPREFIX );
    pDC->SelectObject( pOldFont );
}

void CCthulhuView::LoadMetafiles()
{
    //delete current
    if( m_hEmfSide ) DeleteEnhMetaFile(m_hEmfSide); 
    if( m_hEmfDeltaGreen ) DeleteEnhMetaFile(m_hEmfDeltaGreen);

    //load a new side bar
    CString strFilename = GetDefaultDirectory();
    strFilename += "CoC.emf";
    m_hEmfSide = GetEnhMetaFile( strFilename );

    strFilename = GetDefaultDirectory();
    strFilename += "DG.emf";
    m_hEmfDeltaGreen = GetEnhMetaFile( strFilename );
}

void CCthulhuView::DrawV5CharSheet(CDC * pDC)
{
	CCthulhuDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

    //set text background mode
    pDC->SetBkMode( TRANSPARENT );

    //draw the boxes
    DrawCaptionBox( pDC, "Characteristics & Rolls", m_rcStat );
    DrawCaptionBox( pDC, "Sanity Points", m_rcSan );
    DrawCaptionBox( pDC, "Magic Points", m_rcMagic );
    DrawCaptionBox( pDC, "Hit Points", m_rcHit );
    DrawCaptionBox( pDC, "Investigator Skills", m_rcSkill );
    DrawCaptionBox( pDC, "Hand-To-Hand Weapons", m_rcHand );
    DrawCaptionBox( pDC, "Firearms", m_rcFire );


    //fill the boxes

    /******************************************************************************************************/
    /** ERA BOX *******************************************************************************************/
    /******************************************************************************************************/

    CFont EraFont;          //font for era box '1920s' etc
    EraFont.CreateFont( -50, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, m_strTitleFace );

    CRect rcEra = m_rcEra;
    pDC->FillRect( &rcEra, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcEra.top -= 20;

    //draw text
    CFont* pOldFont = pDC->SelectObject( &EraFont );
    CString strEra;
    switch( pDoc->m_Investigator.m_era )
    {
        case ERA_1890: strEra = "1890s"; break;
        case ERA_1920: strEra = "1920s"; break;
        case ERA_1990: strEra = "1990s"; break;
    }
    pDC->SetTextColor( 0xFFFFFF /*white*/ );
    pDC->SetTextAlign( TA_CENTER|TA_TOP );
    pDC->TextOut( ((rcEra.right-rcEra.left)/2)+rcEra.left, rcEra.top, strEra );

    //draw the metafile for the era
    rcEra.top -= 80;
    rcEra.bottom += 30;
    if( pDoc->m_hEmfEra )
    {
        ENHMETAHEADER emh;
        GetEnhMetaFileHeader( pDoc->m_hEmfEra, sizeof(emh), &emh );

        //calculate a width to keep the metafile 1:1 scaled
        int nScaleWidth = -(int)((((double)rcEra.bottom-(double)rcEra.top) * ((double)emh.rclFrame.right-(double)emh.rclFrame.left)/((double)emh.rclFrame.bottom-(double)emh.rclFrame.top)) / 2.0);
        int nCenterX = rcEra.left+((rcEra.right-rcEra.left)/2);

        //set the left & right positions
        rcEra.left = nCenterX-nScaleWidth;
        rcEra.right = nCenterX+nScaleWidth;

        //draw the metafile
        PlayEnhMetaFile( pDC->m_hDC, pDoc->m_hEmfEra, &rcEra );
    }




    /******************************************************************************************************/
    /** NAME BOX ******************************************************************************************/
    /******************************************************************************************************/
    //draw rectangle
    CRect rcName = m_rcName;
    pDC->Rectangle( &rcName );

    //draw text & lines
    rcName.top -= 55;
    rcName.left += 20;
    rcName.right -= 20;

    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    
    int nTextY = rcName.top;
    CFont TextFontBold;
    TextFontBold.CreateFont( -30, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );
    pDC->SelectObject( &TextFontBold );

    rcName.top -= 40; 
    DrawField( pDC, "Investigator Name", rcName.left, rcName.right, nTextY, pDoc->m_Investigator.m_strName, 270, DT_LEFT );
    if( !pDC->IsPrinting() ) AddField( &pDoc->m_Investigator.m_strName, rcName.left+270, rcName.right, nTextY, "Name" );
    nTextY -= 40;
    DrawField( pDC, "Occupation",        rcName.left, rcName.left+510, nTextY, pDoc->m_Investigator.m_strOccupation, 170, DT_LEFT );
    DrawField( pDC, "Sex",               rcName.left + 510, rcName.left+640, nTextY, pDoc->m_Investigator.m_strSex, 62, DT_LEFT );
    DrawField( pDC, "Age",               rcName.left + 640, rcName.right, nTextY, pDoc->m_Investigator.m_nAge, 75,DT_LEFT );
    if( !pDC->IsPrinting() )
    {
        AddField( &pDoc->m_Investigator.m_strOccupation, rcName.left+170, rcName.left+510, nTextY, "Occupation" );
        AddField( &pDoc->m_Investigator.m_strSex, rcName.left+510+62, rcName.left+640, nTextY, "Sex", 1 );
        AddField( &pDoc->m_Investigator.m_nAge, rcName.left+640+75, rcName.right-32, nTextY, "Age" );//-32 because field is very large and looks silly with a spin control
    }
    nTextY -= 40; 
    DrawField( pDC, "Colleges, Degrees", rcName.left, rcName.right, nTextY, pDoc->m_Investigator.m_strColleges, 260, DT_LEFT );
    if( !pDC->IsPrinting() ) AddField( &pDoc->m_Investigator.m_strColleges, rcName.left+260, rcName.right, nTextY, "Colleges" );
    nTextY -= 40; 
    DrawField( pDC, "Birthplace",        rcName.left, rcName.right, nTextY, pDoc->m_Investigator.m_strBirthplace, 160, DT_LEFT );
    if( !pDC->IsPrinting() ) AddField( &pDoc->m_Investigator.m_strBirthplace, rcName.left+160, rcName.right, nTextY, "Birthplace" );
    nTextY -= 40; 
    DrawField( pDC, "Mental Disorders",  rcName.left, rcName.right, nTextY, pDoc->m_Investigator.m_strMentalDisorders, 260, DT_LEFT );
    if( !pDC->IsPrinting() ) AddField( &pDoc->m_Investigator.m_strMentalDisorders, rcName.left+260, rcName.right, nTextY, "Mental Disorders" );
    nTextY -= 40;
    DrawUserField( pDC, rcName.left, rcName.right, nTextY, "", DT_LEFT );

   

    /******************************************************************************************************/
    /** STATS BOX *****************************************************************************************/
    /******************************************************************************************************/
    //draw top of box
    CRect rcStats = m_rcStat;

    //draw stats
    //draw text & lines
    rcStats.top -= 50 + 70;
    rcStats.right -= 10;

    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );

    //stat titles
    CFont StatFont;
    StatFont.CreateFont( -32, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    pDC->SelectObject( &StatFont );

    nTextY = rcStats.top;

    DrawField( pDC, "STR", rcStats.left+20, rcStats.left+160, nTextY,      pDoc->m_Investigator.m_nSTR, 75, DT_CENTER );
    DrawField( pDC, "CON", rcStats.left+20, rcStats.left+160, nTextY-50,   pDoc->m_Investigator.m_nCON, 80, DT_CENTER );
    DrawField( pDC, "SIZ", rcStats.left+20, rcStats.left+160, nTextY-100,  pDoc->m_Investigator.m_nSIZ, 55, DT_CENTER );

    DrawField( pDC, "DEX", rcStats.left+190, rcStats.left+350, nTextY,     pDoc->m_Investigator.m_nDEX, 70, DT_CENTER );
    DrawField( pDC, "APP", rcStats.left+190, rcStats.left+350, nTextY-50,  pDoc->m_Investigator.m_nAPP, 70, DT_CENTER );
    DrawField( pDC, "SAN", rcStats.left+190, rcStats.left+350, nTextY-100, pDoc->m_Investigator.m_nPOW*5, 70, DT_CENTER );


    DrawField( pDC, "INT", rcStats.left+390, rcStats.left+552, nTextY,     pDoc->m_Investigator.m_nINT, 60, DT_CENTER );
    DrawField( pDC, "POW", rcStats.left+390, rcStats.left+552, nTextY-50,  pDoc->m_Investigator.m_nPOW, 70, DT_CENTER );
    DrawField( pDC, "EDU", rcStats.left+390, rcStats.left+552, nTextY-100, pDoc->m_Investigator.m_nEDU, 70, DT_CENTER );
    
    DrawField( pDC, "Idea", rcStats.left+590, rcStats.right, nTextY,       pDoc->m_Investigator.m_nINT*5, 70, DT_CENTER );
    DrawField( pDC, "Luck", rcStats.left+590, rcStats.right, nTextY-50,    pDoc->m_Investigator.m_nPOW*5, 70, DT_CENTER );
    DrawField( pDC, "Know", rcStats.left+590, rcStats.right, nTextY-100,   pDoc->m_Investigator.m_nEDU*5, 85, DT_CENTER );

    
    if( !pDC->IsPrinting() ) 
    {
        AddField( &pDoc->m_Investigator.m_nSTR, rcStats.left+20+75, rcStats.left+160, nTextY, "STR" );
        AddField( &pDoc->m_Investigator.m_nCON, rcStats.left+20+80, rcStats.left+160, nTextY-50, "CON" );
        AddField( &pDoc->m_Investigator.m_nSIZ, rcStats.left+20+55, rcStats.left+160, nTextY-100, "SIZ" );

        AddField( &pDoc->m_Investigator.m_nDEX, rcStats.left+190+70, rcStats.left+350, nTextY, "DEX" );
        AddField( &pDoc->m_Investigator.m_nAPP, rcStats.left+190+70, rcStats.left+350, nTextY-50, "APP" );

        AddField( &pDoc->m_Investigator.m_nINT, rcStats.left+390+60, rcStats.left+552, nTextY, "INT" );
        AddField( &pDoc->m_Investigator.m_nPOW, rcStats.left+390+70, rcStats.left+552, nTextY-50, "POW" );
        AddField( &pDoc->m_Investigator.m_nEDU, rcStats.left+390+70, rcStats.left+552, nTextY-100, "EDU" );
    }

    CFont StatFontSmall;
    StatFontSmall.CreateFont( -28, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    pDC->SelectObject( &StatFontSmall );

    if( pDoc->m_Investigator.m_nPOW > 0 ) //don't display 99-Cthulhu Mythos if Pow == -1
    {
        CInvestigator::InvestigatorSkillList* pCthulhuMythos = pDoc->m_Investigator.FindSkill( "Cthulhu Mythos" );
        DrawField( pDC, "99-Cthulhu Mythos", rcStats.left+20, rcStats.left+350, nTextY-150, (pCthulhuMythos ? (99-pCthulhuMythos->nCurrent) : 99), 260, DT_CENTER );
    }
    else DrawField( pDC, "99-Cthulhu Mythos", rcStats.left+20, rcStats.left+350, nTextY-150, "", 260, DT_CENTER );
    DrawField( pDC, "Damage Bonus", rcStats.left+390, rcStats.left+800, nTextY-150, GetDBString( pDoc->m_Investigator.m_nSTR, pDoc->m_Investigator.m_nSIZ ), 210, DT_CENTER );


    
    
    /******************************************************************************************************/
    /** SIDE BOX ******************************************************************************************/
    /******************************************************************************************************/
    //draw the box
    CRect rcSide = m_rcSide;
    pDC->Rectangle( &rcSide );

    //draw the CoC picure
    ENHMETAHEADER emh;
    GetEnhMetaFileHeader( m_hEmfSide, sizeof(emh), &emh );

    //set the top & bottom regions to play it into
    CRect rcMetaFile = rcSide;
    rcMetaFile.right -= 15;
    rcMetaFile.left += 15;

    //calculate a height to keep the metafile 1:1 scaled
    int nScaleHeight = -(int)((((double)rcMetaFile.right-(double)rcMetaFile.left) * ((double)emh.rclFrame.bottom-(double)emh.rclFrame.top)/((double)emh.rclFrame.right-(double)emh.rclFrame.left)));

    //set the top & bottom positions
    rcMetaFile.top = rcSide.bottom+30-nScaleHeight;
    rcMetaFile.bottom = rcSide.bottom+30;

    //draw the metafile and release it
    PlayEnhMetaFile( pDC->m_hDC, m_hEmfSide, &rcMetaFile );

    //draw the player name
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );

    //draw the player name caption
    int nCaptionSize;
    if( m_iRotateType != 2 )
    {
        CFont PlayerNameText;

        if( m_iRotateType == 1 && pDC->IsPrinting() == TRUE )
            PlayerNameText.CreateFont( -32, 0, 900, 900, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );
        else
            PlayerNameText.CreateFont( -32, 0, 2700, 2700, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );

        pDC->SelectObject( &PlayerNameText );

        pDC->TextOut( rcSide.right - 28, rcSide.top+(rcSide.Height()/2), "Player's Name" );
        nCaptionSize = pDC->GetTextExtent("Player's Name").cx;
        pDC->MoveTo( rcSide.right - 38, rcSide.top+(rcSide.Height()/2)+10+nCaptionSize );
        pDC->LineTo( rcSide.right - 38, rcSide.top - 90 );
    }

    if( m_fShowText && m_iRotateType != 2 )
    {
        CFont UserTextFontRotate;
        if( m_iRotateType == 1 && pDC->IsPrinting() == TRUE )
            UserTextFontRotate.CreateFont( -32, 0, 900, 900, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, m_strUserTextFace );
        else
            UserTextFontRotate.CreateFont( -32, 0, 2700, 2700, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, m_strUserTextFace );

        pDC->SelectObject( &UserTextFontRotate );
        COLORREF crCurrent = pDC->GetTextColor();
        pDC->SetTextColor( m_crUserText );
        pDC->TextOut( rcSide.right - 34, rcSide.top+(rcSide.Height()/2)+USERTEXT_GAP+nCaptionSize, pDoc->m_Investigator.m_strPlayerName );
        pDC->SetTextColor( crCurrent );
    }

    if( m_iRotateType != 2 )
    {
        //draw "investigator sheet" text
        CFont InvestigatorSheet;
        int nExtra = pDC->GetTextCharacterExtra();
        pDC->SetTextCharacterExtra( 10 );
        if( m_iRotateType == 1 && pDC->IsPrinting() == TRUE )
            InvestigatorSheet.CreateFont( -52, 0, 900, 900, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, m_strTitleFace );
        else
            InvestigatorSheet.CreateFont( -52, 0, 2700, 2700, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, m_strTitleFace );
        pDC->SelectObject( &InvestigatorSheet );

        pDC->SetTextAlign( TA_LEFT|TA_TOP );
        pDC->TextOut( rcSide.left + 10, rcSide.top+(rcSide.Height()/2), "Investigator Sheet" );
        pDC->SetTextCharacterExtra( nExtra );
    }



    /******************************************************************************************************/
    /** SANITY BOX ****************************************************************************************/
    /******************************************************************************************************/
    CRect rcSan = m_rcSan;
    rcSan.top -= 60 + 70;
    rcSan.left += 30;

    //create the table fonts
    CFont TableTextFont;
    TableTextFont.CreateFont( -27, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );

    CFont TableFontBold;
    TableFontBold.CreateFont( -28, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TableFontBold );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    pDC->TextOut( rcSan.left, rcSan.top, "Insanity 0" );
    int nXStep = 48;
    int nYStep = 50;
    int nXStart = 3;

    //draw the table
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    pDC->SelectObject( &TableTextFont );
    rcSan.left += pDC->GetTextExtent("8").cx;

    int nCount = 1;
    int nSAN = pDoc->m_Investigator.m_nSAN;
    if( pDoc->m_Investigator.m_nSAN == -999 && pDoc->m_Investigator.m_nPOW > 0 ) nSAN = pDoc->m_Investigator.m_nSAN = pDoc->m_Investigator.m_nPOW*5;

    for( int nTableY = 0; nTableY < 6; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 17; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nSAN && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcSan.left, (rcSan.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }

            pDC->TextOut( (nTableX*nXStep)+rcSan.left, (rcSan.top-((nTableY*nYStep))), str );

            nCount++;
        }

        nXStart = 0;
    }




    /******************************************************************************************************/
    /** MAGIC BOX******************************************************************************************/
    /******************************************************************************************************/
    //draw the caption
    CRect rcMagic = m_rcMagic;
    rcMagic.top -= 60 + 70;
    rcMagic.left += 40;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TableFontBold );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    int nTextHeight = pDC->GetTextExtent( "8" ).cy;
    pDC->TextOut( rcMagic.left, rcMagic.top, "Unconscious" );
    nXStep = 48;
    nYStep = 50;
    nXStart = 4;

    //draw the table
    pDC->SelectObject( &TableTextFont );
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    rcMagic.left += pDC->GetTextExtent("8").cx;
    pDC->SelectObject( &TableFontBold );

    nCount = 0;
    int nMagic = pDoc->m_Investigator.m_nMagic;
    if( pDoc->m_Investigator.m_nMagic == -999 && pDoc->m_Investigator.m_nPOW > 0 ) nMagic = pDoc->m_Investigator.m_nMagic = pDoc->m_Investigator.m_nPOW;
    for( nTableY = 0; nTableY < 6; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 7; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nMagic && m_fShowText && pDoc->m_Investigator.m_nPOW >= 0 )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcMagic.left, (rcMagic.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }
            
            pDC->TextOut( (nTableX*nXStep)+rcMagic.left, (rcMagic.top-((nTableY*nYStep))), str );

            //change font to non-bold after 0
            if( nCount == 0 ) pDC->SelectObject( &TableTextFont );
            nCount++;
        }

        nXStart = 0;
    }




    /******************************************************************************************************/
    /** HITPOINT BOX **************************************************************************************/
    /******************************************************************************************************/
    CRect rcHit = m_rcHit;
    rcHit.top -= 60 + 70;
    rcHit.left += 40;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TableFontBold );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    pDC->TextOut( rcHit.left, rcHit.top, "Dead" );
    nXStep = 48;
    nYStep = 50;
    nXStart = 2;

    //draw the table
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    rcHit.left += pDC->GetTextExtent("8").cx;

    nCount = -2;
    int nHP = pDoc->m_Investigator.m_nHitPoints;
    if( nHP == -999 )
    {
        if( pDoc->m_Investigator.m_nCON > 0 && pDoc->m_Investigator.m_nSIZ > 0 )
            nHP = pDoc->m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)pDoc->m_Investigator.m_nCON + (double)pDoc->m_Investigator.m_nSIZ ) / 2.0 ) );
    }
    for( nTableY = 0; nTableY < 6; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 7; nTableX++ )
        {
            CString str;
            if( nTableY == 0 && nCount > 0 ) str.Format( "+%d", nCount ); else str.Format( "%d", nCount );

            if( nCount == nHP && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcHit.left, (rcHit.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }

            nCount++;
            pDC->TextOut( (nTableX*nXStep)+rcHit.left, (rcHit.top-((nTableY*nYStep))), str );
        }

        //turn off bold font after first line
        if( nTableY == 0 ) pDC->SelectObject( &TableTextFont );
        nXStart = 0;
    }

    //draw the 'Unconscious' text
    pDC->SetTextAlign( TA_RIGHT|TA_TOP );
    CFont UnconsciousFont;
    UnconsciousFont.CreateFont( -19, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    pDC->SelectObject( &UnconsciousFont );
    pDC->TextOut( rcHit.right-12, rcHit.top+55, "UNCONSCIOUS" );


    
    
    /******************************************************************************************************/
    /** SKILLS BOX ****************************************************************************************/
    /******************************************************************************************************/
    //draw caption
    CRect rcSkill = m_rcSkill;
    rcSkill.top-=70;
    rcSkill.bottom += 30;

    //draw skills
    CFont Text;
    Text.CreateFont( -30, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );
    
    pDC->SelectObject( &Text );


    rcSkill.top -= 52;
    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    int nDrawX = rcSkill.left, nDrawY = rcSkill.top;
    int nColumn = 0;
    int nSkills = 0;
    int nBaseFirearms = 0;
    CInvestigator::InvestigatorSkillList* pSkills = pDoc->m_Investigator.GetList();
    while( pSkills && nColumn < 3 )
    {
        if( pSkills->pSkill->GetType() != ANYSKILL )
        {
            if( pSkills->pSkill->m_strBase[0] == 'F' )
                nBaseFirearms++;
            else
                switch( pSkills->pSkill->GetType() )
                {
                    case SKILL:
                    {
                        //draw skill check box
                        RECT rcCheck = { nDrawX+45, nDrawY+25+2, nDrawX+45+25, nDrawY+2 };
                        if( pSkills->pSkill->m_strBase[0] != '*' ) pDC->Rectangle( &rcCheck );

                        //draw skill name
                        pDC->SelectObject( &Text );
                        pDC->SetTextAlign( DT_BOTTOM|DT_LEFT );

                        const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                        while( *pszName == '*' ) pszName++;
                        CString strText = pszName;
                        strText += " (";
                        strText += GetSkillCode( pSkills->pSkill->m_strBase );
                        strText += ")";
                        pDC->TextOut( nDrawX + 95, nDrawY, strText );

                        //draw current worthiness
                        if( pSkills->nCurrent > -1 && ( pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE || pSkills->pSkill->IsCalculatedBase() == TRUE ) )
                            DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, pSkills->nCurrent, DT_LEFT );
                        else
                            DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, "", DT_LEFT );

                        //add skill rect
/* commenting out allows editing of cthulhu mythos et al
                        if( pSkills->pSkill->m_strBase[0] != '*' ) */
                            if( !pDC->IsPrinting() ) AddField( &pSkills->nCurrent, nDrawX+415, nDrawX+415+110, nDrawY, CString(pszName), pSkills->nMinimumValue );

                        //update drawX & Y
                        nDrawY -= 41;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            if( nColumn == 2 ) while( nDrawY+41 > m_rcPic.bottom - 25 ) nDrawY -= 41;
                            nDrawX += 560;
                        }

                        nSkills++;
                        break;
                    }

                    case SPECIALISESKILL:
                    {
                        //draw skill name
                        pDC->SelectObject( &Text );

                        const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                        while( *pszName == '*' ) pszName++;
                        CString strText = pszName;
                        strText += " (";
                        strText += GetSkillCode( pSkills->pSkill->m_strBase );
                        strText += "):";
                        pDC->TextOut( nDrawX + 95, nDrawY, strText );
                        nDrawY -= 41;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            if( nColumn == 2 ) while( nDrawY+41 > m_rcPic.bottom - 25 ) nDrawY -= 41;
                            nDrawX += 560;
                        }
                        nSkills++;
                        break;
                    }

                    case SPECIALISESKILLITEM:
                    {
                        //draw skill check box
                        RECT rcCheck = { nDrawX+45, nDrawY+25+2, nDrawX+45+25, nDrawY+2 };
                        if( pSkills->pSkill->m_strBase[0] != '*' ) pDC->Rectangle( &rcCheck );

                        //draw skill subname and current worthiness
                        DrawUserField( pDC, nDrawX+95 - 10, nDrawX+95+240, nDrawY, pSkills->pSkill->m_strName, DT_LEFT );

                        if( pSkills->nCurrent > -1 && (pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE || pSkills->pSkill->IsCalculatedBase() == TRUE) && (pSkills->pSkill->m_strName.GetLength() > 0 || ((CSpecialiseSkillSkill*)pSkills->pSkill)->m_pSkill->m_strName.CompareNoCase( "Own Language" ) == 0 ) )
                            DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, pSkills->nCurrent, DT_LEFT );
                        else
                            DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, "", DT_LEFT );

                        //add fields
                        if( !pDC->IsPrinting() )
                        {
                            AddField( &pSkills->pSkill->m_strName, nDrawX+95 - 10, nDrawX+95+240+80, nDrawY, "Skill Name" );
                            if( pSkills->pSkill->m_strBase[0] != '*' )
                                AddField( &pSkills->nCurrent, nDrawX+415, nDrawX+415+110, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );
                        }

                        //update drawX & Y
                        nDrawY -= 41;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            if( nColumn == 2 ) while( nDrawY+41 > m_rcPic.bottom - 25 ) nDrawY -= 41;
                            nDrawX += 560;
                        }

                        nSkills++;
                        break;
                    }
                }
        }

        pSkills = pSkills->next;
    }

    //draw blank lines
    for( int nBlank = 0; nBlank < 68-nSkills-nBaseFirearms; nBlank++ )
    {
        //draw check box
        RECT rcCheck = { nDrawX+45, nDrawY+25+2, nDrawX+45+25, nDrawY+2 };
        pDC->Rectangle( &rcCheck );

        //draw lines
        DrawUserField( pDC, nDrawX+95 - 10, nDrawX+95+240, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, "", DT_LEFT );

        //next line...
        nDrawY -= 41;
        if( nDrawY < rcSkill.bottom )
        {
            //column wrap
            nColumn++;
            nDrawY = rcSkill.top;
            if( nColumn == 2 ) while( nDrawY+41 > m_rcPic.bottom - 25 ) nDrawY -= 41;
            nDrawX += 560;
        }
    }

    if( nColumn < 3 )
    {
        //draw firearms caption
        CFont FirearmsText;
        FirearmsText.CreateFont( -30, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
        pDC->SelectObject( &FirearmsText );
        pDC->TextOut( nDrawX + 95, nDrawY-2, "Firearms" );
        nDrawY -= 41;
        if( nDrawY < rcSkill.bottom )
        {
            //column wrap
            nColumn++;
            nDrawY = rcSkill.top;
            if( nColumn == 2 ) while( nDrawY+41 > m_rcPic.bottom - 25 ) nDrawY -= 41;
            nDrawX += 560;
        }
    }

    //draw basic firearms
    pSkills = pDoc->m_Investigator.GetList();
    while( pSkills && nColumn < 3 )
    {
        if( pSkills->pSkill->GetType() != ANYSKILL )
        {
            if( pSkills->pSkill->m_strBase[0] == 'F' )
            {
                //draw skill check box
                RECT rcCheck = { nDrawX+45, nDrawY+25+2, nDrawX+45+25, nDrawY+2 };
                if( pSkills->pSkill->m_strBase[0] != '*' ) pDC->Rectangle( &rcCheck );

                //draw skill name
                pDC->SelectObject( &Text );
                pDC->SetTextAlign( DT_BOTTOM|DT_LEFT );

                const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                while( *pszName == '*' ) pszName++;
                CString strText = pszName;
                strText += " (";
                strText += GetSkillCode( LPCSTR(pSkills->pSkill->m_strBase)+1 );
                strText += ")";
                pDC->TextOut( nDrawX + 95, nDrawY, strText );

                //draw current worthiness
                if( pSkills->nCurrent > -1 && pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE )
                    DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, pSkills->nCurrent, DT_LEFT );
                else
                    DrawUserField( pDC, nDrawX+415, nDrawX+415+110, nDrawY, "", DT_LEFT );

                //add field
                if( pSkills->pSkill->m_strBase[0] != '*' )
                    if( !pDC->IsPrinting() ) AddField( &pSkills->nCurrent, nDrawX+415, nDrawX+415+110, nDrawY, CString(pszName), pSkills->nMinimumValue );

                //update drawX & Y
                nDrawY -= 41;
                if( nDrawY < rcSkill.bottom )
                {
                    //column wrap
                    nColumn++;
                    nDrawY = rcSkill.top;
                    if( nColumn == 2 ) while( nDrawY+41 > m_rcPic.bottom - 25 ) nDrawY -= 41;
                    nDrawX += 560;
                }
            }
        }

        pSkills = pSkills->next;
    }



    /******************************************************************************************************/
    /** PORTRAIT BOX **************************************************************************************/
    /******************************************************************************************************/
    //draw the box and it's shadow
    CRect rcPictureShadow = m_rcPic;
    rcPictureShadow.left += 25;
    rcPictureShadow.right += 25;
    rcPictureShadow.bottom -= 25;
    rcPictureShadow.top -= 25;
    pDC->FillRect( rcPictureShadow, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    pDC->Rectangle( &m_rcPic );
    DrawPortrait( pDC );




    /******************************************************************************************************/
    /** HAND TO HAND BOX **********************************************************************************/
    /******************************************************************************************************/
    //draw the box
    CRect rcHand = m_rcHand;
    rcHand.top-=70;
    nDrawX = rcHand.left, nDrawY = rcHand.top;

    //draw column headings
    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    CFont CaptionText;
    CaptionText.CreateFont( -23, 0, 0, 0, FW_NORMAL, TRUE/*italic*/, 0, 0, 0, 0, 0, 0, 0, "Arial" );
   
    pDC->SelectObject( &CaptionText );

    int nHeight = pDC->GetTextExtent("X").cy;
    nDrawY -= 40;
    pDC->TextOut( nDrawX+150, nDrawY, "Attack or"  );
    pDC->TextOut( nDrawX+150, nDrawY-nHeight, "Weapon"  );

    pDC->TextOut( nDrawX+317, nDrawY, "Current"  );
    pDC->TextOut( nDrawX+317, nDrawY-nHeight, "Skill %"  );

    pDC->TextOut( nDrawX+454, nDrawY, "Attack"  );
    pDC->TextOut( nDrawX+454, nDrawY-nHeight, "Damage"  );

    pDC->TextOut( nDrawX+585, nDrawY, "Attacks"  );
    pDC->TextOut( nDrawX+585, nDrawY-nHeight, "/Round"  );

    pDC->TextOut( nDrawX+660, nDrawY, "Hit"  );
    pDC->TextOut( nDrawX+660, nDrawY-nHeight, "Points"  );

    //draw melee skills
    pDC->SelectObject( &Text );

    nDrawY = rcHand.top-120;
    nSkills = 0;
    pSkills = pDoc->m_Investigator.GetList();
    while( pSkills && nSkills < 8 )
    {
        if( pSkills->pSkill->GetType() == MELEE )
        {
            //draw skill check box
            RECT rcCheck = { nDrawX+30, nDrawY+25+2, nDrawX+30+25, nDrawY+2 };
            pDC->Rectangle( &rcCheck );

            pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
            if( pSkills->pSkill->m_strName[0] == '*' )
            {
                //draw skill name
                pDC->SelectObject( &Text );
                pDC->SetTextAlign( DT_BOTTOM|DT_LEFT );

                const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                while( *pszName == '*' ) pszName++;
                CString strText = pszName;
                strText += " (";
                strText += GetSkillCode( pSkills->pSkill->m_strBase );
                strText += ")";

                pDC->TextOut( nDrawX+70, nDrawY, strText );
            }
            else
            {
                //draw skill name
                DrawUserField( pDC, nDrawX+70-10, nDrawX+265, nDrawY, pSkills->pSkill->m_strName, DT_LEFT );

                //add field to edit weapon name
                if( !pDC->IsPrinting() ) AddField( &pSkills->pSkill->m_strName, nDrawX+70-10, nDrawX+265, nDrawY, "Weapon Name");
            }

            //draw current worthiness
            if( pSkills->nCurrent > -1 && ( pSkills->pSkill->m_strName[0] != '*' || pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE ) )
                DrawUserField( pDC, nDrawX+280, nDrawX+355, nDrawY, pSkills->nCurrent, DT_LEFT );
            else
                DrawUserField( pDC, nDrawX+280, nDrawX+355, nDrawY, "", DT_LEFT );

            //add skill field
            if( pSkills->pSkill->m_strBase[0] != '*' )
                if( !pDC->IsPrinting() ) AddField( &pSkills->nCurrent, nDrawX+280, nDrawX+355, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );

            //draw other stuff
            CMeleeSkill* pMelee = (CMeleeSkill*)pSkills->pSkill;
            DrawUserField( pDC, nDrawX+370, nDrawX+538, nDrawY, pMelee->m_strDmg, DT_CENTER );
            DrawUserField( pDC, nDrawX+555, nDrawX+615, nDrawY, pMelee->m_strAttacks, DT_CENTER );
            DrawUserField( pDC, nDrawX+630, nDrawX+690, nDrawY, pMelee->m_strHP, DT_CENTER );

            if( !pDC->IsPrinting() ) 
            {
                AddField( &pMelee->m_strDmg,     nDrawX+370, nDrawX+538, nDrawY, "Weapon Damage" );
                AddField( &pMelee->m_strAttacks, nDrawX+555, nDrawX+615, nDrawY, "Weapon Attacks" );
                AddField( &pMelee->m_strHP,      nDrawX+630, nDrawX+690, nDrawY, "Weapon Hit Points" );
            }

            //update drawX & Y
            nDrawY -= 50;
            nSkills++;
        }

        pSkills = pSkills->next;
    }

    //draw blank lines
    for( nBlank = nSkills; nBlank < 8; nBlank++ )
    {
        //draw tick box
        RECT rcCheck = { nDrawX+30, nDrawY+25+2, nDrawX+30+25, nDrawY+2 };
        pDC->Rectangle( &rcCheck );

        //draw dots
        DrawUserField( pDC, nDrawX+70-10, nDrawX+265, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+280, nDrawX+355, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+370, nDrawX+538, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+555, nDrawX+615, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+630, nDrawX+690, nDrawY, "", DT_LEFT );

        nDrawY -= 50;
    }




    /******************************************************************************************************/
    /** FIREARMS BOX **************************************************************************************/
    /******************************************************************************************************/
    CRect rcFire = m_rcFire;
    rcFire.top -= 70;
    nDrawX = rcFire.left;
    nDrawY = rcFire.top;

    //draw column headings
    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
   
    pDC->SelectObject( &CaptionText );
    nDrawY -= nHeight;
    pDC->TextOut( nDrawX + 38, nDrawY-nHeight, "Firearm"  );

    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    pDC->TextOut( nDrawX + 310, nDrawY, "Current"  );
    pDC->TextOut( nDrawX + 310, nDrawY-nHeight, "Skill %"  );

    pDC->TextOut( nDrawX + 460, nDrawY, "Attack"  );
    pDC->TextOut( nDrawX + 460, nDrawY-nHeight, "Damage"  );

    pDC->TextOut( nDrawX + 630, nDrawY, "Base"  );
    pDC->TextOut( nDrawX + 630, nDrawY-nHeight, "Range"  );

    pDC->TextOut( nDrawX + 730, nDrawY, "Shots Per"  );
    pDC->TextOut( nDrawX + 730, nDrawY-nHeight, "Round"  );

    pDC->TextOut( nDrawX + 850, nDrawY, "Shots in"  );
    pDC->TextOut( nDrawX + 850, nDrawY-nHeight, "Gun"  );

    pDC->TextOut( nDrawX + 960, nDrawY, "Malfunct."  );
    pDC->TextOut( nDrawX + 960, nDrawY-nHeight, "Number"  );

    pDC->TextOut( nDrawX + 1060, nDrawY, "Hit"  );
    pDC->TextOut( nDrawX + 1060, nDrawY-nHeight, "Points"  );

    //draw firearms
    pDC->SelectObject( &Text );
    nDrawY = rcFire.top-110;
    nSkills = 0;
    pSkills = pDoc->m_Investigator.GetList();

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    while( pSkills && nSkills < 9 )
    {
        if( pSkills->pSkill->GetType() == FIREARM )
        {
            //draw skill name
            DrawUserField( pDC, nDrawX+35-10, nDrawX+248, nDrawY, pSkills->pSkill->m_strName, DT_RIGHT, TRUE, 30 );

            //draw current worthiness
            DrawUserField( pDC, nDrawX+260-10, nDrawX+352, nDrawY, pSkills->nCurrent, DT_LEFT );

            //draw other stuff
            CFirearmSkill* pFirearm = (CFirearmSkill*)pSkills->pSkill;
            DrawUserField( pDC, nDrawX+367-10, nDrawX+562, nDrawY, pFirearm->m_strDmg, DT_CENTER );
            DrawUserField( pDC, nDrawX+580-10, nDrawX+700, nDrawY, pFirearm->m_strRange, DT_CENTER );
            DrawUserField( pDC, nDrawX+715-10, nDrawX+779, nDrawY, pFirearm->m_strShots, DT_CENTER );
            DrawUserField( pDC, nDrawX+791-10, nDrawX+899, nDrawY, pFirearm->m_strBullets, DT_CENTER );
            DrawUserField( pDC, nDrawX+912-10, nDrawX+1005, nDrawY, pFirearm->m_strMAL, DT_CENTER );
            DrawUserField( pDC, nDrawX+1020-10, nDrawX+1110, nDrawY, pFirearm->m_strHP, DT_CENTER );

            //add skill rect
            if( !pDC->IsPrinting() ) 
            {
                AddField( &pSkills->pSkill->m_strName, nDrawX+35-10, nDrawX+248, nDrawY, "Weapon Name" );
                if( pSkills->pSkill->m_strBase[0] != '*' )
                    AddField( &pSkills->nCurrent, nDrawX+260-10, nDrawX+352, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );

                AddField( &pFirearm->m_strDmg,     nDrawX+367, nDrawX+562, nDrawY, "Firearm Damage" );
                AddField( &pFirearm->m_strRange,   nDrawX+580, nDrawX+700, nDrawY, "Firearm Range" );
                AddField( &pFirearm->m_strShots,   nDrawX+715, nDrawX+779, nDrawY, "Firearm Shots" );
                AddField( &pFirearm->m_strBullets, nDrawX+791, nDrawX+899, nDrawY, "Firearm Bullets" );
                AddField( &pFirearm->m_strMAL,     nDrawX+912, nDrawX+1005,nDrawY, "Firearm MAL Number" );
                AddField( &pFirearm->m_strHP,      nDrawX+1020,nDrawX+1110,nDrawY, "Firearm Hit Points" );
            }

            //update drawX & Y
            nDrawY -= 45;
            nSkills++;
        }

        pSkills = pSkills->next;
    }

    //draw blank lines
    for( nBlank = nSkills; nBlank < 9; nBlank++ )
    {
        DrawUserField( pDC, nDrawX+35-10, nDrawX+248, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+260-10, nDrawX+352, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+367-10, nDrawX+562, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+580-10, nDrawX+700, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+715-10, nDrawX+779, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+791-10, nDrawX+899, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+912-10, nDrawX+1005, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+1020-10, nDrawX+1110, nDrawY, "", DT_LEFT );

        nDrawY -= 45;
    }

    pDC->SelectObject( pOldFont );
}


void CCthulhuView::DrawV5CharSheetSmall( CDC* pDC )
{
	CCthulhuDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

    //set text background mode
    pDC->SetBkMode( TRANSPARENT );

    /******************************************************************************************************/
    /** OUTLINE OF WHOLE SHEET ****************************************************************************/
    /******************************************************************************************************/
    CPen FatPen( PS_SOLID, 5, RGB(0,0,0)/*black*/ );
    CPen* pPenOld = pDC->SelectObject( &FatPen );
    pDC->Rectangle( m_rcPic.left-50, m_rcPic.top+50, m_rcSkill.right+50, m_rcSkill.bottom-50 );

    /******************************************************************************************************/
    /** TABLE BOXES ***************************************************************************************/
    /******************************************************************************************************/

    CFont TitleFont;
    TitleFont.CreateFont( -28, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    CFont* pOldFont = pDC->SelectObject( &TitleFont );
    pDC->SetTextColor( 0xFFFFFF /*white*/ );

    //draw border
    CRect rcCaption = m_rcSan;              pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 35;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcHit;                    pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 35;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcMagic;                  pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 35;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcFire;                   pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 35;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcSkill;                  pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 35;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );

    //caption text
    pDC->DrawText( "SANITY POINTS",       &m_rcSan,    DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "HIT POINTS",          &m_rcHit,    DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "MAGIC POINTS",        &m_rcMagic,  DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "WEAPONS",             &m_rcFire,   DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "INVESTIGATOR POINTS", &m_rcSkill,  DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );



    /******************************************************************************************************/
    /** NAME BOX ******************************************************************************************/
    /******************************************************************************************************/
    //draw rectangle
    CRect rcName = m_rcName;
    pDC->Rectangle( &rcName );

    //captions
    CFont NameFont;
    NameFont.CreateFont( -37, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, m_strTitleFace );
    pDC->SelectObject( &NameFont );
    pDC->SetTextColor( 0xFFFFFF /*white*/ );
    pDC->SetTextAlign( TA_LEFT|TA_TOP );

    CRect rcBox = rcName;
    rcBox.right = rcBox.left + 130;
    pDC->FillRect( rcBox, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    pDC->DrawText( "Name", &rcBox, DT_CENTER|DT_SINGLELINE|DT_VCENTER|DT_NOCLIP|DT_NOPREFIX );
    rcBox = rcName;
    rcBox.left = rcBox.left + 540;
    rcBox.right = rcBox.left + 100;
    pDC->FillRect( rcBox, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    pDC->DrawText( "Age", &rcBox, DT_CENTER|DT_SINGLELINE|DT_VCENTER|DT_NOCLIP|DT_NOPREFIX );
    rcBox = rcName;
    rcBox.left = rcBox.left + 710;
    rcBox.right = rcBox.left + 230;
    pDC->FillRect( rcBox, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    pDC->DrawText( "Occupation", &rcBox, DT_CENTER|DT_SINGLELINE|DT_VCENTER|DT_NOCLIP|DT_NOPREFIX );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    DrawUserField( pDC, rcName.left+120, rcName.left+540, rcName.bottom+20, pDoc->m_Investigator.m_strName, DT_LEFT, FALSE );
    DrawUserField( pDC, rcName.left+630, rcName.left+710, rcName.bottom+20, pDoc->m_Investigator.m_nAge, DT_LEFT, FALSE );
    DrawUserField( pDC, rcName.left+930, rcName.right,    rcName.bottom+20, pDoc->m_Investigator.m_strOccupation, DT_LEFT, FALSE );
    if( !pDC->IsPrinting() ) 
    {
        AddField( &pDoc->m_Investigator.m_strName,       rcName.left+130, rcName.left+540, rcName.bottom+20, "Name" );
        AddField( &pDoc->m_Investigator.m_nAge,          rcName.left+640, rcName.left+710, rcName.bottom+20, "Age" );
        AddField( &pDoc->m_Investigator.m_strOccupation, rcName.left+940, rcName.right,    rcName.bottom+20, "Occupation" );
    }

    /******************************************************************************************************/
    /** PORTRAIT BOX **************************************************************************************/
    /******************************************************************************************************/
    //draw the box and it's shadow
    pDC->Rectangle( &m_rcPic );
    DrawPortrait( pDC );

    //use normal lines again...
    pDC->SelectObject( pPenOld );   
    

    /******************************************************************************************************/
    /** STATS BOX *****************************************************************************************/
    /******************************************************************************************************/
    CRect rcStats = m_rcStat;
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );

    //stat titles
    CFont StatFont;
    StatFont.CreateFont( -26, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    pDC->SelectObject( &StatFont );

    int nTextY = rcStats.top - 40;

    int nHP = (int)( 0.5 + (((double)pDoc->m_Investigator.m_nCON + (double)pDoc->m_Investigator.m_nSIZ ) / 2.0 ) );
    DrawField( pDC, "STR", rcStats.left,     rcStats.left+100, nTextY, pDoc->m_Investigator.m_nSTR,   50, DT_CENTER );
    DrawField( pDC, "CON", rcStats.left+120, rcStats.left+230, nTextY, pDoc->m_Investigator.m_nCON,   60, DT_CENTER );
    DrawField( pDC, "SIZ", rcStats.left+250, rcStats.left+340, nTextY, pDoc->m_Investigator.m_nSIZ,   45, DT_CENTER );
    DrawField( pDC, "INT", rcStats.left+370, rcStats.left+460, nTextY, pDoc->m_Investigator.m_nINT,   40, DT_CENTER );
    DrawField( pDC, "POW", rcStats.left+480, rcStats.left+590, nTextY, pDoc->m_Investigator.m_nPOW,   60, DT_CENTER );
    DrawField( pDC, "Idea",rcStats.left+630, rcStats.left+770, nTextY, pDoc->m_Investigator.m_nINT*5, 50, DT_CENTER );
    DrawField( pDC, "Luck",rcStats.left+820, rcStats.left+990, nTextY, pDoc->m_Investigator.m_nPOW*5, 60, DT_CENTER );
    DrawField( pDC, "Know",rcStats.left+1010,rcStats.left+1190,nTextY, pDoc->m_Investigator.m_nEDU*5, 70, DT_CENTER );
    //FIXME: draw the "%" after Idea Luck & Know

    DrawField( pDC, "DEX", rcStats.left,     rcStats.left+100, nTextY-40, pDoc->m_Investigator.m_nDEX,   50, DT_CENTER );
    DrawField( pDC, "APP", rcStats.left+120, rcStats.left+230, nTextY-40, pDoc->m_Investigator.m_nAPP,   55, DT_CENTER );
    DrawField( pDC, "SAN", rcStats.left+255, rcStats.left+360, nTextY-40, pDoc->m_Investigator.m_nPOW*5, 55, DT_CENTER );
    DrawField( pDC, "EDU", rcStats.left+370, rcStats.left+475, nTextY-40, pDoc->m_Investigator.m_nEDU,   55, DT_CENTER );
    DrawField( pDC, "HP",  rcStats.left+490, rcStats.left+570, nTextY-40, nHP,                           40, DT_CENTER );
   
    
    if( !pDC->IsPrinting() ) 
    {
        AddField( &pDoc->m_Investigator.m_nSTR, rcStats.left+50,     rcStats.left+100, nTextY, "STR" );
        AddField( &pDoc->m_Investigator.m_nCON, rcStats.left+60+120, rcStats.left+230, nTextY, "CON" );
        AddField( &pDoc->m_Investigator.m_nSIZ, rcStats.left+45+250, rcStats.left+340, nTextY, "SIZ" );
        AddField( &pDoc->m_Investigator.m_nINT, rcStats.left+40+370, rcStats.left+460, nTextY, "INT" );
        AddField( &pDoc->m_Investigator.m_nPOW, rcStats.left+60+480, rcStats.left+590, nTextY, "POW" );

        AddField( &pDoc->m_Investigator.m_nDEX, rcStats.left+50,     rcStats.left+100, nTextY-40, "DEX" );
        AddField( &pDoc->m_Investigator.m_nAPP, rcStats.left+55+120, rcStats.left+230, nTextY-40, "APP" );
        AddField( &pDoc->m_Investigator.m_nEDU, rcStats.left+55+370, rcStats.left+475, nTextY-40, "EDU" );
    }

    if( pDoc->m_Investigator.m_nPOW > 0 ) //don't display 99-Cthulhu Mythos if Pow == -1
    {
        CInvestigator::InvestigatorSkillList* pCthulhuMythos = pDoc->m_Investigator.FindSkill( "Cthulhu Mythos" );
        DrawField( pDC, "99 - Cthulhu Mythos",  rcStats.left, rcStats.left+300, nTextY-85, (pCthulhuMythos ? (99-pCthulhuMythos->nCurrent) : 99), 240, DT_CENTER );
    }
    else DrawField( pDC, "99 - Cthulhu Mythos", rcStats.left, rcStats.left+300, nTextY-85, "", 240, DT_CENTER );
    DrawField( pDC, "Damage Bonus:", rcStats.left+330, rcStats.left+600, nTextY-85, GetDBString( pDoc->m_Investigator.m_nSTR, pDoc->m_Investigator.m_nSIZ ), 190, DT_CENTER );


    /******************************************************************************************************/
    /** SANITY BOX ****************************************************************************************/
    /******************************************************************************************************/

    CRect rcSan = m_rcSan;
    rcSan.top -= 65;
    rcSan.left += 20;

    //create the table fonts
    CFont TableTextFont;
    TableTextFont.CreateFont( -18, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );

    CFont TableFontBold;
    TableFontBold.CreateFont( -18, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TableFontBold );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    pDC->TextOut( rcSan.left, rcSan.top, "Insanity 0" );
    int nXStep = 33;
    int nYStep = 40;
    int nXStart = 3;

    //draw the table
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    pDC->SelectObject( &TableTextFont );
    rcSan.left += pDC->GetTextExtent("8").cx;

    int nCount = 1;
    int nSAN = pDoc->m_Investigator.m_nSAN;
    if( pDoc->m_Investigator.m_nSAN == -999 && pDoc->m_Investigator.m_nPOW > 0 ) nSAN = pDoc->m_Investigator.m_nSAN = pDoc->m_Investigator.m_nPOW*5;

    for( int nTableY = 0; nTableY < 6; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 17; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nSAN && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcSan.left, (rcSan.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }

            pDC->TextOut( (nTableX*nXStep)+rcSan.left, (rcSan.top-((nTableY*nYStep))), str );

            nCount++;
        }

        nXStart = 0;
    }



    /******************************************************************************************************/
    /** HITPOINT BOX **************************************************************************************/
    /******************************************************************************************************/
    
    CRect rcHit = m_rcHit;
    rcHit.top -= 65;
    rcHit.left += 20;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TableFontBold );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    pDC->TextOut( rcHit.left, rcHit.top, "Dead" );
    nXStep = 30;
    nYStep = 40;
    nXStart = 2;

    //draw the table
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    rcHit.left += pDC->GetTextExtent("8").cx;

    nCount = -2;
    nHP = pDoc->m_Investigator.m_nHitPoints;
    if( nHP == -999 )
    {
        if( pDoc->m_Investigator.m_nCON > 0 && pDoc->m_Investigator.m_nSIZ > 0 )
            nHP = pDoc->m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)pDoc->m_Investigator.m_nCON + (double)pDoc->m_Investigator.m_nSIZ ) / 2.0 ) );
    }
    for( nTableY = 0; nTableY < 3; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 8; nTableX++ )
        {
            CString str;
            if( nTableY == 0 && nCount > 0 && nCount < 3 ) str.Format( "+%d", nCount ); else str.Format( "%d", nCount );

            if( nCount == nHP && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcHit.left, (rcHit.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }

            nCount++;
            pDC->TextOut( (nTableX*nXStep)+rcHit.left, (rcHit.top-((nTableY*nYStep))), str );
        }

        //turn off bold font after first line
        if( nTableY == 0 ) pDC->SelectObject( &TableTextFont );
        nXStart = 0;
    }

    //draw the 'Unconscious' text
    pDC->SetTextAlign( TA_RIGHT|TA_TOP );
    CFont UnconsciousFont;
    UnconsciousFont.CreateFont( -10, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    pDC->SelectObject( &UnconsciousFont );
    pDC->TextOut( rcHit.right-45, rcHit.top+30, "UNCONSCIOUS" );

    /******************************************************************************************************/
    /** MAGIC BOX******************************************************************************************/
    /******************************************************************************************************/
    //draw the caption
    CRect rcMagic = m_rcMagic;
    rcMagic.top -= 65;
    rcMagic.left += 20;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TableFontBold );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    int nTextHeight = pDC->GetTextExtent( "8" ).cy;
    pDC->TextOut( rcMagic.left, rcMagic.top, "Unconscious" );
    nXStep = 32;
    nYStep = 40;
    nXStart = 4;

    //draw the table
    pDC->SelectObject( &TableTextFont );
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    rcMagic.left += pDC->GetTextExtent("8").cx;
    pDC->SelectObject( &TableFontBold );

    nCount = 0;
    int nMagic = pDoc->m_Investigator.m_nMagic;
    if( pDoc->m_Investigator.m_nMagic == -999 && pDoc->m_Investigator.m_nPOW > 0 ) nMagic = pDoc->m_Investigator.m_nMagic = pDoc->m_Investigator.m_nPOW;
    for( nTableY = 0; nTableY < 3; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 8; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nMagic && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcMagic.left, (rcMagic.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }
            
            pDC->TextOut( (nTableX*nXStep)+rcMagic.left, (rcMagic.top-((nTableY*nYStep))), str );

            //change font to non-bold after 0
            if( nCount == 0 ) pDC->SelectObject( &TableTextFont );
            nCount++;
        }

        nXStart = 0;
    }

    /******************************************************************************************************/
    /** FIREARMS BOX **************************************************************************************/
    /******************************************************************************************************/
    CRect rcFire = m_rcFire;
    rcFire.top -= 55;
    int nDrawX = rcFire.left;
    int nDrawY = rcFire.top;

    //draw column headings
    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
   
    CFont CaptionText;
    CaptionText.CreateFont( -16, 0, 0, 0, FW_NORMAL, TRUE/*italic*/, 0, 0, 0, 0, 0, 0, 0, "Arial" );

    pDC->SelectObject( &CaptionText );
    pDC->TextOut( nDrawX + 18,  nDrawY, "Weapon"  );
    pDC->TextOut( nDrawX + 169, nDrawY, "Skill %"  );
    pDC->TextOut( nDrawX + 255, nDrawY, "Damage"  );
    pDC->TextOut( nDrawX + 350, nDrawY, "Range"  );
    pDC->TextOut( nDrawX + 460, nDrawY, "Shots/Rnd"  );

    //draw firearms
    nDrawY = rcFire.top-40;
    int nSkills = 0;
    CInvestigator::InvestigatorSkillList* pSkills = pDoc->m_Investigator.GetList();

    while( pSkills && nSkills < 3 )
    {
        if( pSkills->pSkill->GetType() == FIREARM )
        {
            //draw skill name
            DrawUserField( pDC, nDrawX+10, nDrawX+160, nDrawY, pSkills->pSkill->m_strName, DT_CENTER, TRUE, 30 );

            //draw current worthiness
            DrawUserField( pDC, nDrawX+172, nDrawX+240, nDrawY, pSkills->nCurrent, DT_CENTER, TRUE, 15 );

            //draw other stuff
            CFirearmSkill* pFirearm = (CFirearmSkill*)pSkills->pSkill;
            DrawUserField( pDC, nDrawX+255, nDrawX+345, nDrawY, pFirearm->m_strDmg, DT_CENTER, TRUE, 15 );
            DrawUserField( pDC, nDrawX+362, nDrawX+460, nDrawY, pFirearm->m_strRange, DT_CENTER, TRUE, 15 );
            DrawUserField( pDC, nDrawX+475, nDrawX+552, nDrawY, pFirearm->m_strShots, DT_CENTER, TRUE, 15 );

            //add skill rect
            if( !pDC->IsPrinting() ) 
            {
                AddField( &pSkills->pSkill->m_strName, nDrawX+10, nDrawX+160, nDrawY, "Weapon Name" );
                if( pSkills->pSkill->m_strBase[0] != '*' )
                    AddField( &pSkills->nCurrent, nDrawX+172, nDrawX+240, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );

                AddField( &pFirearm->m_strDmg,     nDrawX+255, nDrawX+345, nDrawY, "Weapon Damage" );
                AddField( &pFirearm->m_strRange,   nDrawX+362, nDrawX+460, nDrawY, "Weapon Range" );
                AddField( &pFirearm->m_strShots,   nDrawX+475, nDrawX+552, nDrawY, "Weapon Shots" );
            }

            //update drawX & Y
            nDrawY -= 35;
            nSkills++;
        }
        else if( pSkills->pSkill->GetType() == MELEE && pSkills->pSkill->m_strName[0] != '*' )
        {
            //draw skill name
            DrawUserField( pDC, nDrawX+10, nDrawX+160, nDrawY, pSkills->pSkill->m_strName, DT_CENTER, TRUE, 30 );

            //draw current worthiness
            DrawUserField( pDC, nDrawX+172, nDrawX+240, nDrawY, pSkills->nCurrent, DT_CENTER, TRUE, 15 );

            //draw other stuff
            CMeleeSkill* pMelee = (CMeleeSkill*)pSkills->pSkill;
            DrawUserField( pDC, nDrawX+255, nDrawX+345, nDrawY, pMelee->m_strDmg, DT_CENTER, TRUE, 15 );
            DrawUserField( pDC, nDrawX+362, nDrawX+460, nDrawY, "-",              DT_CENTER, TRUE, 15 );
            DrawUserField( pDC, nDrawX+475, nDrawX+552, nDrawY, pMelee->m_strAttacks, DT_CENTER, TRUE, 15 );

            //add skill rect
            if( !pDC->IsPrinting() ) 
            {
                AddField( &pSkills->pSkill->m_strName, nDrawX+10, nDrawX+160, nDrawY, "Weapon Name" );
                if( pSkills->pSkill->m_strBase[0] != '*' )
                    AddField( &pSkills->nCurrent, nDrawX+172, nDrawX+240, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );

                AddField( &pMelee->m_strDmg,     nDrawX+255, nDrawX+345, nDrawY, "Weapon Damage" );
                AddField( &pMelee->m_strAttacks, nDrawX+475, nDrawX+552, nDrawY, "Weapon Shots" );
            }

            //update drawX & Y
            nDrawY -= 35;
            nSkills++;
        }

        pSkills = pSkills->next;
    }

    //draw blank lines
    int nBlank;
    for( nBlank = nSkills; nBlank < 3; nBlank++ )
    {
        DrawUserField( pDC, nDrawX+18,  nDrawX+180, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+172, nDrawX+290, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+255, nDrawX+345, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+362, nDrawX+460, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+475, nDrawX+552, nDrawY, "", DT_LEFT );

        nDrawY -= 35;
    }

    /******************************************************************************************************/
    /** SKILLS BOX ****************************************************************************************/
    /******************************************************************************************************/
    //draw caption
    CRect rcSkill = m_rcSkill;
    rcSkill.top -= 20;
    rcSkill.bottom += 10;

    //draw skills
    CFont Text;
    Text.CreateFont( -20, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
    
    pDC->SelectObject( &Text );


    rcSkill.top -= 48;
    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    nDrawX = rcSkill.left;
    nDrawY = rcSkill.top;
    int nColumn = 0;
    nSkills = 0;
    int nBaseFirearms = 0;
    pSkills = pDoc->m_Investigator.GetList();
    while( pSkills && nColumn < 4 )
    {
        if( pSkills->pSkill->GetType() != ANYSKILL )
        {
            if( pSkills->pSkill->m_strBase[0] == 'F' )
                nBaseFirearms++;
            else
                switch( pSkills->pSkill->GetType() )
                {
                    case SKILL:
                    {
                        //draw skill check box
                        RECT rcCheck = { nDrawX+42, nDrawY+20+2, nDrawX+42+20, nDrawY+2 };
                        if( pSkills->pSkill->m_strBase[0] != '*' ) pDC->Rectangle( &rcCheck );

                        //draw skill name
                        pDC->SelectObject( &Text );
                        pDC->SetTextAlign( DT_BOTTOM|DT_LEFT );

                        const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                        while( *pszName == '*' ) pszName++;
                        CString strText = pszName;
                        strText += " (";
                        strText += GetSkillCode( pSkills->pSkill->m_strBase );
                        strText += ")";
                        pDC->TextOut( nDrawX + 80, nDrawY, strText );

                        //draw current worthiness
                        if( pSkills->nCurrent > -1 && ( pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE || pSkills->pSkill->IsCalculatedBase() == TRUE ) )
                            DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, pSkills->nCurrent, DT_LEFT );
                        else
                            DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, "", DT_LEFT );

                        //add skill rect
                        if( pSkills->pSkill->m_strBase[0] != '*' )
                            if( !pDC->IsPrinting() ) AddField( &pSkills->nCurrent, nDrawX+335, nDrawX+335+65, nDrawY, CString(pszName), pSkills->nMinimumValue );

                        //update drawX & Y
                        nDrawY -= 28;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            nDrawX += 380;
                        }

                        nSkills++;
                        break;
                    }

                    case SPECIALISESKILLITEM:
                    {
                        if( pSkills->pSkill->m_strName.GetLength() > 0 )
                        {
                            //draw skill check box
                            RECT rcCheck = { nDrawX+42, nDrawY+20+2, nDrawX+42+20, nDrawY+2 };
                            if( pSkills->pSkill->m_strBase[0] != '*' ) pDC->Rectangle( &rcCheck );

                            //draw skill subname and current worthiness
                            CSpecialiseSkillSkill* pSpecial = (CSpecialiseSkillSkill*)pSkills->pSkill;
                            CString strSkill = Abbreviate(pSpecial->m_pSkill->m_strName);
                            strSkill += " (";
                            strSkill += pSpecial->m_strName;
                            strSkill += ") ";
                            strSkill += " (";
                            strSkill += GetSkillCode( pSkills->pSkill->m_strBase );
                            strSkill += ")";
                            pDC->TextOut( nDrawX + 80, nDrawY, strSkill );

                            //draw current worthiness
                            if( pSkills->nCurrent > -1 && (pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE || pSkills->pSkill->IsCalculatedBase() == TRUE) && (pSkills->pSkill->m_strName.GetLength() > 0 || ((CSpecialiseSkillSkill*)pSkills->pSkill)->m_pSkill->m_strName.CompareNoCase( "Own Language" ) == 0 ) )
                                DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, pSkills->nCurrent, DT_LEFT );
                            else
                                DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, "", DT_LEFT );

                            //add fields
                            if( !pDC->IsPrinting() )
                            {
                                AddField( &pSkills->pSkill->m_strName, nDrawX+80, nDrawX+80+240, nDrawY, "Skill Name" );
                                if( pSkills->pSkill->m_strBase[0] != '*' )
                                    AddField( &pSkills->nCurrent, nDrawX+335, nDrawX+335+65, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );
                            }

                            //update drawX & Y
                            nDrawY -= 28;
                            if( nDrawY < rcSkill.bottom )
                            {
                                //column wrap
                                nColumn++;
                                nDrawY = rcSkill.top;
                                nDrawX += 380;
                            }

                            nSkills++;
                            break;
                        }
                    }
                }
        }

        pSkills = pSkills->next;
    }

    //draw blank lines
    for( nBlank = 0; nBlank < 55-nSkills-nBaseFirearms && nColumn < 4; nBlank++ )
    {
        //draw check box
        RECT rcCheck = { nDrawX+42, nDrawY+20+2, nDrawX+42+20, nDrawY+2 };
        pDC->Rectangle( &rcCheck );

        //draw lines
        DrawUserField( pDC, nDrawX+80, nDrawX+80+240, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, "", DT_LEFT );

        //next line...
        nDrawY -= 28;
        if( nDrawY < rcSkill.bottom )
        {
            //column wrap
            nColumn++;
            nDrawY = rcSkill.top;
            nDrawX += 380;
        }
    }

    if( nColumn < 4 )
    {
        //draw firearms caption
        CFont FirearmsText;
        FirearmsText.CreateFont( -20, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial" );
        pDC->SelectObject( &FirearmsText );
        pDC->TextOut( nDrawX + 80, nDrawY-2, "Firearms" );
        nDrawY -= 28;
        if( nDrawY < rcSkill.bottom )
        {
            //column wrap
            nColumn++;
            nDrawY = rcSkill.top;
            nDrawX += 380;
        }
    }

    //draw basic firearms
    pSkills = pDoc->m_Investigator.GetList();
    while( pSkills && nColumn < 4 )
    {
        if( pSkills->pSkill->GetType() != ANYSKILL )
        {
            if( pSkills->pSkill->m_strBase[0] == 'F' )
            {
                //draw skill check box
                RECT rcCheck = { nDrawX+42, nDrawY+20+2, nDrawX+42+20, nDrawY+2 };
                if( pSkills->pSkill->m_strBase[0] != '*' ) pDC->Rectangle( &rcCheck );

                //draw skill name
                pDC->SelectObject( &Text );
                pDC->SetTextAlign( DT_BOTTOM|DT_LEFT );

                const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                while( *pszName == '*' ) pszName++;
                CString strText = pszName;
                strText += " (";
                strText += GetSkillCode( LPCSTR(pSkills->pSkill->m_strBase)+1 );
                strText += ")";
                pDC->TextOut( nDrawX + 80, nDrawY, strText );

                //draw current worthiness
                if( pSkills->nCurrent > -1 && pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE )
                    DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, pSkills->nCurrent, DT_LEFT );
                else
                    DrawUserField( pDC, nDrawX+335, nDrawX+335+65, nDrawY, "", DT_LEFT );

                //add field
                if( pSkills->pSkill->m_strBase[0] != '*' )
                    if( !pDC->IsPrinting() ) AddField( &pSkills->nCurrent, nDrawX+335, nDrawX+335+65, nDrawY, CString(pszName), pSkills->nMinimumValue );

                //update drawX & Y
                nDrawY -= 28;
                if( nDrawY < rcSkill.bottom )
                {
                    //column wrap
                    nColumn++;
                    nDrawY = rcSkill.top;
                    nDrawX += 380;
                }
            }
        }

        pSkills = pSkills->next;
    }

    pDC->SelectObject( pOldFont );
}

void CCthulhuView::DrawPortrait( CDC* pDC )
{
    CCthulhuDoc* pDoc = GetDocument();
    ASSERT(pDoc);

    //FIXME!

    if( pDoc->m_pBitmap && pDoc->m_uHeight > 0 && pDoc->m_uWidth > 0 )
    {
        CRect rcPic = m_rcPic;

        pDC->SelectPalette( CPalette::FromHandle(pDoc->m_hPalPortrait), FALSE );
        pDC->RealizePalette();

        //shrink the portrait box to leave margins
        rcPic.left += 10;
        rcPic.right -= 10;
        rcPic.top -= 10;
        rcPic.bottom += 10;

        //scale picture to fill gap
        int nBoxWidth = rcPic.Width();
        int nBoxHeight = rcPic.Height();

        double dScale = 1.0;
        if( pDoc->m_uWidth > pDoc->m_uHeight ) //scale to smallest dimension
            dScale = double(nBoxWidth) / double(pDoc->m_uWidth);
        else
            dScale = double(nBoxHeight) / double(pDoc->m_uHeight);

        int nScaleWidth = int( double(pDoc->m_uWidth) * dScale );
        int nScaleHeight = -int( double(pDoc->m_uHeight) * dScale );

        int nPicMarginX = (nBoxWidth-nScaleWidth) / 2;
        int nPicMarginY = (nBoxHeight-nScaleHeight) / 2;

        //create a device to draw the bitmap into
        CDC DCPortrait;
        DCPortrait.CreateCompatibleDC( pDC );
        DCPortrait.SelectPalette( CPalette::FromHandle(pDoc->m_hPalPortrait), FALSE );
        DCPortrait.RealizePalette();

        //create the device-dependant bitmap
        HBITMAP hbmCurrent;
        LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)pDoc->m_pBitmap;
        hbmCurrent = CreateDIBitmap( pDC->m_hDC, (LPBITMAPINFOHEADER)lpbi, (LONG)CBM_INIT, (LPSTR)lpbi + lpbi->biSize + (lpbi->biClrUsed * sizeof(RGBQUAD)), (LPBITMAPINFO)lpbi, DIB_RGB_COLORS );
        DCPortrait.SelectObject( hbmCurrent );

        //stretch it into the box
        pDC->StretchBlt( rcPic.left+nPicMarginX, rcPic.top+nPicMarginY, nScaleWidth, nScaleHeight, &DCPortrait, 0, 0, pDoc->m_uWidth, pDoc->m_uHeight, SRCCOPY );

        //release the bitmap
        DeleteObject( hbmCurrent );
    }
}



void CCthulhuView::DrawDeltaGreenCharSheet( CDC* pDC  )
{
	CCthulhuDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

    //set text background mode
    pDC->SetBkMode( TRANSPARENT );

    /******************************************************************************************************/
    /** OUTLINE OF WHOLE SHEET ****************************************************************************/
    /******************************************************************************************************/
    CPen FatPen( PS_SOLID, 10, RGB(0,0,0)/*black*/ );
    CPen* pPenOld = pDC->SelectObject( &FatPen );
    pDC->Rectangle( m_rcEra.left, m_rcEra.top, m_rcFire.right, m_rcFire.bottom );

    /******************************************************************************************************/
    /** TABLE BOXES ***************************************************************************************/
    /******************************************************************************************************/

    CFont TitleFont;
    TitleFont.CreateFont( -68, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, m_strDeltaGreenFace );
    CFont* pOldFont = pDC->SelectObject( &TitleFont );
    pDC->SetTextColor( 0xFFFFFF /*white*/ );

    //draw border
    CRect rcCombatSkills( m_rcHand.left, m_rcHand.top, m_rcFire.right, m_rcFire.bottom );
    CRect rcCaption = m_rcSan;              pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 80;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcHit;                    pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 80;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcMagic;                  pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 80;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcHand;                   pDC->Rectangle( &rcCaption );
    rcCaption = m_rcFire;                   pDC->Rectangle( &rcCaption );
    rcCaption = rcCombatSkills;             
    rcCaption.bottom = rcCaption.top - 80;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcSkill;                  pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 80;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );
    rcCaption = m_rcStat;                   pDC->Rectangle( &rcCaption );
    rcCaption.bottom = rcCaption.top - 80;  pDC->FillRect( rcCaption, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH)) );

    //fixme: white lines between stat, magic & hit?
    pDC->MoveTo( m_rcSkill.left+640, m_rcSkill.top - 80 ); pDC->LineTo( m_rcSkill.left+640, m_rcSkill.bottom );
    pDC->MoveTo( m_rcSkill.left+1270, m_rcSkill.top - 80 ); pDC->LineTo( m_rcSkill.left+1270, m_rcSkill.bottom );


    //caption text
    pDC->DrawText( "SANITY POINTS AND MENTAL HEALTH", &m_rcSan,    DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "HIT POINTS",          &m_rcHit,    DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "MAGIC POINTS",        &m_rcMagic,  DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "COMBAT SKILLS",       &rcCombatSkills, DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "INVESTIGATOR SKILLS", &m_rcSkill,  DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );
    pDC->DrawText( "INVESTIGATOR STATISTICS", m_rcStat,DT_CENTER|DT_SINGLELINE|DT_TOP|DT_NOCLIP|DT_NOPREFIX );

    //put old, thin pen back
    pDC->SelectObject( pPenOld );
    

    /******************************************************************************************************/
    /** DELTA GREEN LOGO **********************************************************************************/
    /******************************************************************************************************/

    CRect rcLogo = m_rcEra;
    //draw logo background
    pDC->FillRect( &rcLogo, CBrush::FromHandle((HBRUSH)GetStockObject(BLACK_BRUSH) ) );

    //draw the delta green logo
    rcLogo.left += 10;
    rcLogo.right -= 10;
    rcLogo.top -= 10;
    rcLogo.bottom += 10;
    PlayEnhMetaFile( pDC->m_hDC, m_hEmfDeltaGreen, &rcLogo );


    /******************************************************************************************************/
    /** NAME BOX ******************************************************************************************/
    /******************************************************************************************************/
    //draw rectangle
    CRect rcName = m_rcName;
    pDC->Rectangle( &rcName );

    //draw text & lines
    rcName.top -= 80;
    rcName.left += 15;
    rcName.right -= 15;

    
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    
    int nTextY = rcName.top;
    CFont TextFont;
    TextFont.CreateFont( -35, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );
    pDC->SelectObject( &TextFont );

    rcName.top -= 75; 
    DrawField( pDC, "Name", rcName.left, rcName.left+760, nTextY, pDoc->m_Investigator.m_strName, 95, DT_LEFT );
    DrawField( pDC, "Occupation",        rcName.left+760, rcName.right, nTextY, pDoc->m_Investigator.m_strOccupation, 170, DT_LEFT );
    if( !pDC->IsPrinting() ) 
    {
        AddField( &pDoc->m_Investigator.m_strName, rcName.left+95, rcName.left+760, nTextY, "Name" );
        AddField( &pDoc->m_Investigator.m_strOccupation, rcName.left+760+170, rcName.right, nTextY, "Occupation" );
    }
    nTextY -= 85;
    DrawField( pDC, "Sex",               rcName.left, rcName.left+150, nTextY, pDoc->m_Investigator.m_strSex, 50, DT_LEFT );
    DrawField( pDC, "Age",               rcName.left+150, rcName.left+325, nTextY, pDoc->m_Investigator.m_nAge, 60,DT_LEFT );
    DrawField( pDC, "Nationality",       rcName.left+325, rcName.left+860, nTextY, pDoc->m_Investigator.m_strBirthplace, 160, DT_LEFT );
    DrawField( pDC, "Affiliation",       rcName.left+860, rcName.right, nTextY, pDoc->m_Investigator.m_strDG_Affiliation, 150, DT_LEFT );
    if( !pDC->IsPrinting() )
    {
        AddField( &pDoc->m_Investigator.m_strSex, rcName.left+50, rcName.left+150, nTextY, "Sex", 1 );
        AddField( &pDoc->m_Investigator.m_nAge, rcName.left+150+60, rcName.left+280, nTextY, "Age" ); //note: not +325 as spin button looked odd
        AddField( &pDoc->m_Investigator.m_strBirthplace, rcName.left+325+160, rcName.left+860, nTextY, "Nationality" );
        AddField( &pDoc->m_Investigator.m_strDG_Affiliation, rcName.left+860+150, rcName.right, nTextY, "Affiliation" );
    }
    nTextY -= 85; 
    DrawField( pDC, "Schools & Degrees", rcName.left, rcName.left+830, nTextY, pDoc->m_Investigator.m_strColleges, 270, DT_LEFT );
    DrawField( pDC, "Code Name", rcName.left+840, rcName.right, nTextY, pDoc->m_Investigator.m_strDG_CodeName, 170, DT_LEFT );
    if( !pDC->IsPrinting() ) 
    {
        AddField( &pDoc->m_Investigator.m_strColleges, rcName.left+270, rcName.left+830, nTextY, "Schools" );
        AddField( &pDoc->m_Investigator.m_strDG_CodeName, rcName.left+840+170, rcName.right, nTextY, "Code Name" );
    }



    
    /******************************************************************************************************/
    /** STATS BOX *****************************************************************************************/
    /******************************************************************************************************/
    //draw top of box
    CRect rcStats = m_rcStat;

    //draw stats
    //draw text & lines
    rcStats.top -= 80+8;
    rcStats.left += 20;

    DrawDGStatBox( pDC, rcStats.left, rcStats.top, "STR", &pDoc->m_Investigator.m_nSTR, TRUE );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-95, "CON", &pDoc->m_Investigator.m_nCON, TRUE );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-190, "SIZ", &pDoc->m_Investigator.m_nSIZ, TRUE );

    rcStats.left += 190;
    int nSAN = pDoc->m_Investigator.m_nPOW*5;
    DrawDGStatBox( pDC, rcStats.left, rcStats.top, "DEX", &pDoc->m_Investigator.m_nDEX, TRUE );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-95, "APP", &pDoc->m_Investigator.m_nAPP, TRUE );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-190, "SAN", &nSAN );

    rcStats.left += 190;
    DrawDGStatBox( pDC, rcStats.left, rcStats.top, "INT", &pDoc->m_Investigator.m_nINT, TRUE );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-95, "POW", &pDoc->m_Investigator.m_nPOW, TRUE );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-190, "EDU", &pDoc->m_Investigator.m_nEDU, TRUE );

    rcStats.left += 190;
    int nIdea = pDoc->m_Investigator.m_nINT*5;
    int nLuck = pDoc->m_Investigator.m_nPOW*5;
    int nKnow = pDoc->m_Investigator.m_nEDU*5;
    DrawDGStatBox( pDC, rcStats.left, rcStats.top, "Idea", &nIdea );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-95, "Luck", &nLuck );
    DrawDGStatBox( pDC, rcStats.left, rcStats.top-190, "Know", &nKnow );

    rcStats.left += 340;
    pDC->SetTextAlign( TA_CENTER|TA_BOTTOM );
    pDC->TextOut( rcStats.left, rcStats.top-60, "Damage Bonus" );
    pDC->TextOut( rcStats.left, rcStats.top-190, "Current Date" );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    DrawUserField( pDC, rcStats.left-95, rcStats.left+95, rcStats.top-120, GetDBString( pDoc->m_Investigator.m_nSTR, pDoc->m_Investigator.m_nSIZ ), DT_CENTER ); 
    DrawUserField( pDC, rcStats.left-95, rcStats.left+95, rcStats.top-250, pDoc->m_Investigator.m_strDG_CurrentDate, DT_CENTER ); 
    AddField( &pDoc->m_Investigator.m_strDG_CurrentDate, rcStats.left-95, rcStats.left+95, rcStats.top-250, "Current Date" );



    /******************************************************************************************************/
    /** MAGIC BOX******************************************************************************************/
    /******************************************************************************************************/
    //draw the caption
    CRect rcMagic = m_rcMagic;
    rcMagic.top -= 80+40;
    rcMagic.left += 40;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    CFont TextFontBold;
    TextFontBold.CreateFont( -35, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );
    pDC->SelectObject( &TextFontBold );

    CFont TableFont;
    TableFont.CreateFont( -30, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, "Times New Roman" );

    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    int nTextHeight = pDC->GetTextExtent( "8" ).cy;
    pDC->TextOut( rcMagic.left, rcMagic.top, "Unconscious=" );
    int nXStep = 73;
    int nYStep = 48;
    int nXStart = 3;

    //draw the table

    int nCount = 0;
    int nMagic = pDoc->m_Investigator.m_nMagic;
    if( pDoc->m_Investigator.m_nMagic == -999 && pDoc->m_Investigator.m_nPOW > 0 ) nMagic = pDoc->m_Investigator.m_nMagic = pDoc->m_Investigator.m_nPOW;
    for( int nTableY = 0; nTableY < 6; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 5; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nMagic && m_fShowText && pDoc->m_Investigator.m_nPOW >= 0 )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcMagic.left+(sz.cx/2), (rcMagic.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }
            
            pDC->TextOut( (nTableX*nXStep)+rcMagic.left, (rcMagic.top-((nTableY*nYStep))), str );

            //change font to non-bold after 0
            if( nCount == 0 ) pDC->SelectObject( &TableFont );
            nCount++;
        }

        nXStart = 0;
    }



    /******************************************************************************************************/
    /** HITPOINT BOX **************************************************************************************/
    /******************************************************************************************************/
    CRect rcHit = m_rcHit;
    rcHit.top -= 80+40;
    rcHit.left += 40;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );
    pDC->SelectObject( &TextFontBold );

    pDC->TextOut( rcHit.left, rcHit.top, "Dead=" );
    nXStep = 73;
    nYStep = 48;
    nXStart = 2;

    nCount = 0;
    int nHP = pDoc->m_Investigator.m_nHitPoints;
    if( nHP == -999 )
    {
        if( pDoc->m_Investigator.m_nCON > 0 && pDoc->m_Investigator.m_nSIZ > 0 )
            nHP = pDoc->m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)pDoc->m_Investigator.m_nCON + (double)pDoc->m_Investigator.m_nSIZ ) / 2.0 ) );
    }
    for( nTableY = 0; nTableY < 6; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 5; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nHP && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcHit.left+(sz.cx/2), (rcHit.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }

            pDC->TextOut( (nTableX*nXStep)+rcHit.left, (rcHit.top-((nTableY*nYStep))), str );

            //change font to non-bold after 0
            if( nCount == 0 ) pDC->SelectObject( &TableFont );
            nCount++;

        }
        nXStart = 0;
    }


    /******************************************************************************************************/
    /** SANITY BOX ****************************************************************************************/
    /******************************************************************************************************/
    CRect rcSan = m_rcSan;
    rcSan.top -= 80+65;
    rcSan.left += 40;
    rcSan.right -= 50;

    //draw the table leading text
    pDC->SetTextColor( 0x000000 /*black*/ );

    pDC->SelectObject( &TextFont );
    if( pDoc->m_Investigator.m_nPOW > 0 ) //don't display 99-Cthulhu Mythos if Pow == -1
    {
        CInvestigator::InvestigatorSkillList* pCthulhuMythos = pDoc->m_Investigator.FindSkill( "Cthulhu Mythos" );
        DrawField( pDC, "(99-Cthulhu Mythos", rcSan.left, rcSan.left+375, rcSan.top, (pCthulhuMythos ? (99-pCthulhuMythos->nCurrent) : 99), 310, DT_CENTER );
    }
    else DrawField( pDC, "(99-Cthulhu Mythos", rcSan.left, rcSan.left+375, rcSan.top, "", 310, DT_CENTER );
    pDC->TextOut( rcSan.left+385, rcSan.top, ")" );
    
    pDC->SelectObject( &TextFontBold );

    pDC->TextOut( rcSan.left+450, rcSan.top, "Insanity=" );
    nXStep = 63;
    nYStep = 56;
    nXStart = 10;

    nCount = 0;
    nSAN = pDoc->m_Investigator.m_nSAN;
    if( pDoc->m_Investigator.m_nSAN == -999 && pDoc->m_Investigator.m_nPOW > 0 ) nSAN = pDoc->m_Investigator.m_nSAN = pDoc->m_Investigator.m_nPOW*5;

    for( nTableY = 0; nTableY < 4; nTableY++ )
    {
        for( int nTableX = nXStart; nTableX < 29; nTableX++ )
        {
            CString str;
            str.Format( "%d", nCount );

            if( nCount == nSAN && m_fShowText )
            {
                CPen Pen( PS_SOLID, TABLECIRCLE_PENSIZE, m_crUserText ), *pOldPen;
                pOldPen = pDC->SelectObject( &Pen );
                CSize sz = pDC->GetTextExtent(str);
                POINT pt = { (nTableX*nXStep)+rcSan.left+(sz.cx/2), (rcSan.top-((nTableY*nYStep)))+(sz.cy/2) };

                pDC->Ellipse( pt.x-TABLECIRCLE_SIZE, pt.y+TABLECIRCLE_SIZE, pt.x+TABLECIRCLE_SIZE, pt.y-TABLECIRCLE_SIZE );
                pDC->SelectObject( pOldPen );
            }

            pDC->TextOut( (nTableX*nXStep)+rcSan.left, (rcSan.top-((nTableY*nYStep))), str );

            //change font to non-bold after 0
            if( nCount == 0 ) pDC->SelectObject( &TableFont );
            nCount++;
            if( nCount == 100 ) break;
        }
        nXStart = 0;
        if( nCount == 100 ) break;
    }

    pDC->SelectObject( &TextFont );
    nTextY = rcSan.top-220;
    DrawField( pDC, "Phobias", rcSan.left, rcSan.left+870, nTextY, pDoc->m_Investigator.m_strMentalDisorders, 110, DT_LEFT );
    DrawField( pDC, "Insanities",        rcSan.left+890, rcSan.right, nTextY, pDoc->m_Investigator.m_strDG_Insanities, 130, DT_LEFT );
    if( !pDC->IsPrinting() ) 
    {
        AddField( &pDoc->m_Investigator.m_strMentalDisorders, rcSan.left+110, rcSan.left+870, nTextY, "Phobias" );
        AddField( &pDoc->m_Investigator.m_strDG_Insanities, rcSan.left+890+130, rcSan.right, nTextY, "Insanities" );
    }



    
    /******************************************************************************************************/
    /** SKILLS BOX ****************************************************************************************/
    /******************************************************************************************************/
    //draw caption
    CRect rcSkill = m_rcSkill;
    rcSkill.top -= 80;
    rcSkill.bottom += 30;

    //draw skills
    pDC->SelectObject( &TextFont );


    rcSkill.top -= 40;
    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );
    int nDrawX = rcSkill.left+40, nDrawY = rcSkill.top;
    int nColumn = 0;
    int nSkills = 0;
    CInvestigator::InvestigatorSkillList* pSkills = pDoc->m_Investigator.GetList();
    while( pSkills && nColumn < 3 )
    {
        if( pSkills->pSkill->GetType() != ANYSKILL )
        {
            if( pSkills->pSkill->m_strBase[0] != 'F' )
                switch( pSkills->pSkill->GetType() )
                {
                    case SKILL:
                    {
                        //draw skill check box
                        if( pSkills->pSkill->m_strBase[0] != '*' ) DrawDGCheckBox( pDC, nDrawX, nDrawY );

                        //draw skill name
                        pDC->SelectObject( &TextFont );
                        pDC->SetTextAlign( DT_BOTTOM|DT_LEFT );

                        const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                        while( *pszName == '*' ) pszName++;
                        CString strText = pszName;
                        strText += " (";
                        strText += GetSkillCode( pSkills->pSkill->m_strBase );
                        strText += ")";
                        pDC->TextOut( nDrawX + 45, nDrawY, strText );

                        //draw current worthiness
                        if( pSkills->nCurrent > -1 && ( pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE || pSkills->pSkill->IsCalculatedBase() == TRUE ) )
                            DrawUserField( pDC, nDrawX+445, nDrawX+445+130, nDrawY, pSkills->nCurrent, DT_LEFT );
                        else
                            DrawUserField( pDC, nDrawX+445, nDrawX+445+130, nDrawY, "", DT_LEFT );

                        //add skill rect
/* commenting out allows editing of cthulhu mythos et al
                        if( pSkills->pSkill->m_strBase[0] != '*' ) */
                            if( !pDC->IsPrinting() ) AddField( &pSkills->nCurrent, nDrawX+445, nDrawX+445+130, nDrawY, CString(pszName), pSkills->nMinimumValue );

                        //update drawX & Y
                        nDrawY -= 42;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            nDrawX += 635;
                        }

                        nSkills++;
                        break;
                    }

                    case SPECIALISESKILL:
                    {
                        //draw skill name
                        pDC->SelectObject( &TextFont );

                        const char * pszName = LPCSTR(pSkills->pSkill->m_strName);
                        while( *pszName == '*' ) pszName++;
                        CString strText = pszName;
                        strText += " (";
                        strText += GetSkillCode( pSkills->pSkill->m_strBase );
                        strText += "):";
                        pDC->TextOut( nDrawX + 45, nDrawY, strText );
                        nDrawY -= 42;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            nDrawX += 635;
                        }
                        nSkills++;
                        break;
                    }

                    case SPECIALISESKILLITEM:
                    {
                        //draw skill check box
                        if( pSkills->pSkill->m_strBase[0] != '*' ) DrawDGCheckBox( pDC, nDrawX, nDrawY );

                        //draw skill subname and current worthiness
                        DrawUserField( pDC, nDrawX+45, nDrawX+350, nDrawY, pSkills->pSkill->m_strName, DT_LEFT );

                        if( pSkills->nCurrent > -1 && (pDoc->m_Investigator.IsBase( pSkills->nCurrent, pSkills->pSkill ) == FALSE || pSkills->pSkill->IsCalculatedBase() == TRUE) && (pSkills->pSkill->m_strName.GetLength() > 0 || ((CSpecialiseSkillSkill*)pSkills->pSkill)->m_pSkill->m_strName.CompareNoCase( "Own Language" ) == 0 ) )
                            DrawUserField( pDC, nDrawX+445, nDrawX+445+130, nDrawY, pSkills->nCurrent, DT_LEFT );
                        else
                            DrawUserField( pDC, nDrawX+445, nDrawX+445+130, nDrawY, "", DT_LEFT );

                        //add fields
                        if( !pDC->IsPrinting() )
                        {
                            AddField( &pSkills->pSkill->m_strName, nDrawX+45, nDrawX+350+80, nDrawY, "Skill Name" );
                            if( pSkills->pSkill->m_strBase[0] != '*' )
                                AddField( &pSkills->nCurrent, nDrawX+445, nDrawX+445+130, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );
                        }

                        //update drawX & Y
                        nDrawY -= 42;
                        if( nDrawY < rcSkill.bottom )
                        {
                            //column wrap
                            nColumn++;
                            nDrawY = rcSkill.top;
                            nDrawX += 635;
                        }

                        nSkills++;
                        break;
                    }
                }
        }

        pSkills = pSkills->next;
    }

    //draw blank lines
    for( int nBlank = 0; nBlank < 75-nSkills; nBlank++ )
    {
        //draw check box
        DrawDGCheckBox( pDC, nDrawX, nDrawY );

        //draw lines
        DrawUserField( pDC, nDrawX+45, nDrawX+350, nDrawY, "", DT_LEFT );
        DrawUserField( pDC, nDrawX+445, nDrawX+445+130, nDrawY, "", DT_LEFT );

        //next line...
        nDrawY -= 42;
        if( nDrawY < rcSkill.bottom )
        {
            //column wrap
            nColumn++;
            nDrawY = rcSkill.top;
            nDrawX += 635;
        }
    }


    /******************************************************************************************************/
    /** COMBAT SKILLS *************************************************************************************/
    /******************************************************************************************************/

    pDC->SetTextColor(0x000000/*black*/);
    pDC->SetTextAlign( TA_LEFT|TA_BOTTOM );

    pSkills = pDoc->m_Investigator.GetList();

    CRect rcCombat = m_rcFire;
    for( int iCombatIndex = 0; iCombatIndex < 2; iCombatIndex++ )
    {
        int nDrawX = rcCombat.left+30;
        int nDrawY = rcCombat.top-35-80;

        //draw headings
        pDC->SelectObject( &TextFont );
        pDC->TextOut( nDrawX + 40,  nDrawY, "Weapon" );
        pDC->TextOut( nDrawX + 220, nDrawY, "Shots" );
        pDC->TextOut( nDrawX + 330, nDrawY, "Attk%" );
        pDC->TextOut( nDrawX + 460, nDrawY, "Impale" );
        pDC->TextOut( nDrawX + 590, nDrawY, "Damage" );
        pDC->TextOut( nDrawX + 730, nDrawY, "HP" );
        pDC->TextOut( nDrawX + 810, nDrawY, "Ammo" );

        //draw firearms
        nDrawY -= 55;
        int nSkills = 0;

        while( pSkills && nSkills < 3 )
        {
            if( pSkills->pSkill->GetType() == FIREARM )
            {
                CFirearmSkill* pFirearm = (CFirearmSkill*)pSkills->pSkill;

                //draw check box
                DrawDGCheckBox( pDC, nDrawX, nDrawY );

                //draw skill name
                DrawUserField( pDC, nDrawX+45, nDrawX+170, nDrawY, pSkills->pSkill->m_strName, DT_LEFT, TRUE, 30 );

                //draw shots
                DrawUserField( pDC, nDrawX+205, nDrawX+280, nDrawY, pFirearm->m_strShots, DT_CENTER, TRUE, 15 );

                //draw current worthiness
                DrawUserField( pDC, nDrawX+320, nDrawX+410, nDrawY, pSkills->nCurrent, DT_CENTER, TRUE, 15 );

                //draw impale (FIXME: WTF is this???)
                DrawUserField( pDC, nDrawX+445, nDrawX+540, nDrawY, "" , DT_CENTER, TRUE, 15 );

                //draw damage
                DrawUserField( pDC, nDrawX+570, nDrawX+685, nDrawY, pFirearm->m_strDmg, DT_CENTER, TRUE, 15 );

                //draw HP
                DrawUserField( pDC, nDrawX+715, nDrawX+765, nDrawY, pFirearm->m_strHP, DT_CENTER, TRUE, 15 );

                //draw ammo
                DrawUserField( pDC, nDrawX+795, nDrawX+890, nDrawY, pFirearm->m_strBullets, DT_CENTER, TRUE, 15 );

                //add skill rect
                if( !pDC->IsPrinting() ) 
                {
                    AddField( &pSkills->pSkill->m_strName, nDrawX+25, nDrawX+170, nDrawY, "Weapon Name" );
                    AddField( &pFirearm->m_strShots,   nDrawX+205, nDrawX+280, nDrawY, "Weapon Shots" );
                    if( pSkills->pSkill->m_strBase[0] != '*' )
                        AddField( &pSkills->nCurrent, nDrawX+320, nDrawX+410, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );
                    //FIXME: impale?!?!
                    AddField( &pFirearm->m_strDmg,     nDrawX+570, nDrawX+685, nDrawY, "Weapon Damage" );
                    AddField( &pFirearm->m_strHP,      nDrawX+715, nDrawX+765, nDrawY, "Weapon Hit Points" );
                    AddField( &pFirearm->m_strBullets, nDrawX+795, nDrawX+890, nDrawY, "Weapon Ammo" );
                }

                //update drawX & Y
                nDrawY -= 58;
                nSkills++;
            }
            else if( pSkills->pSkill->GetType() == MELEE && pSkills->pSkill->m_strName[0] != '*' )
            {
                CMeleeSkill* pMelee = (CMeleeSkill*)pSkills->pSkill;

                //draw check box
                DrawDGCheckBox( pDC, nDrawX, nDrawY );

                //draw skill name
                DrawUserField( pDC, nDrawX+45, nDrawX+170, nDrawY, pSkills->pSkill->m_strName, DT_LEFT, TRUE, 30 );

                //draw attacks (shots)
                DrawUserField( pDC, nDrawX+205, nDrawX+280, nDrawY, pMelee->m_strAttacks, DT_CENTER, TRUE, 15 );

                //draw current worthiness
                DrawUserField( pDC, nDrawX+320, nDrawX+410, nDrawY, pSkills->nCurrent, DT_CENTER, TRUE, 15 );

                //draw impale (FIXME: WTF is this???)
                DrawUserField( pDC, nDrawX+445, nDrawX+540, nDrawY, "" , DT_CENTER, TRUE, 15 );

                //draw damage
                DrawUserField( pDC, nDrawX+570, nDrawX+685, nDrawY, pMelee->m_strDmg, DT_CENTER, TRUE, 15 );

                //draw HP
                DrawUserField( pDC, nDrawX+715, nDrawX+765, nDrawY, pMelee->m_strHP, DT_CENTER, TRUE, 15 );

                //draw ammo
                DrawUserField( pDC, nDrawX+795, nDrawX+890, nDrawY, "-",              DT_CENTER, TRUE, 15 );

                //add skill rect
                if( !pDC->IsPrinting() ) 
                {
                    AddField( &pSkills->pSkill->m_strName, nDrawX+25, nDrawX+170, nDrawY, "Weapon Name" );
                    AddField( &pMelee->m_strAttacks, nDrawX+205, nDrawX+280, nDrawY, "Weapon Shots" );
                    if( pSkills->pSkill->m_strBase[0] != '*' )
                        AddField( &pSkills->nCurrent, nDrawX+320, nDrawX+410, nDrawY, pSkills->pSkill->m_strName, pSkills->nMinimumValue );
                    AddField( &pMelee->m_strDmg,      nDrawX+570, nDrawX+685, nDrawY, "Weapon Damage" );
                    AddField( &pMelee->m_strHP,       nDrawX+715, nDrawX+765, nDrawY, "Weapon Hit Points" );
                }

                //update drawX & Y
                nDrawY -= 58;
                nSkills++;
            }

            pSkills = pSkills->next;
        }

        //draw blank lines
        int nBlank;
        for( nBlank = nSkills; nBlank < 3; nBlank++ )
        {
            DrawUserField( pDC, nDrawX+25,  nDrawX+170, nDrawY, "", DT_CENTER, TRUE );
            DrawUserField( pDC, nDrawX+205, nDrawX+280, nDrawY, "", DT_CENTER, TRUE );
            DrawUserField( pDC, nDrawX+320, nDrawX+410, nDrawY, "", DT_CENTER, TRUE );
            DrawUserField( pDC, nDrawX+445, nDrawX+540, nDrawY, "" ,DT_CENTER, TRUE );
            DrawUserField( pDC, nDrawX+570, nDrawX+685, nDrawY, "", DT_CENTER, TRUE );
            DrawUserField( pDC, nDrawX+715, nDrawX+765, nDrawY, "", DT_CENTER, TRUE );
            DrawUserField( pDC, nDrawX+795, nDrawX+890, nDrawY, "", DT_CENTER, TRUE );

            nDrawY -= 58;
        }

        //now repeat it with the other box
        rcCombat = m_rcHand;
    }





    pDC->SelectObject( pOldFont );
}

void CCthulhuView::DrawDGStatBox(CDC * pDC, int x, int y, CString strCaption, int* nValue, BOOL fEdit /*FALSE*/ )
{
    //draw the box
    pDC->MoveTo( x, y-20 );
    pDC->LineTo( x, y-65 );
    pDC->LineTo( x+85, y-65 );
    pDC->LineTo( x+85, y-85 );
    pDC->LineTo( x+180, y-85 );
    pDC->LineTo( x+180, y );
    pDC->LineTo( x+85, y );
    pDC->LineTo( x+85, y-20 );
    pDC->LineTo( x, y-20 );

    //draw the caption
    pDC->SetTextColor( 0x000000 /*white*/ );
    pDC->SetTextAlign( TA_LEFT|TA_TOP );
    pDC->TextOut( x+10, y-25, strCaption );

    //draw the current value
    DrawUserField( pDC, x+75, x+170, y-65, *nValue, DT_CENTER|DT_BOTTOM, FALSE ); 

    //add field
    if( fEdit && pDC->IsPrinting() == FALSE ) 
        AddField( nValue, x+85, x+145, y-55, strCaption );
}

void CCthulhuView::DrawDGCheckBox(CDC * pDC, int x, int y)
{
    RECT rc = { x, y+20, x+20, y };
    pDC->FillRect( &rc, CBrush::FromHandle( (HBRUSH)GetStockObject(BLACK_BRUSH) ) );

    RECT rc2 = { x-5, y+25, x+15, y+5 };
    pDC->Rectangle( &rc2 );
}


void CCthulhuView::DrawField(CDC * pDC, CString strName, int x1, int x2, int y, int nText, int nTextOffset, UINT uExtraFormat )
{
    TEXTMETRIC tm;
    pDC->GetTextMetrics( &tm );
    
    pDC->TextOut( x1, y-(tm.tmDescent/2), strName );
    DrawUserField( pDC, x1+nTextOffset, x2, y, nText, uExtraFormat );
}

void CCthulhuView::DrawField(CDC * pDC, CString strName, int x1, int x2, int y, CString strText, int nTextOffset, UINT uExtraFormat )
{
    TEXTMETRIC tm;
    pDC->GetTextMetrics( &tm );
    
    pDC->TextOut( x1, y-(tm.tmDescent/2), strName );
    DrawUserField( pDC, x1+nTextOffset, x2, y, strText, uExtraFormat );
}

void CCthulhuView::DrawUserField(CDC * pDC, int x1, int x2, int y, int nText, UINT uExtraFormat, BOOL fDrawLine /*TRUE*/, int nAllowLeft /*0*/  )
{
    if( nText < 0 )
        DrawUserField( pDC, x1, x2, y, "", uExtraFormat, fDrawLine, nAllowLeft );
    else
    {
        CString strText;
        strText.Format( "%d", nText );
        DrawUserField( pDC, x1, x2, y, strText, uExtraFormat, fDrawLine );
    }
}


void CCthulhuView::DrawUserField(CDC * pDC, int x1, int x2, int y, CString strText, UINT uExtraFormat, BOOL fDrawLine /*TRUE*/, int nAllowLeft /*0*/ )
{
    TEXTMETRIC tm;

    //draw the line
    if( fDrawLine ) 
    {
        pDC->MoveTo( x1 + 10, y ); pDC->LineTo( x2, y );
    }
    
    //see if there is text to draw as well
    if( strText.GetLength() == 0 || m_fShowText == FALSE ) return; 

    //set up text formatting rectangle
    CRect rcText;
    rcText.left = x1 + USERTEXT_GAP;
    rcText.right = x2;
    rcText.bottom = y;

    COLORREF crCurrent = pDC->GetTextColor();
    pDC->SetTextColor( m_crUserText );

    //find a font that will fit
    CFont UserTextFont;
    CFont* pOldFont;
    int nSize = -30;
    while( nSize < -10 )
    {
        //select the new font into the device
        UserTextFont.DeleteObject();
        UserTextFont.CreateFont( nSize, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, m_strUserTextFace );
        pOldFont = pDC->SelectObject( &UserTextFont );

        //calculate drawing rectangle
        if( pDC->GetTextExtent( strText ).cx > (rcText.Width()+( ( uExtraFormat & DT_CENTER ) ? 36 : 10+nAllowLeft ) ) )  //3mm overhang
        {
            pDC->SelectObject( pOldFont );
            nSize++; //keep shrinking the font
        }
        else 
        {
            pDC->SelectObject( pOldFont );
            break;
        }
    }

    pDC->GetTextMetrics( &tm );
    rcText.top = rcText.bottom-tm.tmDescent;
    if( uExtraFormat & DT_CENTER ) 
        { rcText.right += 18; rcText.left -= 18; }
    else
        rcText.left -= nAllowLeft;

    UserTextFont.DeleteObject();
    UserTextFont.CreateFont( nSize, 0, 0, 0, FW_NORMAL, 0, 0, 0, 0, 0, 0, 0, 0, m_strUserTextFace );
    pOldFont = pDC->SelectObject( &UserTextFont );

    pDC->DrawText( strText, rcText, uExtraFormat|DT_TOP|DT_NOPREFIX|DT_NOCLIP|DT_END_ELLIPSIS|DT_SINGLELINE );

    pDC->SelectObject( pOldFont );
    pDC->SetTextColor( crCurrent );
}


/************************************************************************************
*************************************************************************************
*************************************************************************************
*************************************************************************************
************************************************************************************/


CCthulhuView::SheetHitTest CCthulhuView::BoxFromPoint( LPPOINT pt )
{
    if( pt->x > m_rcPic.left && pt->x < m_rcPic.right && pt->y < m_rcPic.top && pt->y > m_rcPic.bottom ) return HT_PIC;
    if( pt->x > m_rcEra.left && pt->x < m_rcEra.right && pt->y < m_rcEra.top && pt->y > m_rcEra.bottom ) return HT_ERA;
    if( pt->x > m_rcName.left && pt->x < m_rcName.right && pt->y < m_rcName.top && pt->y > m_rcName.bottom ) return HT_NAME;
    if( pt->x > m_rcStat.left && pt->x < m_rcStat.right && pt->y < m_rcStat.top && pt->y > m_rcStat.bottom ) return HT_STAT;
    if( pt->x > m_rcSide.left && pt->x < m_rcSide.right && pt->y < m_rcSide.top && pt->y > m_rcSide.bottom ) return HT_SIDE;
    if( pt->x > m_rcSan.left && pt->x < m_rcSan.right && pt->y < m_rcSan.top && pt->y > m_rcSan.bottom ) return HT_SAN;
    if( pt->x > m_rcMagic.left && pt->x < m_rcMagic.right && pt->y < m_rcMagic.top && pt->y > m_rcMagic.bottom ) return HT_MAGIC;
    if( pt->x > m_rcHit.left && pt->x < m_rcHit.right && pt->y < m_rcHit.top && pt->y > m_rcHit.bottom ) return HT_HIT;
    if( pt->x > m_rcSkill.left && pt->x < m_rcSkill.right && pt->y < m_rcSkill.top && pt->y > m_rcSkill.bottom ) return HT_SKILL;
    if( pt->x > m_rcHand.left && pt->x < m_rcHand.right && pt->y < m_rcHand.top && pt->y > m_rcHand.bottom ) return HT_HAND;
    if( pt->x > m_rcFire.left && pt->x < m_rcFire.right && pt->y < m_rcFire.top && pt->y > m_rcFire.bottom ) return HT_FIRE;

    return HT_NONE;
}


CRect CCthulhuView::RectFromBox(SheetHitTest ht)
{
    //determine area
    switch( ht )
    {
        case HT_ERA:    return m_rcEra;
        case HT_NAME:   return m_rcName;
        case HT_STAT:   return m_rcStat;
        case HT_SIDE:   return m_rcSide;
        case HT_SAN:    return m_rcSan;
        case HT_MAGIC:  return m_rcMagic;
        case HT_HIT:    return m_rcHit;
        case HT_SKILL:  return m_rcSkill;
        case HT_PIC:    return m_rcPic;
        case HT_HAND:   return m_rcHand;
        case HT_FIRE:   return m_rcFire;
    }

    //must be HT_NONE
    CRect rcEmpty( -1, 1, -1, 1 ); //off the top of the screen
    return rcEmpty;
}

void CCthulhuView::ViewToSheet(CPoint & point)
{
	CWindowDC dc(NULL);
	dc.SetMapMode(m_nMapMode);
	dc.DPtoLP( &point );

    CPoint ptScroll = GetScrollPosition();
    point.x += ptScroll.x;
    point.y += ptScroll.y;
}


void CCthulhuView::SheetToView(CPoint & point)
{
    CPoint ptScroll = GetScrollPosition();
    point.x -= ptScroll.x;
    point.y -= ptScroll.y;

    CWindowDC dc(NULL);
	dc.SetMapMode(m_nMapMode);
	dc.LPtoDP( &point );
}

void CCthulhuView::ViewToSheet(CRect & rect)
{
	CWindowDC dc(NULL);
	dc.SetMapMode(m_nMapMode);
	dc.DPtoLP( &rect );

    CPoint ptScroll = GetScrollPosition();
    rect.left += ptScroll.x;
    rect.right += ptScroll.x;
    rect.top += ptScroll.y;
    rect.bottom += ptScroll.y;
}

void CCthulhuView::SheetToView(CRect & rect)
{
    CPoint ptScroll = GetScrollPosition();
    rect.left -= ptScroll.x;
    rect.right -= ptScroll.x;
    rect.top -= ptScroll.y;
    rect.bottom -= ptScroll.y;

    CWindowDC dc(NULL);
	dc.SetMapMode(m_nMapMode);
	dc.LPtoDP( &rect );
}


/************************************************************************************
*************************************************************************************
*************************************************************************************
*************************************************************************************
************************************************************************************/


void CCthulhuView::OnLButtonUp(UINT nFlags, CPoint point) 
{
    ViewToSheet( point );
    FieldList* pField = FindField( &point );

    if( m_EditControl.Editing() ) m_EditControl.EndEditing( TRUE );

    if( pField )
    {
        BeginEditing( pField );
    }
    else
    {
        switch( BoxFromPoint( &point ) )
        {
            case HT_HIT:    GetDocument()->OnEditHitpointsEdit(); break;
            case HT_SAN:    GetDocument()->OnEditSanityEdit(); break;
            case HT_MAGIC:  GetDocument()->OnEditMagicEdit(); break;
            case HT_SIDE:   GetDocument()->OnEditPlayerinformation(); break;
            case HT_PIC:    GetDocument()->OnPortraitBrowse(); break;
        }
    }

	CScrollView::OnLButtonUp(nFlags, point);
    
}


void CCthulhuView::OnRButtonUp(UINT nFlags, CPoint point) 
{
    //stop editing
    if( m_EditControl.Editing() ) m_EditControl.EndEditing( TRUE );

    //call default handler
	CScrollView::OnRButtonUp(nFlags, point);

    //change focus
    ViewToSheet( point );

    SheetHitTest ht = BoxFromPoint( &point );

    //show a context menu
    CMenu* pMenuPopup = NULL;
    CMenu* pMenu = AfxGetMainWnd()->GetMenu();
    if( pMenu == NULL ) return;

    //find the edit menu
    CMenu* pMenuEdit = NULL;
	int iItem = pMenu->GetMenuItemCount();
	while (iItem--)
	{
		CMenu* pMenuPop = pMenu->GetSubMenu(iItem);
		if( pMenuPop != NULL )
		{
			int iItemMax = pMenuPop->GetMenuItemCount();
			for( int iItemPop = 0; iItemPop < iItemMax; iItemPop++ )
			{
				if( pMenuPop->GetMenuItemID(iItemPop) == ID_EDIT_UNDO )
                {
                    pMenuEdit = pMenuPop;
                    break;
                }
			}
		}
	}

    if( pMenuEdit == NULL ) return;

    CMenu NoMenu;

    switch( ht )
    {
        case HT_NONE:
        case HT_ERA:
            NoMenu.CreatePopupMenu();
            NoMenu.InsertMenu( 0, MF_GRAYED, 0, "(nothing to edit)" );
            pMenuPopup = &NoMenu;
            break;

        case HT_NAME:   pMenuPopup = pMenuEdit->GetSubMenu(3); break;
        case HT_STAT:   pMenuPopup = pMenuEdit->GetSubMenu(4); break;
        case HT_SAN:    pMenuPopup = pMenuEdit->GetSubMenu(5); break;
        case HT_MAGIC:  pMenuPopup = pMenuEdit->GetSubMenu(6); break;
        case HT_HIT:    pMenuPopup = pMenuEdit->GetSubMenu(7); break;
        case HT_SIDE:   pMenuPopup = pMenuEdit->GetSubMenu(8); break;
        case HT_PIC:    pMenuPopup = pMenuEdit->GetSubMenu(9); break;
        case HT_SKILL:  pMenuPopup = pMenuEdit->GetSubMenu(10); break;
        case HT_HAND:
        case HT_FIRE:   pMenuPopup = pMenuEdit->GetSubMenu(11); break;
    }

    if( pMenuPopup )
    {
        SheetToView( point );
        ClientToScreen( &point );
        pMenuPopup->TrackPopupMenu( TPM_LEFTALIGN|TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd() );
    }
    else
    {
        TRACE0("NULL popup menu!\n");
    }
}

void CCthulhuView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
    ViewToSheet( point );
    switch( BoxFromPoint( &point ) )
    {
        case HT_NONE:
        case HT_ERA:
            break;

        case HT_HIT:    break;//GetDocument()->OnEditHitPointsEdit(); break;
        case HT_SAN:    break;//(done on LButtonUp) GetDocument()->OnEditSanityEdit(); break;
        case HT_MAGIC:  break;//(done on LButtonUp) GetDocument()->OnEditMagicEdit(); break;
        case HT_NAME:   GetDocument()->OnEditGeneralinformation(); break;
        case HT_STAT:   GetDocument()->OnEditCharacteristics(); break;
        case HT_SIDE:   break;//(done on LButtonUp) GetDocument()->OnEditPlayerinformation(); break;
        case HT_PIC:    break;//(done on LButtonUp) GetDocument()->OnPortraitBrowse(); break;
        case HT_SKILL:  GetDocument()->OnEditSkills(); break;
        case HT_HAND:
        case HT_FIRE:   GetDocument()->OnAddWeapon(); break;
    }
	
	CScrollView::OnLButtonDblClk(nFlags, point);
}

BOOL CCthulhuView::OnEraseBkgnd(CDC* pDC) 
{
    CBrush br( GetSysColor( COLOR_APPWORKSPACE ));
    FillOutsideRect( pDC, &br );
    return TRUE;
}

void CCthulhuView::OnEditShowText() 
{
    m_fShowText = !m_fShowText;
    GetDocument()->m_fNeedRedrawRecalculation = TRUE;
    GetDocument()->UpdateAllViews(NULL);
}

void CCthulhuView::OnUpdateEditShowText(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_fShowText );
}


/************************************************************************************************************/


CCthulhuView::FieldList* CCthulhuView::FindField( LPPOINT lppt )
{
    FieldList* pList = m_pFieldList;

    if( pList )
    {
        do {
            if( lppt->x > pList->rc.left && lppt->x < pList->rc.right && lppt->y < pList->rc.top+20 && lppt->y >= pList->rc.bottom 
              ) return pList;

            pList = pList->next;
        } while( pList != m_pFieldList );
    }

    return NULL;
}


CCthulhuView::FieldList* CCthulhuView::FindField( int i )
{
    FieldList* pList = m_pFieldList;

    if( pList )
    {
        do {
            if( pList->iField == i ) return pList;
            pList = pList->next;
        } while( pList != m_pFieldList );
    }

    return NULL;
}



void CCthulhuView::OnMouseMove(UINT nFlags, CPoint point) 
{
    ViewToSheet( point );

    if( FindField( &point ) )
        SetCursor( AfxGetApp()->LoadStandardCursor( IDC_IBEAM ) );
    else
    {
        switch( BoxFromPoint( &point ) )
        {
            case HT_HIT:
            case HT_SAN:
            case HT_MAGIC:
            case HT_SIDE:
            case HT_PIC:
                SetCursor( AfxGetApp()->LoadCursor( IDC_HAND ) ); 
                break;
            default:
                SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) ); 
                break;
        }
    }

	CScrollView::OnMouseMove(nFlags, point);
}

BOOL CCthulhuView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
    if( nHitTest == HTCLIENT )
        //OnMouseMove handles the cursor if it's in the client area
        return TRUE;
    else
        return CScrollView::OnSetCursor(pWnd, nHitTest, message);
}

int CCthulhuView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CScrollView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
    //create the edit control
    if( !m_EditControl.Create( ES_NOHIDESEL|WS_CHILD|ES_AUTOHSCROLL|ES_LEFT, CRect(-1,-1,-1,-1), this, IDC_USERTEXTEDIT ) )
        AfxMessageBox( "Could not create child window control: m_EditControl!" );
    else
        m_EditControl.CreateFont( m_strUserTextFace, m_crUserText );
	
	return 0;
}


void CCthulhuView::OnEditCopy() 
{
    if( m_EditControl.Editing() )
        m_EditControl.SendMessage( WM_COPY );
}

void CCthulhuView::OnEditCut() 
{
    if( m_EditControl.Editing() )
        m_EditControl.SendMessage( WM_CUT );
}

void CCthulhuView::OnEditPaste() 
{
    if( m_EditControl.Editing() )
        m_EditControl.SendMessage( WM_PASTE );
}

void CCthulhuView::OnEditUndo() 
{
    if( m_EditControl.Editing() )
        m_EditControl.SendMessage( WM_UNDO );
    else
        GetDocument()->Undo();
}

void CCthulhuView::OnEditRedo() 
{
    if( m_EditControl.Editing() == FALSE )
        GetDocument()->Redo();
}

void CCthulhuView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
    if( m_EditControl.Editing() )
    {
        CMenu* pMenu = AfxGetMainWnd()->GetMenu();
	    pMenu->ModifyMenu( ID_EDIT_UNDO, MF_BYCOMMAND|MF_STRING, ID_EDIT_UNDO, "&Undo\tCtrl+Z" );

        pCmdUI->Enable( m_EditControl.CanUndo() );
    }
    else
    {
        GetDocument()->UpdateUndoRedoMenu();
        pCmdUI->Enable( GetDocument()->IsUndo()  );	
    }
}

void CCthulhuView::OnUpdateEditRedo(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable( GetDocument()->IsRedo() );	
}

void CCthulhuView::LoadSettings()
{
	m_crUserText = AfxGetApp()->GetProfileInt( "Sheet", "UserTextColour", RGB(0,0,128) );
	m_strUserTextFace = AfxGetApp()->GetProfileString( "Sheet", "UserTextFace", "Comic Sans MS" );
	m_strTitleFace = AfxGetApp()->GetProfileString( "Sheet", "TitleFace", "Times New Roman" );
    m_strDeltaGreenFace = AfxGetApp()->GetProfileString( "Sheet", "DeltaGreen", "Arial" );

    m_nTopMargin = AfxGetApp()->GetProfileInt( "Sheet", "TopMargin", 180 );
    m_iRotateType = AfxGetApp()->GetProfileInt( "Sheet", "RotateType", 0 );

    //reset it
    SetSheetSize();
}


            //set up all of the drawing regions (CoC5 example used below)
	        /*

            (0,0)                                                         (1880,0)
	          +---+--------------------------+------------------------------+       
	          |   |                          |                              |       
	          | A |            B             |                C             |       
	          |   |                          |                              |       A era
	          +---+--------------------------+----------------+-------------+       B name
	          |   |                          |                |             |       C stat
	          |   |            E             |        F       |      G      |       D side
	          |   |                          |                |             |       E san
	          |   +--------------------------+----------------+-------------+       F magic
	          |   |                                       +---------------+ |       G hit
	          |   |                                       |               | |       H skill
	          |   |                                       |               | |       I pic
	          |   |                                       |               | |       J hand
	          |   |                                       |      I        | |       K fire
	          |   |                   H                   |               | |
	          |   |                                       |               | |
	          | D |                                       |               | |
	          |   |                                       +---------------+ |
            ___________________________________________________________________
            ___________________________________________________________________
	          |   |                                                         |
              +---+---------------------+-----------------------------------+
              |                         |                                   |
              |           J             |                  K                |
              |                         |                                   |
              +-------------------------+-----------------------------------+
            (0,-2790)                                                     (1880, -2510)

	        */

void CCthulhuView::SetSheetSize()
{
            //note: all measurements are in 0.1 mm
    switch( m_DisplayType )
    {
        default:
        case DT_COC50:
            m_rcEra.  SetRect( MARGIN_X,                    -m_nTopMargin,                      MARGIN_X+170,       -m_nTopMargin-300 ); 
            m_rcName. SetRect( MARGIN_X+170+12,   -m_nTopMargin,                      MARGIN_X+1050,      -m_nTopMargin-300 );
            m_rcStat. SetRect( MARGIN_X+1050+12,  -m_nTopMargin,                      MARGIN_X+1880,      -m_nTopMargin-300 );
            m_rcSide. SetRect( MARGIN_X,                    -m_nTopMargin-300-12,     MARGIN_X+170,       -m_nTopMargin-1940 );
            m_rcSan.  SetRect( MARGIN_X+170+12,   -m_nTopMargin-300-12,     MARGIN_X+1050,      -m_nTopMargin-710 );
            m_rcMagic.SetRect( MARGIN_X+1050+12,  -m_nTopMargin-300-12,     MARGIN_X+1460,      -m_nTopMargin-710 );
            m_rcHit.  SetRect( MARGIN_X+1460+12,  -m_nTopMargin-300-12,     MARGIN_X+1880,      -m_nTopMargin-710 );
            m_rcSkill.SetRect( MARGIN_X+170+12,   -m_nTopMargin-710-12,     MARGIN_X+1880,      -m_nTopMargin-1940 );
            m_rcPic.  SetRect( MARGIN_X+1335,               -m_nTopMargin-765,                  MARGIN_X+1335+480,-m_nTopMargin-765-480 );
            m_rcHand. SetRect( MARGIN_X,                    -m_nTopMargin-1940-12,    MARGIN_X+715,       -m_nTopMargin-2510 );
            m_rcFire. SetRect( MARGIN_X+715+12,   -m_nTopMargin-1940-12,    MARGIN_X+1880,      -m_nTopMargin-2510 );
            break;

        case DT_DELTAGREEN:
            m_rcEra.  SetRect( MARGIN_X,                    -m_nTopMargin,                      MARGIN_X+480,       -m_nTopMargin-290 );
            m_rcName. SetRect( MARGIN_X+480,                -m_nTopMargin,                      MARGIN_X+1900,      -m_nTopMargin-290 );
            m_rcStat. SetRect( MARGIN_X,                    -m_nTopMargin-290,                  MARGIN_X+1100,      -m_nTopMargin-670 );
            m_rcSide. SetRectEmpty();
            m_rcSan.  SetRect( MARGIN_X,                    -m_nTopMargin-670,                  MARGIN_X+1900,      -m_nTopMargin-1060 );
            m_rcMagic.SetRect( MARGIN_X+1100,               -m_nTopMargin-290,                  MARGIN_X+1500,      -m_nTopMargin-670 );
            m_rcHit.  SetRect( MARGIN_X+1500,               -m_nTopMargin-290,                  MARGIN_X+1900,      -m_nTopMargin-670 );
            m_rcSkill.SetRect( MARGIN_X,                    -m_nTopMargin-1060,                 MARGIN_X+1900,      -m_nTopMargin-2220 );
            m_rcPic.  SetRectEmpty();
            m_rcHand. SetRect( MARGIN_X,                    -m_nTopMargin-2220,                 MARGIN_X+950,       -m_nTopMargin-2550 );
            m_rcFire. SetRect( MARGIN_X+950,                -m_nTopMargin-2220,                 MARGIN_X+1900,      -m_nTopMargin-2550 );
            break;

        //custom
        case DT_COC50SMALL:
            m_rcEra.   SetRectEmpty();
            m_rcName.  SetRect( MARGIN_X+425,               -m_nTopMargin-50,                   MARGIN_X+1620,      -m_nTopMargin-110 );
            m_rcStat.  SetRect( MARGIN_X+425,               -m_nTopMargin-110,                  MARGIN_X+1040,      -m_nTopMargin-260 );
            m_rcSide.  SetRectEmpty();
            m_rcSan.   SetRect( MARGIN_X+425,               -m_nTopMargin-260,                  MARGIN_X+1015,      -m_nTopMargin-550 );
            m_rcMagic. SetRect( MARGIN_X+1330,              -m_nTopMargin-180,                  MARGIN_X+1620,      -m_nTopMargin-350 );
            m_rcHit.   SetRect( MARGIN_X+1040,              -m_nTopMargin-180,                  MARGIN_X+1320,      -m_nTopMargin-350 );
            m_rcSkill. SetRect( MARGIN_X+50,                -m_nTopMargin-575,                  MARGIN_X+1620,      -m_nTopMargin-1040 );
            m_rcPic.   SetRect( MARGIN_X+50,                -m_nTopMargin-50,                   MARGIN_X+380,       -m_nTopMargin-550 );
            m_rcHand.  SetRectEmpty();
            m_rcFire.  SetRect( MARGIN_X+1040,              -m_nTopMargin-360,                  MARGIN_X+1620,      -m_nTopMargin-550 );
            break;
    }

    CPoint pt;
    GetSheetSize( pt );
    SetScrollSizes( MM_LOMETRIC, CSize( pt.x, pt.y ) );
}


void CCthulhuView::GetSheetSize( CPoint& pt )
{
    switch( m_DisplayType )
    {
        case DT_COC50:
            pt.x = 1880+(2*MARGIN_X);
            pt.y = 2510+m_nTopMargin+BOTTOM_MARGIN;

        case DT_DELTAGREEN:
            pt.x = 1880+(2*MARGIN_X);
            pt.y = 2550+m_nTopMargin+BOTTOM_MARGIN;
            break;

        case DT_COC50SMALL:
            pt.x = 1670+(2*MARGIN_X);
            pt.y = 1100+m_nTopMargin+BOTTOM_MARGIN;
            break;

        default:
            ASSERT(FALSE);
    }
}



void CCthulhuView::UpdateSettings()
{
    //reload the settings
    LoadSettings();

    //update the text edit control
    m_EditControl.CreateFont( m_strUserTextFace, m_crUserText );
}


void CCthulhuView::AddFieldListItem( FieldList* pItem )
{
    if( m_pFieldList == NULL )
    {
        //first item
        pItem->next = pItem->prev = pItem;
        m_pFieldList = pItem;
    }
    else
    {
        //not the first item... insert it after the last item (just before the first)
        pItem->next = m_pFieldList;
        pItem->prev = m_pFieldList->prev;
        m_pFieldList->prev = pItem;
        pItem->prev->next = pItem;
    }
}

void CCthulhuView::AddField( int* pn, int x1, int x2, int y, CString strDescription, int nMinValue /*0*/ )
{
    FieldList* pNew = new FieldList;
    pNew->pn = pn;
    pNew->pstr = NULL;
    pNew->nConstraintValue = nMinValue;
    pNew->rc = CRect(x1,y+EDIT_FIELD_HEIGHT,x2,y);
    pNew->strDescription = strDescription;
    pNew->iField = m_nFieldCount++;
    AddFieldListItem( pNew );
}

void CCthulhuView::AddField( CString* pstr, int x1, int x2, int y, CString strDescription, int nMaxLength /*0*/ )
{
    FieldList* pNew = new FieldList;
    pNew->pn = NULL;
    pNew->pstr = pstr;
    pNew->nConstraintValue = nMaxLength;
    pNew->rc = CRect(x1,y+EDIT_FIELD_HEIGHT,x2,y);
    pNew->strDescription = strDescription;
    pNew->iField = m_nFieldCount++;
    AddFieldListItem( pNew );
}

void CCthulhuView::DeleteFieldList()
{
    //if there is a list...
    if( m_pFieldList )
    {
        //break the circle
        m_pFieldList->prev->next = NULL;

        //delete the list
        while( m_pFieldList )
        {
            FieldList* pItem = m_pFieldList;
            m_pFieldList = m_pFieldList->next;
            delete pItem;
        }
    }
    m_nFieldCount = 0;
}

void CCthulhuView::BeginEditing( FieldList* pField )
{
    m_iCurrent = pField->iField;

    CRect rc = pField->rc;
    SheetToView( rc );

    if( pField->pn == NULL )
        m_EditControl.BeginEditing( pField->pstr, rc, CString( "Modify " + pField->strDescription ), pField->nConstraintValue );
    else
        m_EditControl.BeginEditing( pField->pn, rc, CString( "Modify " + pField->strDescription ), pField->nConstraintValue );
}


void CCthulhuView::OnEditNextField() 
{
    if( m_EditControl.Editing() ) m_EditControl.EndEditing( TRUE );

    if( m_iCurrent != -1 )
    {
        FieldList* pCurrent = FindField( m_iCurrent );
        BeginEditing( pCurrent->next );
    }
    else
        if( m_pFieldList )
            BeginEditing( m_pFieldList );
}

void CCthulhuView::OnEditPrevField() 
{
    if( m_EditControl.Editing() ) m_EditControl.EndEditing( TRUE );

    if( m_iCurrent != -1 )
    {
        FieldList* pCurrent = FindField( m_iCurrent );
        BeginEditing( pCurrent->prev );
    }
    else
        if( m_pFieldList )
            BeginEditing( m_pFieldList->prev );
}

void CCthulhuView::OnSheetStyleChange( UINT uID )
{
    //stop editing
    if( m_EditControl.Editing() ) m_EditControl.EndEditing( TRUE );
    m_iCurrent = -1;


    //change id
    switch( uID )
    {
        case ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50:       m_DisplayType = DT_COC50; break;
        case ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50_SMALL: m_DisplayType = DT_COC50SMALL; break;
        case ID_VIEW_CHARACTERSHEETSTYLES_DELTAGREEN:            m_DisplayType = DT_DELTAGREEN; break;
        default: ASSERT(FALSE);
    }

    CCthulhuDoc* pDoc = GetDocument();

    if( pDoc )
    {
	    //display changes
        SetSheetSize();
        pDoc->m_fNeedRedrawRecalculation = TRUE;
        InvalidateRect( NULL, TRUE );
        RedrawWindow();
    }
}

void CCthulhuView::OnUpdateSheetStyle(CCmdUI* pCmdUI)
{
    /*
    switch( pCmdUI->m_nID )
    {
        case ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50:       pCmdUI->SetRadio( m_DisplayType == DT_COC50 ); break;
        case ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50_SMALL: pCmdUI->SetRadio( m_DisplayType == DT_COC50SMALL ); break;
        case ID_VIEW_CHARACTERSHEETSTYLES_DELTAGREEN:            pCmdUI->SetRadio( m_DisplayType == DT_DELTAGREEN ); break;
    }
    */
    //fixme: this is disabled until the popup menu is displaying the correct version of the blob
    //in other words: How do I get MFC to update it's CCmdUIs automatically?
}

void CCthulhuView::OnViewWhatSheetType() 
{
	//find out the item rect of the execute script button
	RECT rc;
	((CMainFrame*)AfxGetMainWnd())->GetToolBar().GetItemRect( ((CMainFrame*)AfxGetMainWnd())->GetToolBar().CommandToIndex(ID_VIEW_WHATSHEETTYPE), &rc );
	((CMainFrame*)AfxGetMainWnd())->GetToolBar().ClientToScreen( &rc );

    //find view menu
    CMenu* pMenu = AfxGetMainWnd()->GetMenu();
    if( pMenu == NULL || ::IsMenu(pMenu->m_hMenu) == FALSE ) return;

    CMenu* pMenuView = NULL;
	int iItem = pMenu->GetMenuItemCount();
	while( iItem-- )
	{
		CMenu* pMenuPop = pMenu->GetSubMenu(iItem);
		if( pMenuPop != NULL )
		{
			int iItemMax = pMenuPop->GetMenuItemCount();
			for( int iItemPop = 0; iItemPop < iItemMax; iItemPop++ )
			{
				if( pMenuPop->GetMenuItemID(iItemPop) == ID_VIEW_STATUS_BAR )
                {
                    pMenuView = pMenuPop;
                    break;
                }
			}
		}
	}
    if( pMenuView == NULL ) return;

    //now find view style submenu
    CMenu* pMenuSheetStyle = NULL;
	iItem = pMenuView->GetMenuItemCount();
	while( iItem-- )
	{
		CMenu* pMenuPop = pMenuView->GetSubMenu(iItem);
		if( pMenuPop != NULL )
		{
			int iItemMax = pMenuPop->GetMenuItemCount();
			for( int iItemPop = 0; iItemPop < iItemMax; iItemPop++ )
			{
				if( pMenuPop->GetMenuItemID(iItemPop) == ID_VIEW_CHARACTERSHEETSTYLES_CALLOFCTHULHU50 )
                {
                    pMenuSheetStyle = pMenuPop;
                    break;
                }
			}
		}
	}
    if( pMenuSheetStyle == NULL ) return;

    //update option blob
    //fixme:

    //display the menu
    pMenuSheetStyle->TrackPopupMenu( TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, rc.left, rc.bottom, this );
}
