// Skill.cpp: implementation of the CSkill class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <strstrea.h>
#include "Cthulhu.h"
#include "OptionalSkillsDlg.h"
#include "Skill.h"
#include "Options.h"
#include "Splash.h"
#include <ios>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CSkillList g_CoreSkills;
CSkillList g_Firearms;
CSkillList g_Melee;

COccupationList g_Occupations;



BOOL g_fError = NULL;


/* CSkill ********************************************************/

CSkill::CSkill( CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990 )
{
    m_strName = strName;
    m_strBase = strBase;
    m_f1890 = f1890;
    m_f1920 = f1920;
    m_f1990 = f1990;
    m_Type = SKILL;

    m_nRef = 0;
}

CSkill::CSkill()
{
    m_f1890 = m_f1920 = m_f1990 = TRUE; 
    m_Type = ANYSKILL; 
    m_nRef = 0;
}

CSkill::~CSkill()
{
}

CSkill* CSkill::GetCopy()
{
    if( m_Type == ANYSKILL )
        return new CSkill();
    else
        return new CSkill( m_strName, m_strBase, m_f1890, m_f1920, m_f1990 );
}


void CSkill::AddRef()
{
    m_nRef++;
}

void CSkill::RemoveRef()
{
    m_nRef--;

    if( m_nRef <= 0 )
    {
        delete this;
    }
}

BOOL CSkill::IsCalculatedBase()
{
    const char* pszBase = LPCSTR(m_strBase);

    //skip over known, allowable prefixes
    if( *pszBase == 'F' || *pszBase == '*' ) pszBase++;

    //check whole string for non-numbers
    while( *pszBase != '\0' )
    {
        if( isdigit( (int)*pszBase ) == FALSE ) return TRUE;
        pszBase++;
    }

    return FALSE;
}


BOOL CSkill::IsEra( ERA era )
{
    switch( era )
    {
        case ERA_1890: return m_f1890;
        case ERA_1920: return m_f1920;
        case ERA_1990: return m_f1990;
    }

    return FALSE;
}

SkillType CSkill::GetType()
{
    return m_Type;
}

int CSkill::CompareName( CSkill* pOtherSkill )
{
    return CompareName( pOtherSkill->m_strName );
}

int CSkill::CompareName( CString pString )
{
    const char* psz1 = LPCSTR(m_strName);
    const char* psz2 = LPCSTR(pString);

    int iRead1 = 0, iRead2 = 0;

    while( *psz1 == '*' ) psz1++;
    while( *psz2 == '*' ) psz2++;

	while( psz1[iRead1] != '\0' && psz2[iRead2] != '\0' && psz1[iRead1] != '$' && psz2[iRead2] != '$' )
	{
		if( psz1[iRead1] == psz2[iRead2] )
		{
			iRead1++;
			iRead2++;
		}
		else
            return (psz1[iRead1] > psz2[iRead2]) ? 1 : -1;
	}

    BOOL fEnd1 = (psz1[iRead1] == '\0' || psz1[iRead1] == '$' );
    BOOL fEnd2 = (psz2[iRead2] == '\0' || psz2[iRead2] == '$' );

    if( fEnd1 && fEnd2 )
        return 0;

    if( fEnd1 )
        return 1;

    return -1;
}


/* CSpecialiseSkill **********************************************/

CSpecialiseSkill::CSpecialiseSkill( int nSkills, CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990, BOOL fInitialise /*TRUE*/ )
{
    m_nSkills = nSkills;
    m_pSkillSpecialities = new CSpecialiseSkillSkill*[nSkills];

    //initialise the child item types
    if( fInitialise )
    {
        for( int i = 0; i < nSkills; i++ )
        {
            m_pSkillSpecialities[i] = new CSpecialiseSkillSkill();
            m_pSkillSpecialities[i]->AddRef();
            m_pSkillSpecialities[i]->m_strBase = strBase;
            m_pSkillSpecialities[i]->m_f1890 = f1890;
            m_pSkillSpecialities[i]->m_f1920 = f1920;
            m_pSkillSpecialities[i]->m_f1990 = f1990;
            m_pSkillSpecialities[i]->m_pSkill = this;
        }
    }
    else for( int i = 0; i < nSkills; i++ ) m_pSkillSpecialities[i] = NULL;

    //store default values
    m_strName = strName;
    m_strBase = strBase;
    m_f1890 = f1890;
    m_f1920 = f1920;
    m_f1990 = f1990;
    m_Type = SPECIALISESKILL;
    m_nRef = 0;
}

CSpecialiseSkill::~CSpecialiseSkill()
{
    for( int i = 0; i < m_nSkills; i++ )
    {
        if( m_pSkillSpecialities[i] ) 
            m_pSkillSpecialities[i]->RemoveRef();
    }

    delete[] m_pSkillSpecialities;
}

CSkill* CSpecialiseSkill::GetCopy()
{
    return new CSpecialiseSkill( m_nSkills, m_strName, m_strBase, m_f1890, m_f1920, m_f1990 );
}

/* CSpecialiseSkillSkill *****************************************/

CSpecialiseSkillSkill::CSpecialiseSkillSkill()
{
    m_pSkill = NULL; 
    m_Type = SPECIALISESKILLITEM; 
    m_f1890 = m_f1920 = m_f1990 = FALSE; 
    m_strBase = "0"; 
    m_nRef = 0;
}

CSkill* CSpecialiseSkillSkill::GetCopy()
{
    //can't copy one of these!
    ASSERT(FALSE);
    return NULL;
}

/* CFirearmSkill *************************************************/

CFirearmSkill::CFirearmSkill( CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990, CString strDmg, CString strRange, CString strShots, CString strBullets, CString strHP, CString strMAL )
{
    m_strName = strName;
    m_strBase = strBase;
    m_f1890 = f1890;
    m_f1920 = f1920;
    m_f1990 = f1990;
    m_strDmg = strDmg;
    m_strRange = strRange;
    m_strShots = strShots;
    m_strBullets = strBullets;
    m_strHP = strHP;
    m_strMAL = strMAL;

    m_Type = FIREARM;
    m_nRef = 0;
}

CFirearmSkill::~CFirearmSkill()
{
}

CSkill* CFirearmSkill::GetCopy()
{
    return new CFirearmSkill( m_strName, m_strBase, m_f1890, m_f1920, m_f1990, m_strDmg, m_strRange, m_strShots, m_strBullets, m_strHP, m_strMAL );
}



/* CHandToHandSkill **********************************************/

CMeleeSkill::CMeleeSkill(CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990, CString strDmg, CString strAttacks, CString strHP )
{
    m_strName = strName;
    m_strBase = strBase;
    m_f1890 = f1890;
    m_f1920 = f1920;
    m_f1990 = f1990;
    m_strDmg = strDmg;
    m_strAttacks = strAttacks;
    m_strHP = strHP;

    m_Type = MELEE;
    m_nRef = 0;
}

CMeleeSkill::~CMeleeSkill()
{
}

CSkill* CMeleeSkill::GetCopy()
{
    return new CMeleeSkill( m_strName, m_strBase, m_f1890, m_f1920, m_f1990, m_strDmg, m_strAttacks, m_strHP );
}



/* CSkillList ****************************************************/

CSkillList::CSkillList()
{
    m_pSkillList = NULL;
}

CSkillList::~CSkillList()
{
    ResetList();
}

BOOL CSkillList::AddSkill( CSkill *pSkill )
{
    if( pSkill->GetType() != SPECIALISESKILLITEM && pSkill->GetType() != ANYSKILL ) ASSERT( pSkill->m_strName.GetLength() > 0 );

    //add it at the start
    SkillList* pNew = new SkillList;
    pSkill->AddRef();
    pNew->pSkill = pSkill;
    pNew->next = m_pSkillList;
    m_pSkillList = pNew;

    return TRUE;
}

CSkill* CSkillList::FindSkill( CString strName )
{
    SkillList* pList = m_pSkillList;

    while( pList )
    {
        if( pList->pSkill->CompareName( strName ) == 0 )
            return pList->pSkill;

        pList = pList->next;
    }

    return NULL;
}

void CSkillList::ResetList()
{
    while( m_pSkillList )
    {
        SkillList* pTmp = m_pSkillList;
        m_pSkillList = m_pSkillList->next;
        pTmp->pSkill->RemoveRef();
        delete pTmp;
    }
}


int CSkillList::GetCount()
{
    int nSkills = 0;
    SkillList* pList = m_pSkillList;

    while( pList )
    {
        nSkills++;
        pList = pList->next;
    }

    return nSkills;
}

const CSkillList& CSkillList::operator=(const CSkillList& src )
{
    ResetList();

    SkillList* pList = src.m_pSkillList;
    while( pList )
    {
        AddSkill( pList->pSkill );
        pList = pList->next;
    }

    return *this;
}



/* COccupation ***************************************************/

COccupation::COccupation()
{
    m_nChoice = 0;
    m_f1890 = m_f1920 = m_f1990 = TRUE;
}

COccupation::~COccupation()
{
}

BOOL COccupation::IsEra( ERA era )
{
    switch( era )
    {
        case ERA_1890: return m_f1890;
        case ERA_1920: return m_f1920;
        case ERA_1990: return m_f1990;
    }

    return FALSE;
}




/* COccupationList ***********************************************/

COccupationList::COccupationList()
{
    m_pOccupationList = NULL;
}

COccupationList::~COccupationList()
{
    ResetList();
}

BOOL COccupationList::AddOccupation( COccupation *pOccupation )
{
    if( !FindOccupation( pOccupation->m_strName ) )
    {
        //add it at the start
        OccupationList* pNew = new OccupationList;
        pNew->pOccupation = pOccupation;
        pNew->next = m_pOccupationList;
        m_pOccupationList = pNew;

        return TRUE;
    }
    else
        return FALSE; //already exists
}

COccupation* COccupationList::FindOccupation( CString strName )
{
    OccupationList* pList = m_pOccupationList;

    while( pList )
    {
        if( pList->pOccupation->m_strName.CompareNoCase( strName ) == 0 )
            return pList->pOccupation;

        pList = pList->next;
    }

    return NULL;
}

void COccupationList::ResetList()
{
    while( m_pOccupationList )
    {
        OccupationList* pTmp = m_pOccupationList;
        m_pOccupationList = m_pOccupationList->next;

        /* note: it is VERY important that we delete the pOccupation as well as the list item.
                 the consequences of not doing so could not only tear the thin fabric of the
                 universe as we know it, but also cause one to spend about seven hours tracking
                 down a fucking memory leak which appears to be coming from nowhere... (ahem) */
        delete pTmp->pOccupation; 
        delete pTmp;
    }
}

int COccupationList::GetCount()
{
    int nOccupations = 0;
    OccupationList* pList = m_pOccupationList;

    while( pList )
    {
        nOccupations++;
        pList = pList->next;
    }

    return nOccupations;
}





/* CInvestigator *************************************************/

CInvestigator::CInvestigator()
{
    m_pSkillList = NULL;

    m_nSAN = -999;
    m_nHitPoints = -999;
    m_nMagic = -999;
}

CInvestigator::CInvestigator( CInvestigator& src )
{
    m_pSkillList = NULL;
    CopyInvestigator( src );
}

CInvestigator::~CInvestigator()
{
    DeleteSkills();
}

void CInvestigator::Serialize( CArchive& ar )
{
	if (ar.IsStoring())
    {
    	int iSaveType = AfxGetApp()->GetProfileInt( "Save", "SaveVersion", 1 );

        switch( iSaveType )
        {
            case 0: //1.0
                ar.WriteString( SHEET_FILE_HEADER10 ); ar << '\n'; break;

            case 1: //2.0
                ar.WriteString( SHEET_FILE_HEADER20 ); ar << '\n'; break;

            default:
                AfxMessageBox( "Invalid setting for save version - using 2.0", MB_ICONEXCLAMATION );
                ar.WriteString( SHEET_FILE_HEADER20 ); ar << '\n'; break;
        }

        //basic stats
        ar.WriteString( m_strName ); ar << '\n';
        ar.WriteString( m_strOccupation ); ar << '\n';
        ar.WriteString( m_strBirthplace ); ar << '\n';
        ar.WriteString( m_strColleges ); ar << '\n';
        ar.WriteString( m_strMentalDisorders ); ar << '\n';
        ar.WriteString( m_strSex ); ar << '\n';
        ar << m_nAge;
        ar << int(m_era);
        ar.WriteString( m_strPortrait ); ar << '\n';
        ar << m_nAPP;
        ar << m_nCON;
        ar << m_nDEX;
        ar << m_nEDU;
        ar << m_nINT;
        ar << m_nPOW;
        ar << m_nSIZ;
        ar << m_nSTR;

        //count skills
        int nSkills = 0;
        InvestigatorSkillList* pSkills = m_pSkillList;
        while( pSkills )
        {
            nSkills++;
            pSkills = pSkills->next;
        }

        //write out skill count
        ar << nSkills;;

        //write out skills
        pSkills = m_pSkillList;
        while( pSkills )
        {
            //write out additional skill info
            ar << pSkills->nCurrent;
            ar << pSkills->nMinimumValue;
            ar << pSkills->fOccupational;

            //write out skill
            WriteSkill( ar, pSkills->pSkill );

            pSkills = pSkills->next;
        }

        //write player name
        ar.WriteString( m_strPlayerName ); ar << '\n';

        //new for version 1.1:
        ar << m_nSAN;
        ar << m_nHitPoints;
        ar << m_nMagic;
        ar.WriteString( m_strDG_Affiliation ); ar << '\n';
        ar.WriteString( m_strDG_CodeName ); ar << '\n';
        ar.WriteString( m_strDG_CurrentDate ); ar << '\n';
        ar.WriteString( m_strDG_Insanities ); ar << '\n';
    }
    else
    {
        int nVersionMajor = 0, nVersionMinor = 0;

        //header
        CString strHeader;
        ar.ReadString( strHeader );

        if( strHeader.Compare( SHEET_FILE_HEADER10 ) == 0 )
        {
            nVersionMajor = 1;
            nVersionMinor = 0;
        }
        else
            if( strHeader.Compare( SHEET_FILE_HEADER20 ) == 0 )
            {
                nVersionMajor = 2;
                nVersionMinor = 0;
            }
            else
            {
                AfxThrowArchiveException( CArchiveException::badIndex, "" );
                return;
            }

        //basic stats
        if( nVersionMajor >= 1 && nVersionMinor >= 0 )
        {
            ar.ReadString( m_strName );
            ar.ReadString( m_strOccupation );
            ar.ReadString( m_strBirthplace );
            ar.ReadString( m_strColleges );
            ar.ReadString( m_strMentalDisorders );
            ar.ReadString( m_strSex );
            ar >> m_nAge;
            int nEra;
            ar >> nEra;
            m_era = ERA(nEra);
            ar.ReadString( m_strPortrait );
            ar >> m_nAPP;
            ar >> m_nCON;
            ar >> m_nDEX;
            ar >> m_nEDU;
            ar >> m_nINT;
            ar >> m_nPOW;
            ar >> m_nSIZ;
            ar >> m_nSTR;

            //read in skill count
            int nSkills = 0;

            //write out skill count
            ar >> nSkills;

            //read in skills
            DeleteSkills();
            for( int i = 0; i < nSkills; i++ )
            {
                int nCurrent, nMinimumValue;
                BOOL fOccupational;

                //read in additional skill info
                ar >> nCurrent;
                ar >> nMinimumValue;
                ar >> fOccupational;

                //read in skill
                if( ReadSkill( ar, fOccupational, nCurrent, nMinimumValue ) == 0 )
                    AfxThrowArchiveException( CArchiveException::badIndex, "" );
            }

            //read player name
            ar.ReadString( m_strPlayerName );

            //*** new for version 2.0: ***

            //plug in defaults
            m_nSAN = m_nPOW*5;
            m_nHitPoints = (int)( 0.5 + (((double)m_nCON + (double)m_nSIZ ) / 2.0 ) );
            m_nMagic = m_nPOW;

            //only load SAN etc if the version number was >= 2.0
            if( nVersionMajor >= 2 && nVersionMinor >= 2 )
            {
                ar >> m_nSAN;
                ar >> m_nHitPoints;
                ar >> m_nMagic;
                ar.ReadString( m_strDG_Affiliation );
                ar.ReadString( m_strDG_CodeName );
                ar.ReadString( m_strDG_CurrentDate );
                ar.ReadString( m_strDG_Insanities );

                //*** add file format extensions here: ***

            } //version 2.0

        } //version 1.0


        //sort all skills into order
        SortSkillList();
    }
}

int CInvestigator::FindStatValue( const char* pszStatName )
{
	//check basic stats
	if( strcmpi( pszStatName, "APP" ) == 0 ) return m_nAPP;
	if( strcmpi( pszStatName, "CON" ) == 0 ) return m_nCON;
	if( strcmpi( pszStatName, "DEX" ) == 0 ) return m_nDEX;
	if( strcmpi( pszStatName, "EDU" ) == 0 ) return m_nEDU;
	if( strcmpi( pszStatName, "INT" ) == 0 ) return m_nINT;
	if( strcmpi( pszStatName, "POW" ) == 0 ) return m_nPOW;
	if( strcmpi( pszStatName, "SIZ" ) == 0 ) return m_nSIZ;
	if( strcmpi( pszStatName, "STR" ) == 0 ) return m_nSTR;

	//check calculated stats
	if( strcmpi( pszStatName, "LUCK" ) == 0 ) return m_nPOW * 5;
	if( strcmpi( pszStatName, "KNOW" ) == 0 ) return m_nEDU * 5;
	if( strcmpi( pszStatName, "IDEA" ) == 0 ) return m_nINT * 5;

    //check extended stats
	if( strcmpi( pszStatName, "SAN" ) == 0 ) return m_nSAN;
	if( strcmpi( pszStatName, "HP" ) == 0 ) return m_nHitPoints;
	if( strcmpi( pszStatName, "HIT" ) == 0 ) return m_nHitPoints;
	if( strcmpi( pszStatName, "MAGIC" ) == 0 ) return m_nMagic;

	//failed...
	CString str;
	str.Format( "Unknown character statistic: %s", pszStatName );
	AfxMessageBox( str, MB_ICONEXCLAMATION );
	return 0;
}	

int CInvestigator::FindSkillValue( const char* pszSkillName )
{
    InvestigatorSkillList* pSkill = FindSkill( pszSkillName );
    if( pSkill )
        return pSkill->nCurrent;

	//failed...
	CString str;
	str.Format( "Unknown skill name: %s", pszSkillName );
	AfxMessageBox( str, MB_ICONEXCLAMATION );
	return 0;
}

int CInvestigator::GetSkillNumber( const char* pszValue )
{
	char* pszCode = (char*)pszValue;
	int nResult = 0;

    BOOL fGotStatMultiplier = FALSE;

	while( pszCode && *pszCode != '\0' )
	{
		switch( *pszCode )
		{
			case '^': //'^' == skill stat
			{
				//get the stat on the end of the '^'
				CString strSkill = "";
				pszCode++;
				while( isalpha(*pszCode) && *pszCode != '\0' ) 
				{
					strSkill = strSkill + *pszCode;
					pszCode++;
				}

				nResult += FindStatValue( strSkill );
                fGotStatMultiplier = TRUE;
				break;	
			}

			case '*':  //'*' == Can't modify
				pszCode++; 
				break;

			case '%':
			{
				//read in the skill
				CString strSkill = "";
				pszCode++;
				while( (*pszCode) != '%' && *pszCode != '\0' ) 
				{
					strSkill = strSkill + *pszCode;
					pszCode++;
				}
				pszCode++;

				nResult += FindSkillValue( strSkill );
				break;	
			}

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
			{
				//rip out the number
				CString strValue = "";
				while( isdigit(*pszCode) && *pszCode != '\0' ) 
				{
					strValue = strValue + *pszCode;
					pszCode++;
				}

				if( strValue.GetLength() > 0 )
					if( fGotStatMultiplier )
						nResult *= atoi( LPCSTR(strValue) );
					else
						nResult = atoi( LPCSTR(strValue) );
				break;	
			}

			default:
				pszCode++;
				break;

		}
	}

	return nResult;
}

void CInvestigator::RemoveSkill( CSkill* pSkill )
{
    InvestigatorSkillList* pList = m_pSkillList;

    if( pList->pSkill == pSkill )
    {
        //it's the first item
        m_pSkillList = pList->next;
        pList->pSkill->RemoveRef();
        delete pList;
    }
    else
        //scan down the list and find it
        while( pList )
        {
            if( pList->next && pList->next->pSkill == pSkill )
            {
                InvestigatorSkillList* pDelete = pList->next;
                pList->next = pDelete->next;

                pDelete->pSkill->RemoveRef();
                delete pDelete;
            }

            pList = pList->next;
        }
}


CInvestigator::InvestigatorSkillList* CInvestigator::AddSkill( CSkill *pSkill, BOOL fOccupational )
{
    if( pSkill->GetType() != SPECIALISESKILLITEM && pSkill->GetType() != ANYSKILL && pSkill->GetType() != MELEE && pSkill->GetType() != FIREARM ) 
    {
        //don't add NULL types or existing skills
        if( pSkill->m_strName.GetLength() <= 0 || FindSkill( pSkill->m_strName ) ) return NULL;
    }

    //add it at the start
    InvestigatorSkillList* pNew = new InvestigatorSkillList;
    pNew->pSkill = pSkill;
    pSkill->AddRef();
    pNew->next = m_pSkillList;
    pNew->fOccupational = fOccupational;
    pNew->nCurrent = 0;
    pNew->nMinimumValue = 0;
    m_pSkillList = pNew;

    return pNew;
}

CInvestigator::InvestigatorSkillList* CInvestigator::FindSkill( CString strName )
{
    InvestigatorSkillList* pList = m_pSkillList;

    while( pList )
    {
        if( pList->pSkill->CompareName( strName ) == 0 )
            return pList;

        pList = pList->next;
    }

    return NULL;
}

void CInvestigator::EvaluateSkills()
{
    //update the skill list current percentiles
    InvestigatorSkillList* pList = m_pSkillList;

    while( pList )
    {
        pList->nCurrent = pList->nMinimumValue = GetSkillNumber( pList->pSkill->m_strBase );
        pList = pList->next;
    }
}

int CInvestigator::CompareSkills(CSkill * pSkill1, CSkill * pSkill2)
{
    CString strSortName1, strSortName2;
    SkillType Type1 = pSkill1->GetType(), Type2 = pSkill2->GetType();

    //if it's a special, sort on parent's name initially...
    if( Type1 == SPECIALISESKILLITEM ) strSortName1 = ((CSpecialiseSkillSkill*)pSkill1)->m_pSkill->m_strName; else strSortName1 = pSkill1->m_strName;
    if( Type2 == SPECIALISESKILLITEM ) strSortName2 = ((CSpecialiseSkillSkill*)pSkill2)->m_pSkill->m_strName; else strSortName2 = pSkill2->m_strName;

    //resort to a sort on skill name alone
    const char* psz1 = LPCSTR(strSortName1);
    const char* psz2 = LPCSTR(strSortName2);

    int iRead1 = 0, iRead2 = 0;
    //don't skip the stars - they keep the hand to hand & main skill lists in order

    int nResult = stricmp( psz1, psz2 );

    if( nResult == 0 )
    {
        //sort sub items
        if( Type1 == SPECIALISESKILLITEM && Type2 == SPECIALISESKILLITEM )
        {
            //alphabetic sort on sub items, gaps at the bottom
            if( pSkill1->m_strName.GetLength() == 0 )
                return ( pSkill2->m_strName.GetLength() == 0 ) ? 0 : 1;
            if( pSkill2->m_strName.GetLength() == 0 )
                return -1;
            return pSkill1->m_strName.CompareNoCase( pSkill2->m_strName );
        }

        //put parent items before subitems
        if( Type1 == SPECIALISESKILLITEM && Type2 == SPECIALISESKILL ) return 1;
        if( Type1 == SPECIALISESKILL     && Type2 == SPECIALISESKILLITEM ) return -1;
        return 0;
    }
    else return nResult;
}

CInvestigator::InvestigatorSkillList* CInvestigator::Sort_RemoveLargest( CInvestigator::InvestigatorSkillList* start )
{
    CInvestigator::InvestigatorSkillList* current, *previous;

    previous = start;
    current = start;

    do {
        if( CompareSkills( current->pSkill, current->next->pSkill ) > 0 )
        {
            //swap 'em over
            CSkill* pSkillTmp = current->pSkill; current->pSkill = current->next->pSkill; current->next->pSkill = pSkillTmp;
            int     nCurrentTmp = current->nCurrent; current->nCurrent = current->next->nCurrent; current->next->nCurrent = nCurrentTmp;
            int     nMinValueTmp = current->nMinimumValue;current->nMinimumValue = current->next->nMinimumValue; current->next->nMinimumValue = nMinValueTmp;
            BOOL    fOccupationalTmp = current->fOccupational; current->fOccupational = current->next->fOccupational; current->next->fOccupational = fOccupationalTmp;
        }
        previous = current;
        current = current->next;

    } while( current->next != NULL );

    previous->next = NULL;
    return current;
}

void CInvestigator::SortSkillList()
{
    InvestigatorSkillList* start = m_pSkillList, *result, *max;

    if( start != NULL )
    {
        result = NULL;
        while( start->next != NULL )
        {
            max = Sort_RemoveLargest( start );
            max->next = result;
            result = max;
        }
        start->next = result;
        m_pSkillList = start;
    }
}

CInvestigator::InvestigatorSkillList* CInvestigator::AddSkillCopy(CSkill * pSkill, BOOL fOccupational)
{
    if( pSkill->GetType() != SPECIALISESKILLITEM ) //don't add special items directly - add them with their parent
    {
        if( pSkill->GetType() == SPECIALISESKILL )
        {
            CSpecialiseSkill* pSpecialCopy = (CSpecialiseSkill*)pSkill->GetCopy();

            //add it
            InvestigatorSkillList* pReturn = AddSkill( pSpecialCopy, fOccupational );

            for( int i = 0; i < pSpecialCopy->m_nSkills; i++ )
            {
                AddSkill( pSpecialCopy->m_pSkillSpecialities[i], fOccupational );
            }

            return pReturn;
        }
        else return AddSkill( pSkill->GetCopy(), fOccupational );
    }
    else
        return NULL;
}

BOOL CInvestigator::SetOccupationSkills( COccupation* pOccupation )
{
    CSkillList::SkillList* pSkills;

    //clear the skill pointer and store the old list
    InvestigatorSkillList* pList = m_pSkillList;
    m_pSkillList = NULL;

    /* add occupational skills */
    if( pOccupation )
    {
        //let them pick optional skills
	    if( pOccupation->GetChoiceCount() > 0 )
	    {
		    COptionalSkillsDlg OptionalSkills;
		    OptionalSkills.m_strNumSkills.Format( "%d", pOccupation->GetChoiceCount() );
		    OptionalSkills.m_pInvestigator = this;
		    OptionalSkills.m_ChoicesList = pOccupation->GetChoiceList();
		    
		    if( OptionalSkills.DoModal() == IDOK ) 
            {
                for( pSkills = OptionalSkills.m_OptionalSkills.GetList(); pSkills != NULL; pSkills = pSkills->next )
                {
                    AddSkillCopy( pSkills->pSkill, TRUE );
                }
            }
		    else
            {
                //go no further & set the list back to normal
                m_pSkillList = pList;
                return FALSE;
            }
	    }

        //normal occupational skills
        for( pSkills = pOccupation->GetSkillList().GetList(); pSkills != NULL; pSkills = pSkills->next )
        {
            if( pSkills->pSkill->IsEra( m_era ) ) AddSkillCopy( pSkills->pSkill, TRUE );
        }
    }


    /* add basic skills */
    for( pSkills = g_CoreSkills.GetList(); pSkills != NULL; pSkills = pSkills->next )
    {
        if( pSkills->pSkill->GetType() != SPECIALISESKILLITEM && pSkills->pSkill->IsEra( m_era ) && !FindSkill( pSkills->pSkill->m_strName ) ) //&& (pSkills->pSkill->GetType() == SKILL || pSkills->pSkill->GetType() == SPECIALISESKILL || pSkills->pSkill->GetType() == SPECIALISESKILLITEM ))
            AddSkillCopy( pSkills->pSkill, FALSE );
    }


    /* add normal melee skills ( fist/punch etc )*/
    for( pSkills = g_Melee.GetList(); pSkills != NULL; pSkills = pSkills->next )
    {
        if( pSkills->pSkill->m_strName[0] == '*' && pSkills->pSkill->IsEra( m_era ) )
            AddSkillCopy( pSkills->pSkill, FALSE );
    }


    /* Sort the list */
    SortSkillList();

    /* Calculate Base chances */
    EvaluateSkills();

    /* Delete old list */
    while( pList )
    {
        InvestigatorSkillList* pTmp = pList;
        pList = pList->next;

        pTmp->pSkill->RemoveRef();
        delete pTmp;
    }


    return TRUE;
}

void CInvestigator::DeleteSkills()
{
    while( m_pSkillList )
    {
        InvestigatorSkillList* pTmp = m_pSkillList;
        m_pSkillList = m_pSkillList->next;

        pTmp->pSkill->RemoveRef();
        delete pTmp;
    }
}

void CInvestigator::ClearWeapons()
{
    InvestigatorSkillList* pList = m_pSkillList, *pPrev = NULL;

    while( pList )
    {
        //if it is a firearm or melee weapon, remove it
        if( (pList->pSkill->GetType() == MELEE || pList->pSkill->GetType() == FIREARM) && pList->pSkill->m_strName[0] != '*' )
        {
            InvestigatorSkillList* pRemove = pList;

            //unlink it from the list
            if( pPrev ) 
            {
                pPrev->next = pList->next; 
                pList = pPrev;
            }
            else 
            {
                m_pSkillList = pList->next;
                pList = m_pSkillList;
            }


            //delete it
            pRemove->pSkill->RemoveRef();
            delete pRemove;

        }

        //store previous and advance list
        pPrev = pList;
        pList = pList->next;
    }
}


void CInvestigator::CopyInvestigator( const CInvestigator& src )
{
    //delete current skills
    DeleteSkills();
    ASSERT( m_pSkillList == NULL );
    
    //copy basic stats
	m_strName = src.m_strName;
	m_strOccupation = src.m_strOccupation;
	m_strBirthplace = src.m_strBirthplace;
    m_strColleges = src.m_strColleges;
    m_strMentalDisorders = src.m_strMentalDisorders;
    m_strSex = src.m_strSex;
	m_nAge = src.m_nAge;
	m_era = src.m_era;

    m_strPortrait = src.m_strPortrait;
	
	m_nAPP = src.m_nAPP;
	m_nCON = src.m_nCON;
	m_nDEX = src.m_nDEX;
	m_nEDU = src.m_nEDU;
	m_nINT = src.m_nINT;
	m_nPOW = src.m_nPOW;
	m_nSIZ = src.m_nSIZ;
	m_nSTR = src.m_nSTR;

    m_nSAN = src.m_nSAN;
    m_nHitPoints = src.m_nHitPoints;
    m_nMagic = src.m_nMagic;

    m_strDG_Affiliation = src.m_strDG_Affiliation;
    m_strDG_CodeName = src.m_strDG_CodeName;
    m_strDG_CurrentDate = src.m_strDG_CurrentDate;
    m_strDG_Insanities = src.m_strDG_Insanities;


    //copy over the skill list
    InvestigatorSkillList* pList = src.m_pSkillList;
    while( pList )
    {
        if( pList->pSkill->GetType() != SPECIALISESKILLITEM )
        {
            //add the skill
            InvestigatorSkillList* pNew = AddSkillCopy( pList->pSkill, pList->fOccupational );
            if( pNew )
            {
                pNew->nCurrent = pList->nCurrent;
                pNew->nMinimumValue = pList->nMinimumValue;

                //modify sub-items, if it's got any
                if( pNew->pSkill->GetType() == SPECIALISESKILL )
                {
                    CSpecialiseSkill* pSpecial = (CSpecialiseSkill*)pNew->pSkill;

                    for( int i = 0; i < pSpecial->m_nSkills; i++ )
                    {
                        //find it in the new & old lists
                        InvestigatorSkillList* pNewSub = m_pSkillList;
                        InvestigatorSkillList* pSub = src.m_pSkillList;
                        while( pNewSub && pNewSub->pSkill != pSpecial->m_pSkillSpecialities[i] ) pNewSub = pNewSub->next;
                        while( pSub && pSub->pSkill != ((CSpecialiseSkill*)pList->pSkill)->m_pSkillSpecialities[i] ) pSub = pSub->next;

                        //copy over current values etc.
                        if( pNewSub && pSub )
                        {
                            pNewSub->fOccupational = pNew->fOccupational;

                            pNewSub->pSkill->m_strName = pSub->pSkill->m_strName;
                            pNewSub->nCurrent = pSub->nCurrent;
                            pNewSub->nMinimumValue = pSub->nMinimumValue;
                        }
                        else
                            TRACE0("Could not find new or existing subskill!\n" );
                    }
                }
            }
        }

        pList = pList->next;
    }

    //sort the skill list
    SortSkillList();
}

const CInvestigator& CInvestigator::operator=(const CInvestigator& src )
{
    CopyInvestigator( src );

    //move along... move along...
	return *this;
}

int CInvestigator::ReadSkill( CArchive& ar, BOOL fOccupational, int nCurrent, int nMinimumValue )
{
    int nType;
    ar >> nType;

    switch( SkillType(nType) )
    {
        case ANYSKILL:
        {
            InvestigatorSkillList* pNew = AddSkill( new CSkill, fOccupational );
            pNew->nCurrent = nCurrent;
            pNew->nMinimumValue = nMinimumValue;
            return 1;
            break;
        }

        case SKILL:
        {
            CString strName, strBase;
            BOOL f1890, f1920, f1990;
            ar.ReadString( strName );
            ar.ReadString( strBase );
            ar >> f1890;
            ar >> f1920;
            ar >> f1990;
            InvestigatorSkillList* pNew = AddSkill( new CSkill( strName, strBase, f1890, f1920, f1990 ), fOccupational );
            pNew->nCurrent = nCurrent;
            pNew->nMinimumValue = nMinimumValue;
            return 1;
            break;
        }

        case FIREARM:
        {
            CString strName, strBase, strDmg, strRange, strShots, strBullets, strHP, strMAL;
            BOOL f1890, f1920, f1990;
            ar.ReadString( strName );
            ar.ReadString( strBase );
            ar >> f1890;
            ar >> f1920;
            ar >> f1990;

            ar.ReadString( strDmg );
            ar.ReadString( strRange );
            ar.ReadString( strShots );
            ar.ReadString( strBullets );
            ar.ReadString( strHP );
            ar.ReadString( strMAL );

            InvestigatorSkillList* pNew = AddSkill( new CFirearmSkill( strName, strBase, f1890, f1920, f1990, strDmg, strRange, strShots, strBullets, strHP, strMAL ), fOccupational );
            pNew->nCurrent = nCurrent;
            pNew->nMinimumValue = nMinimumValue;
            return 1;
            break;
        }

        case MELEE:
        {
            CString strName, strBase, strDmg, strAttacks, strHP;
            BOOL f1890, f1920, f1990;
            ar.ReadString( strName );
            ar.ReadString( strBase );
            ar >> f1890;
            ar >> f1920;
            ar >> f1990;

            ar.ReadString( strDmg );
            ar.ReadString( strAttacks );
            ar.ReadString( strHP );

            InvestigatorSkillList* pNew = AddSkill( new CMeleeSkill( strName, strBase, f1890, f1920, f1990, strDmg, strAttacks, strHP ), fOccupational );
            pNew->nCurrent = nCurrent;
            pNew->nMinimumValue = nMinimumValue;
            return 1;
            break;
        }

        case SPECIALISESKILL:
        {
            CString strName, strBase;
            BOOL f1890, f1920, f1990;
            int nSkills;
            ar.ReadString( strName );
            ar.ReadString( strBase );
            ar >> f1890;
            ar >> f1920;
            ar >> f1990;

            ar >> nSkills;

            InvestigatorSkillList* pNew = AddSkill( new CSpecialiseSkill( nSkills, strName, strBase, f1890, f1920, f1990, FALSE /*DONT INITIALISE CHILD SKILLS*/ ), fOccupational );
            pNew->nCurrent = nCurrent;
            pNew->nMinimumValue = nMinimumValue;
            return 1;
            break;
        }

        case SPECIALISESKILLITEM:
        {
            CString strName, strBase, strParent;
            BOOL f1890, f1920, f1990;
            ar.ReadString( strName );
            ar.ReadString( strBase );
            ar >> f1890;
            ar >> f1920;
            ar >> f1990;

            ar.ReadString( strParent );

            InvestigatorSkillList* pParent = FindSkill( strParent );
            if( pParent && pParent->pSkill->GetType() == SPECIALISESKILL )
            {
                CSpecialiseSkillSkill* pNewSkill = new CSpecialiseSkillSkill;
                pNewSkill->m_strName = strName;
                pNewSkill->m_strBase = strBase;
                pNewSkill->m_f1890 = f1890;
                pNewSkill->m_f1920 = f1920;
                pNewSkill->m_f1990 = f1990;

                pNewSkill->m_pSkill = (CSpecialiseSkill*)pParent->pSkill;

                //now link the parent to it
                CSpecialiseSkill* pSpecial = (CSpecialiseSkill*)pParent->pSkill;

                //find free slot on the parent
                int i = 0;
                while( i < pSpecial->m_nSkills && pSpecial->m_pSkillSpecialities[i] != NULL ) i++;
                if( i < pSpecial->m_nSkills )
                {
                    pSpecial->m_pSkillSpecialities[i] = pNewSkill;
                    pNewSkill->AddRef();

                    //add it to the global list
                    InvestigatorSkillList* pNew = AddSkill( pNewSkill, fOccupational );
                    pNew->nCurrent = nCurrent;
                    pNew->nMinimumValue = nMinimumValue;
                    return 1;
                }
                else
                {
                    delete pNewSkill;
                    AfxThrowArchiveException( CArchiveException::badIndex, "" );
                    return 0;
                }
            }
            else
            {
                AfxThrowArchiveException( CArchiveException::badIndex, "" );
                return 0;
            }
        }
    }

    //unknown type
    AfxThrowArchiveException( CArchiveException::badIndex, "" );
    return 0;
}

void CInvestigator::WriteSkill( CArchive& ar, CSkill* pSkill )
{
    switch( pSkill->GetType() )
    {
        case ANYSKILL:
            ar << pSkill->GetType();
            break;

        case SKILL:
            ar << pSkill->GetType();
            ar.WriteString( pSkill->m_strName ); ar << '\n';
            ar.WriteString( pSkill->m_strBase ); ar << '\n';
            ar << pSkill->m_f1890;
            ar << pSkill->m_f1920;
            ar << pSkill->m_f1990;
            break;

        case FIREARM:
        {
            CFirearmSkill* pFirearm = (CFirearmSkill*)pSkill;
            ar << pFirearm->GetType();
            ar.WriteString( pFirearm->m_strName ); ar << '\n';
            ar.WriteString( pFirearm->m_strBase ); ar << '\n';
            ar << pFirearm->m_f1890;
            ar << pFirearm->m_f1920;
            ar << pFirearm->m_f1990;

            ar.WriteString( pFirearm->m_strDmg ); ar << '\n';
            ar.WriteString( pFirearm->m_strRange ); ar << '\n';
            ar.WriteString( pFirearm->m_strShots ); ar << '\n';
            ar.WriteString( pFirearm->m_strBullets ); ar << '\n';
            ar.WriteString( pFirearm->m_strHP ); ar << '\n';
            ar.WriteString( pFirearm->m_strMAL ); ar << '\n';
            break;
        }

        case MELEE:
        {
            CMeleeSkill* pMelee = (CMeleeSkill*)pSkill;
            ar << pMelee->GetType();
            ar.WriteString( pMelee->m_strName ); ar << '\n';
            ar.WriteString( pMelee->m_strBase ); ar << '\n';
            ar << pMelee->m_f1890;
            ar << pMelee->m_f1920;
            ar << pMelee->m_f1990;

            ar.WriteString( pMelee->m_strDmg ); ar << '\n';
            ar.WriteString( pMelee->m_strAttacks ); ar << '\n';
            ar.WriteString( pMelee->m_strHP ); ar << '\n';
            break;
        }

        case SPECIALISESKILL:
        {
            CSpecialiseSkill* pSpecial = (CSpecialiseSkill*)pSkill;
            ar << pSpecial->GetType();
            ar.WriteString( pSpecial->m_strName ); ar << '\n';
            ar.WriteString( pSpecial->m_strBase ); ar << '\n';
            ar << pSpecial->m_f1890;
            ar << pSpecial->m_f1920;
            ar << pSpecial->m_f1990;

            ar << pSpecial->m_nSkills;
            break;
        }

        case SPECIALISESKILLITEM:
        {
            CSpecialiseSkillSkill* pItem = (CSpecialiseSkillSkill*)pSkill;
            ar << pItem->GetType();
            ar.WriteString( pItem->m_strName ); ar << '\n';
            ar.WriteString( pItem->m_strBase ); ar << '\n';
            ar << pItem->m_f1890;
            ar << pItem->m_f1920;
            ar << pItem->m_f1990;

            //write out the *name* of the parent skill
            ar.WriteString( pItem->m_pSkill->m_strName ); ar << '\n';
            break;
        }
    }
}

BOOL CInvestigator::IsBase( int nPercent, CSkill* pSkill )
{
    int nBase = GetSkillNumber( pSkill->m_strBase );

    return( nPercent == nBase );
}



/*****************************************************************/




CSkill* FindSkill( char* pszSkill )
{
    CSkill* pSkill;

    pSkill = g_CoreSkills.FindSkill( pszSkill );
    if( pSkill ) return pSkill;

    pSkill = g_Firearms.FindSkill( pszSkill );
    if( pSkill ) return pSkill;

    pSkill = g_Melee.FindSkill( pszSkill );
    if( pSkill ) return pSkill;

    return NULL;
}


int LoadError( const char* pszFilename, const char* pszErrorBit, const char* pszErrorMessage )
{
	if( g_fError == FALSE )
	{
		CString str;
		str.Format( "Error in %s\n\'%s\' - %s", pszFilename, pszErrorBit, pszErrorMessage );
		AfxMessageBox( str, MB_ICONEXCLAMATION );

		g_fError = TRUE;
	}

	return 0;
}

char* LoadBuffer( const char* pszFilename, const char* pszSection )
{
    //open the file and load it into a nice big buffer
    CFile File;
	if( !File.Open( pszFilename, CFile::modeRead) ) 
    {
        LoadError( "", pszFilename, "Cannot open file" );
        return NULL;
    }

	unsigned int nLength = File.GetLength();
	if( nLength == 0 ) return NULL;

	char* pszBuffer = new char[nLength+1];

	if( File.Read( pszBuffer, nLength ) < nLength ) { delete[] pszBuffer; LoadError( "", pszFilename, "Unexpected end of file" ); return NULL; }
	pszBuffer[nLength] = '\0';
	File.Close();

    //remove spaces from the section name
    char szSection[128];
    strcpy( szSection, pszSection );
    for( unsigned int iPos = 0; iPos < strlen(szSection); iPos++ ) if( szSection[iPos] == ' ' ) szSection[iPos] = '_';

    //zip through the buffer and find the start of the required section
    char* pszStart = pszBuffer;
    char szLineBuffer[1024]; //10K max line size (should be *more* than enough...)
    BOOL fFound = FALSE, fEndOfBuffer = FALSE;

    while( !fFound && !fEndOfBuffer )
    {
        //read a line
    	istrstream buf( pszStart, 0 );
		buf.getline( szLineBuffer, 1023 ); buf >> ws; 

        //check it
        char szWord1[1024], szWord2[1024];
        sscanf( szLineBuffer, "%s%s", szWord1, szWord2 );

        if( stricmp( szWord1, "_BEGIN" ) == 0 && stricmp( szWord2, szSection ) == 0 )
            fFound = TRUE;

        //skip over the line, *even* if we've found it (skill etc. parsers can't handle a _BEGIN etc line)
        pszStart = strchr( pszStart, '\n' );
        if( pszStart == NULL ) fEndOfBuffer = TRUE; else pszStart++;
    }

    if( fEndOfBuffer )
    {
        delete pszBuffer;
        //don't display an error if it's not found
        //LoadError( pszFilename, szSection, "_BEGIN not found for this section!" );
        return NULL;
    }

    //now find the end of the required section
    fEndOfBuffer = fFound = FALSE;
    char* pszEnd = pszStart;

    while( !fFound && !fEndOfBuffer )
    {
        //read a line
    	istrstream buf( pszEnd, 0 );
		buf.getline( szLineBuffer, 1023 ); buf >> ws; 

        //check it
        char szWord1[1024], szWord2[1024];
        sscanf( szLineBuffer, "%s%s", szWord1, szWord2 );

        if( stricmp( szWord1, "_END" ) == 0 && stricmp( szWord2, szSection ) == 0 )
            fFound = TRUE;
        else
        {
            pszEnd = strchr( pszEnd, '\n' );
            if( pszEnd == NULL ) fEndOfBuffer = TRUE; else pszEnd++;
        }
    }

    if( fEndOfBuffer )
    {
        delete[] pszBuffer;
        LoadError( pszFilename, szSection, "_END not found for this section!" );
        return NULL;
    }

    //now copy everything into the buffer
    *pszEnd = '\0';

    char* pszSectionBuffer = new char[ strlen(pszStart) + 1 ];
    strcpy( pszSectionBuffer, pszStart );

    delete[] pszBuffer;
	return pszSectionBuffer;
}


/* WARNING: the sscanf()s in next three functions can only be described as "Only the liveliest awfulness" */
BOOL LoadSkills( const char* pszFilename, CSkillList& SkillList )
{
	//load the skills list file
    CString strFilename = GetDefaultDirectory();
    strFilename += pszFilename;
	char* pszBuffer = LoadBuffer( strFilename, "SKILLS" );
	if( pszBuffer == NULL ) { if( g_fError == FALSE ) LoadError( pszFilename, "SKILLS", "section not found" ); return FALSE; }

	//read in the skills list
	istrstream buf( pszBuffer, 0 );
	char szLineBuffer[1024], szBase[32], szSkillName[128], szExtra[1024], cEra1890, cEra1920, cEra1990;
	int nExtra;

	buf >> ws; 
	while( buf.eof() == FALSE )
	{
		//there is no whitespace at the start of the line...
		buf.getline( szLineBuffer, 1023 ); buf >> ws; 
		if( szLineBuffer[0] == szLineBuffer[1] && szLineBuffer[1] == '/' ) continue; //comment

		if( 6 != sscanf( szLineBuffer, "\"%[^\"]\" %d \"%[^\"]\" %c%c%c %[^]", szBase, &nExtra, szSkillName, &cEra1890, &cEra1920, &cEra1990, szExtra ) )
		{ 
			LoadError( pszFilename, szLineBuffer, "Syntax Error" );
			break;
		}

		//extract info
        if( nExtra )
        {
            CSpecialiseSkill* pSpecial = new CSpecialiseSkill( nExtra, szSkillName, szBase, ( cEra1890 != '-' ), ( cEra1920 != '-' ), ( cEra1990 != '-' ) );
            SkillList.AddSkill( pSpecial );

            //add all subskills too
            for( int i = 0; i < pSpecial->m_nSkills; i++ )
                SkillList.AddSkill( pSpecial->m_pSkillSpecialities[i] );
        }
        else
            SkillList.AddSkill( new CSkill( szSkillName, szBase, ( cEra1890 != '-' ), ( cEra1920 != '-' ), ( cEra1990 != '-' ) ) );
	}

	delete[] pszBuffer;
	return TRUE;
}

BOOL LoadFirearms( const char* pszFilename, CSkillList& SkillList )
{
	//load the firearms list file
    CString strFilename = GetDefaultDirectory();
    strFilename += pszFilename;
	char* pszBuffer = LoadBuffer( strFilename, "FIREARMS" );
	if( pszBuffer == NULL ) { if( g_fError == FALSE ) LoadError( pszFilename, "FIREARMS", "section not found" ); return FALSE; }

	//read in the firearms list
	istrstream buf( pszBuffer, 0 );
	char szLineBuffer[1024], szBase[32], szDmg[32], szRange[32], szShots[32], szBullets[32], szHP[16], szMAL[16], szFirearmName[128], szExtra[1024], cEra1890, cEra1920, cEra1990;

	buf >> ws; 
	while( buf.eof() == FALSE )
	{
		//there is no whitespace at the start of the line...
		buf.getline( szLineBuffer, 1023 ); buf >> ws; 
		if( szLineBuffer[0] == szLineBuffer[1] && szLineBuffer[1] == '/' ) continue; //comment

		if( 11 != sscanf( szLineBuffer, "\"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" %c%c%c %[^]", szBase,szDmg, szRange, szShots, szBullets, szHP, szMAL, szFirearmName, &cEra1890, &cEra1920, &cEra1990, szExtra ) )
		{ 
			LoadError( pszFilename, szLineBuffer, "Syntax Error" );
			break;
		}

		//extract info
        SkillList.AddSkill( new CFirearmSkill( /*( *szFirearmName == '*' ) ? &szFirearmName[1] :*/ szFirearmName, szBase, ( cEra1890 != '-' ), ( cEra1920 != '-' ), ( cEra1990 != '-' ), szDmg, szRange, szShots, szBullets, szHP, szMAL ) );
	}

	delete[] pszBuffer;
	return TRUE;
}


BOOL LoadMelee( const char* pszFilename, CSkillList& SkillList )
{
	//load the skills list file
    CString strFilename = GetDefaultDirectory();
    strFilename += pszFilename;
	char* pszBuffer = LoadBuffer( strFilename, "MELEE" );
	if( pszBuffer == NULL ) { if( g_fError == FALSE ) LoadError( pszFilename, "MELEE", "section not found" ); return FALSE; }

	//read in the melee weapons list
	istrstream buf( pszBuffer, 0 );
	char szLineBuffer[1024], szBase[32], szDmg[32], szAttacks[32], szHP[32], szWeaponName[128], szExtra[1024], cEra1890, cEra1920, cEra1990;

	buf >> ws; 
	while( buf.eof() == FALSE )
	{
		//there is no whitespace at the start of the line...
		buf.getline( szLineBuffer, 1023 ); buf >> ws; 
		if( szLineBuffer[0] == szLineBuffer[1] && szLineBuffer[1] == '/' ) continue; //comment

		if( 8 != sscanf( szLineBuffer, "\"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" \"%[^\"]\" %c%c%c %[^]", szBase, szDmg, szAttacks, szHP, szWeaponName, &cEra1890, &cEra1920, &cEra1990, szExtra ) )
		{ 
			LoadError( pszFilename, szLineBuffer, "Syntax Error" );
			break;
		}

		//extract info
        SkillList.AddSkill( new CMeleeSkill( szWeaponName, szBase, ( cEra1890 != '-' ), ( cEra1920 != '-' ), ( cEra1990 != '-' ), szDmg, szAttacks, szHP ) );
	}

	delete[] pszBuffer;
	return TRUE;
}

char* Skip( char* pszBuf, char skipOver )
{
	while( pszBuf && *pszBuf != '\0' && *pszBuf != skipOver ) pszBuf++;
	if( *pszBuf == skipOver ) pszBuf++;

	return pszBuf;
}

/* MORE WARNING: This function was not meant for human viewing - to see it is to die */
COccupation* DecodeOccupation( char* pszBuffer )
{
//FORMAT:
// "skill name"		( [skills] ) xxx
// 
// [skills] format:
//  
//  "skill"
//	*			(any skill)
//	n { "skill" "skill" ... }      n=number to pick from subset
//	
//eg: (cut down to fit on one line)
//	"Tribal Warrior" (	1 {	"Large Club"	"Small Club" }	"Track"	"Spot Hidden"	*	)
//
//
// FIXME:
// also need to support "SpecialSkill$Subitem" - eg:
//                                  "Other Language$Latin"   (FIXME!)


	//get occupation name and entire skills section, except the enclosing ( and )
	char szOccupationName[128];
	char szSkills[2048];
    char cEra1890 = 'x', cEra1920 = 'x', cEra1990 = 'x';

	sscanf( pszBuffer, "\"%[^\"]\" (%[^)]) %c%c%c", szOccupationName, szSkills, &cEra1890, &cEra1920, &cEra1990 );

	char* pszSkillString = szSkills;
	while( pszSkillString && iswspace(*pszSkillString) ) pszSkillString++;

    COccupation* pOccupation = new COccupation;
    pOccupation->m_strName = szOccupationName;
    pOccupation->m_f1890 = ( cEra1890 != '-' );
    pOccupation->m_f1920 = ( cEra1920 != '-' );
    pOccupation->m_f1990 = ( cEra1990 != '-' );

	while( *pszSkillString != '\0' && g_fError == FALSE )
	{
		switch( *pszSkillString )
		{
			case '*':	//wildcard
                pOccupation->AddSkill( new CSkill );
				pszSkillString++;
				break;

			case '"':	//normal skill
			{
				//get the skill name
				char szSkill[128];
				sscanf( pszSkillString, "\"%[^\"]\"", szSkill );

				//advance the stream past this skill
				pszSkillString = Skip( ++pszSkillString, '\"' );

				//look up the skill
				CSkill* pSkill = FindSkill( szSkill );
				if( pSkill )
                    pOccupation->AddSkill( pSkill );
				else
				{
					CString str;
					str.Format( "Error loading occupations\n\'%s' : '%s' - Skill not found", szOccupationName, szSkill );
					AfxMessageBox( str, MB_ICONEXCLAMATION );
					g_fError = TRUE;
				}
				break;
			}

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':	//numerical subset
			{
				if( pOccupation->GetChoiceCount() != 0 )
				{
					CString str;
					str.Format( "Error loading occupations\n\'%s' - More than one subset", szOccupationName );
					AfxMessageBox( str, MB_ICONEXCLAMATION );
					g_fError = TRUE;
				}
				else
				{
                    int nChoice;

					//read the number of subset occupations and their names
					char szSubset[2048];
					/*{*/ sscanf( pszSkillString, "%d {%[^}]}", &nChoice, szSubset ); //note: comments round curly braces allow the 'find matching brace' feature on MSDEV to keep working...

					//advance the stream past the subset
					/*{*/ pszSkillString = Skip( ++pszSkillString, '}' );

					//read in each skill
					char* pszSubset = szSubset;
					while( pszSubset && iswspace(*pszSubset) ) pszSubset++;

                    pOccupation->SetChoiceCount( nChoice );
					while( *pszSubset != '\0' && g_fError == FALSE )
					{
						//get the skill name
						char szSkill[128];
						sscanf( pszSubset, "\"%[^\"]\"", szSkill );

						//advance the stream past this skill
						pszSubset = Skip( ++pszSubset, '\"' );

						//look up the skill
						CSkill* pSkill = FindSkill( szSkill );
						if( pSkill )
                            pOccupation->AddChoice( pSkill );
						else
						{
							CString str;
							str.Format( "Error loading occupations\n\'%s' - Skill not found", szSkill );
							AfxMessageBox( str, MB_ICONEXCLAMATION );
							g_fError = TRUE;
						}

						//remove whitespace
						while( pszSubset && iswspace(*pszSubset) ) pszSubset++;
					}
				}
				break;
			}

			default:
			{
				CString str;
				str.Format( "Error loading occupations\n\'%s' : '%c' - Syntax Error", szOccupationName, *pszSkillString );
				AfxMessageBox( str, MB_ICONEXCLAMATION );
				g_fError = TRUE;
				break;
			}
			
		}

		//remove whitespace
		while( pszSkillString && iswspace(*pszSkillString) ) pszSkillString++;
	}

	if( g_fError == FALSE )
		return pOccupation;
	else
	{
        delete pOccupation;
		return NULL;
	}
}

BOOL LoadOccupations( const char* pszFilename, COccupationList& Occupations )
{
	//load the skills list file
    CString strFilename = GetDefaultDirectory();
    strFilename += pszFilename;
	char* pszBuffer = LoadBuffer( strFilename, "OCCUPATIONS" );
    if( pszBuffer == NULL ) { if( g_fError == FALSE ) LoadError( pszFilename, "OCCUPATIONS", "section not found" ); return FALSE; }

	//read in the skills list
	istrstream buf( pszBuffer, 0 );

	char szLineBuffer[2048];
	buf >> ws; 
	while( buf.eof() == FALSE )
	{
		//there is no whitespace at the start of the line...
		buf.getline( szLineBuffer, 2047 ); buf >> ws; 
		if( szLineBuffer[0] == szLineBuffer[1] && szLineBuffer[1] == '/' ) continue; //comment

		//extract info
        COccupation* pOccupation = DecodeOccupation( szLineBuffer );
        if( pOccupation )
        {
            if( Occupations.AddOccupation( pOccupation ) == FALSE )
            {
                LoadError( pszFilename, pOccupation->m_strName, "Duplicate Occupation" );
                delete pOccupation;
                break;
            }
        }
		else
		{
			LoadError( pszFilename, szLineBuffer, "Syntax Error" );
			break;
		}

	}

	delete[] pszBuffer;
	return TRUE;
}


void LoadEverything()
{
    CSplashWnd::DisplayStatus( "Loading Settings..." );
    CString strDatafile = AfxGetApp()->GetProfileString( "Data", "Rules", "coc5.rul" );
    if( strDatafile.GetLength() == 0 ) strDatafile = "coc5.rul";

    //load all skills
    CSplashWnd::DisplayStatus( "Loading Skills..." );
    LoadSkills( LPCSTR(strDatafile), g_CoreSkills );
    
    CSplashWnd::DisplayStatus( "Loading Firearms..." );
    LoadFirearms( LPCSTR(strDatafile), g_Firearms );

    CSplashWnd::DisplayStatus( "Loading Melee Weapons..." );
    LoadMelee( LPCSTR(strDatafile), g_Melee );

    //load occupations
    CSplashWnd::DisplayStatus( "Loading Occupations..." );
    LoadOccupations( LPCSTR(strDatafile), g_Occupations );
}

BOOL LoadDataTables()
{
    BOOL fContinue = FALSE;
    do {
        LoadEverything();

        if( g_fError )
        {
            //display an error message box
            AfxMessageBox( "Unable to load data file - The program cannot continue\nPlease select a new data file", MB_ICONERROR );

            //delete currently loaded stuff
            g_CoreSkills.ResetList();
            g_Firearms.ResetList();
            g_Melee.ResetList();
            g_Occupations.ResetList();

            //display options dialog
            COptions Options;
            Options.SetActivePage(3);
            if( Options.DoModal() == IDOK )
                g_fError = FALSE;
            else 
                fContinue = TRUE;

        } else fContinue = TRUE;
    } while( fContinue == FALSE );
    return !g_fError;
}


/* special escape code expansion / translation functions *******************/

CString GetSkillCode( const char* pszValue )
{
	char* pszCode = (char*)pszValue;
	CString strResult = "";

    BOOL fShowPercent = TRUE;

	while( pszCode && *pszCode != '\0' )
	{
		switch( *pszCode )
		{
			case '^': //'^' == skill stat
			{
				//get the stat on the end of the '^'
				CString strSkill = "";
				pszCode++;
				while( isalpha(*pszCode) && *pszCode != '\0' ) 
				{
					strSkill = strSkill + *pszCode;
					pszCode++;
				}

				if( strResult.GetLength() > 0 ) strResult += "+";
				strResult += strSkill;
                fShowPercent = FALSE;
				break;	
			}

			case '*':  //'*' == Can't modify
				pszCode++; 
				break;

			case '%':
			{
				//read in the skill
				CString strSkill = "";
				pszCode++;
				while( (*pszCode) != '%' && *pszCode != '\0' ) 
				{
					strSkill = strSkill + *pszCode;
					pszCode++;
				}
				pszCode++;

				strResult += strSkill;
				break;	
			}

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
			{
				//rip out the number
				CString strValue = "";
				while( isdigit(*pszCode) && *pszCode != '\0' ) 
				{
					strValue = strValue + *pszCode;
					pszCode++;
				}

				if( strResult.GetLength() > 0 )
				{
					strResult += "x";
					if( strValue != "1" ) strResult += strValue;
                    fShowPercent = FALSE;
				}
				else
                {
                    //slap on a leading 0
                    if( atoi( LPCSTR(strValue)) < 10 ) strValue = "0" + strValue;
                    strResult += strValue;
                }
				break;	
			}

			default:
				strResult += *pszCode;
				pszCode++;
				break;

		}
	}

    //final checks
    if( strResult.Compare("00") == 0 ) fShowPercent = FALSE;
    
    if( fShowPercent ) strResult += "%";

	return strResult;
}


CString GetDBString( int nStr, int nSiz )
{
    if( nStr < 0 || nSiz < 0 ) return ""; //< 0 == don't display it

	CString strDB;
	int nDB = nStr + nSiz;

    if( nDB <= 12 )
        strDB = "-1D6";
    else
        if( nDB <= 16 )
            strDB = "-1D4";
        else
            if( nDB <= 24 )
                strDB = "none";
            else
                if( nDB <= 32 )
                    strDB = "+1D4";
                else
					strDB.Format( "+%dD6", 1 + ( (nDB-25) / 16 )  );

	return strDB;
}


void GetRandomName( CString& strName, CString& strSex )
{
    AfxGetApp()->BeginWaitCursor();

    //load options for this
    CString* pstrMaleNames = GetOptions( "MaleNames" );
    CString* pstrFemaleNames = GetOptions( "FemaleNames" );
    CString* pstrSurnames = GetOptions( "Surnames" );

    if( pstrMaleNames == NULL || pstrFemaleNames == NULL || pstrSurnames == NULL )
    {
        strName = "(Can't Find Options!)";
        if( pstrMaleNames ) delete[] pstrMaleNames;
        if( pstrFemaleNames ) delete[] pstrFemaleNames;
        if( pstrSurnames ) delete[] pstrSurnames;
        AfxGetApp()->EndWaitCursor();
        return;
    }
    

	int nMNames = 0, nFNames = 0, nSurnames = 0;

	for( nMNames = 0; pstrMaleNames[nMNames].GetLength() > 0; nMNames++ );
	for( nFNames = 0; pstrFemaleNames[nFNames].GetLength() > 0; nFNames++ );
	for( nSurnames = 0; pstrSurnames[nSurnames].GetLength() > 0; nSurnames++ );

    if( nMNames == 0 || nFNames == 0 || nSurnames == 0 )
    {
        strName = "(Not Enough Names!)";
        delete[] pstrMaleNames;
        delete[] pstrFemaleNames;
        delete[] pstrSurnames;
        AfxGetApp()->EndWaitCursor();
        return;
    }

    //get first name & sex
	if( rand() % 4 == 0 ) //greater male than female weighting (sexist, I know, but...)
	{
		strSex = "F";
		strName = pstrFemaleNames[ rand() % nFNames ];
	}
	else
	{
		strSex = "M";
		strName = pstrMaleNames[ rand() % nMNames ];
	}

	strName += " ";

    //possibly add something fancy
    if( rand() % 7 == 0 ) //1 in 7 chance of a middle name or double-barrel name
    {
        strName += pstrSurnames[ rand() % nSurnames ];
        if( rand() % 2 == 0 ) strName += "-"; else strName += " ";
    }
    else
        if( rand() % 50 == 0 ) //1 in 50 chance of being a st-wossname
            strName += "St. ";

    //get surname
	strName += pstrSurnames[ rand() % nSurnames ];

    //free resources
    delete[] pstrMaleNames;
    delete[] pstrFemaleNames;
    delete[] pstrSurnames;

    AfxGetApp()->EndWaitCursor();
}


CString* GetOptions( const char* pszSection, const char* pszFile /*"OPTIONS.LST"*/ )
{
    //load the buffer
    char* pszBuffer = LoadBuffer( pszFile, pszSection );
    if( pszBuffer == NULL ) return NULL;

    //count the lines
    int nLines = 0;
    istrstream buf( pszBuffer, 0 );
    while( buf.eof() == FALSE )
    {
        char szLineBuffer[2048];
        buf.getline( szLineBuffer, 2047 ); buf >> ws;
        nLines++;
    }

    
    //allocate a the first dimension of the array
    CString* pArray = new CString[nLines+1]; //1 extra must be "" to indicate the end of the list

    istrstream inbuf( pszBuffer, 0 );
    for( int nCount = 0; nCount < nLines; nCount++ )
    {
        char szLineBuffer[2048];
        inbuf.getline( szLineBuffer, 2047 ); inbuf >> ws;
        pArray[nCount] = szLineBuffer;
        pArray[nCount].TrimLeft();
        pArray[nCount].TrimRight();
    }

    delete[] pszBuffer;
    return pArray;
}

CString Abbreviate( const char* pszText )
{
    //trim *s from front
    while( pszText && *pszText == '*' && *pszText != '\0' ) pszText++;

    //load abbreviations
    CString* pstrAbbreviations = GetOptions( "Abbreviations" );
    if( pstrAbbreviations == NULL ) return pszText;
 
    //find this one in the list
    int nAbrv = 0;
    char szFull[128], szAbrv[128];
    while( pstrAbbreviations[nAbrv].GetLength() > 0 )
    {
        sscanf( LPCSTR(pstrAbbreviations[nAbrv]), "\"%[^\"]\" \"%[^\"]\"", szFull, szAbrv );
        if( strcmpi( szFull, pszText ) == 0 )
        {
            delete[] pstrAbbreviations;
            return szAbrv;
        }
        nAbrv++;
    }

    delete[] pstrAbbreviations;
    return pszText;
}